@@ -488,8 +488,9 @@ void lcCamera::CopyPosition(const lcCamera* camera)
 	mState |= (camera->mState&LC_CAMERA_ORTHO);
 }
 
-void lcCamera::DrawInterface(lcContext* Context) const
+void lcCamera::DrawInterface(lcContext* Context, const lcScene& Scene) const
 {
+	Q_UNUSED(Scene);
 	Context->SetMaterial(LC_MATERIAL_UNLIT_COLOR);
 
 	lcMatrix44 ViewWorldMatrix = lcMatrix44AffineInverse(mWorldView);
@@ -634,6 +635,18 @@ void lcCamera::DrawInterface(lcContext* Context) const
 	}
 }
 
+void lcCamera::RemoveKeyFrames()
+{
+	mPositionKeys.RemoveAll();
+	ChangeKey(mPositionKeys, mPosition, 1, true);
+
+	mTargetPositionKeys.RemoveAll();
+	ChangeKey(mTargetPositionKeys, mTargetPosition, 1, true);
+
+	mUpVectorKeys.RemoveAll();
+	ChangeKey(mUpVectorKeys, mUpVector, 1, true);
+}
+
 void lcCamera::RayTest(lcObjectRayTest& ObjectRayTest) const
 {
 	lcVector3 Min = lcVector3(-LC_CAMERA_POSITION_EDGE, -LC_CAMERA_POSITION_EDGE, -LC_CAMERA_POSITION_EDGE);
@@ -983,6 +996,28 @@ void lcCamera::SetViewpoint(lcViewpoint Viewpoint)
 	UpdatePosition(1);
 }
 
+void lcCamera::SetViewpoint(const lcVector3& Position)
+{
+	mPosition = Position;
+	mTargetPosition = lcVector3(0, 0, 0);
+
+	lcVector3 UpVector(0, 0, 1), FrontVector(Position), SideVector;
+	FrontVector.Normalize();
+	if (fabsf(lcDot(UpVector, FrontVector)) > 0.99f)
+		SideVector = lcVector3(-1, 0, 0);
+	else
+		SideVector = lcCross(FrontVector, UpVector);
+	UpVector = lcCross(SideVector, FrontVector);
+	UpVector.Normalize();
+	mUpVector = UpVector;
+
+	ChangeKey(mPositionKeys, mPosition, 1, false);
+	ChangeKey(mTargetPositionKeys, mTargetPosition, 1, false);
+	ChangeKey(mUpVectorKeys, mUpVector, 1, false);
+
+	UpdatePosition(1);
+}
+
 void lcCamera::SetAngles(float Latitude, float Longitude)
 {
 	mPosition = lcVector3(0, -1, 0);
