@@ -261,7 +261,8 @@ public:
 public:
 	virtual void RayTest(lcObjectRayTest& ObjectRayTest) const override;
 	virtual void BoxTest(lcObjectBoxTest& ObjectBoxTest) const override;
-	virtual void DrawInterface(lcContext* Context) const override;
+	virtual void DrawInterface(lcContext* Context, const lcScene& Scene) const override;
+	virtual void RemoveKeyFrames() override;
 
 	void InsertTime(lcStep Start, lcStep Time);
 	void RemoveTime(lcStep Start, lcStep Time);
@@ -283,6 +284,7 @@ public:
 	void MoveSelected(lcStep Step, bool AddKey, const lcVector3& Distance);
 	void MoveRelative(const lcVector3& Distance, lcStep Step, bool AddKey);
 	void SetViewpoint(lcViewpoint Viewpoint);
+	void SetViewpoint(const lcVector3& Position);
 	void SetAngles(float Latitude, float Longitude);
 
 	char m_strName[81];
