@@ -25,6 +25,12 @@ void lcPreferences::LoadDefaults()
 	mDrawGridLines = lcGetProfileInt(LC_PROFILE_GRID_LINES);
 	mGridLineSpacing = lcGetProfileInt(LC_PROFILE_GRID_LINE_SPACING);
 	mGridLineColor = lcGetProfileInt(LC_PROFILE_GRID_LINE_COLOR);
+	mViewSphereLocation = (lcViewSphereLocation)lcGetProfileInt(LC_PROFILE_VIEW_SPHERE_LOCATION);
+	mViewSphereSize = lcGetProfileInt(LC_PROFILE_VIEW_SPHERE_SIZE);
+	mViewSphereColor = lcGetProfileInt(LC_PROFILE_VIEW_SPHERE_COLOR);
+	mViewSphereTextColor = lcGetProfileInt(LC_PROFILE_VIEW_SPHERE_TEXT_COLOR);
+	mViewSphereHighlightColor = lcGetProfileInt(LC_PROFILE_VIEW_SPHERE_HIGHLIGHT_COLOR);
+	mAutoLoadMostRecent = lcGetProfileInt(LC_PROFILE_AUTOLOAD_MOSTRECENT);
 }
 
 void lcPreferences::SaveDefaults()
@@ -40,6 +46,12 @@ void lcPreferences::SaveDefaults()
 	lcSetProfileInt(LC_PROFILE_GRID_LINES, mDrawGridLines);
 	lcSetProfileInt(LC_PROFILE_GRID_LINE_SPACING, mGridLineSpacing);
 	lcSetProfileInt(LC_PROFILE_GRID_LINE_COLOR, mGridLineColor);
+	lcSetProfileInt(LC_PROFILE_VIEW_SPHERE_LOCATION, (int)mViewSphereLocation);
+	lcSetProfileInt(LC_PROFILE_VIEW_SPHERE_SIZE, mViewSphereSize);
+	lcSetProfileInt(LC_PROFILE_VIEW_SPHERE_COLOR, mViewSphereColor);
+	lcSetProfileInt(LC_PROFILE_VIEW_SPHERE_TEXT_COLOR, mViewSphereTextColor);
+	lcSetProfileInt(LC_PROFILE_VIEW_SPHERE_HIGHLIGHT_COLOR, mViewSphereHighlightColor);
+	lcSetProfileInt(LC_PROFILE_AUTOLOAD_MOSTRECENT, mAutoLoadMostRecent);
 }
 
 lcApplication::lcApplication(int& Argc, char** Argv)
@@ -99,11 +111,11 @@ void lcApplication::SetProject(Project* Project)
 {
 	SaveTabLayout();
 
+	gMainWindow->RemoveAllModelTabs();
+
 	delete mProject;
 	mProject = Project;
 
-	gMainWindow->RemoveAllModelTabs();
-
 	Project->SetActiveModel(0);
 	lcGetPiecesLibrary()->RemoveTemporaryPieces();
 
@@ -181,7 +193,7 @@ bool lcApplication::Initialize(QList<QPair<QString, bool>>& LibraryPaths, bool&
 	int ImageEnd = 0;
 	int PartImagesWidth = -1;
 	int PartImagesHeight = -1;
-	float CameraLatitude, CameraLongitude;
+	float CameraLatitude = 0.0f, CameraLongitude = 0.0f;
 	QString ImageName;
 	QString ModelName;
 	QString CameraName;
@@ -199,6 +211,9 @@ bool lcApplication::Initialize(QList<QPair<QString, bool>>& LibraryPaths, bool&
 	{
 		const QString& Param = Arguments[ArgIdx];
 
+		if (Param.isEmpty())
+			continue;
+
 		if (Param[0] != '-')
 		{
 			ProjectName = Param;
@@ -233,6 +248,23 @@ bool lcApplication::Initialize(QList<QPair<QString, bool>>& LibraryPaths, bool&
 				printf("Not enough parameters for the '%s' argument.\n", Arguments[ArgIdx].toLatin1().constData());
 		};
 
+		auto ParseFloat = [&ArgIdx, &Arguments, NumArguments](float& Value)
+		{
+			if (ArgIdx < NumArguments - 1 && Arguments[ArgIdx + 1][0] != '-')
+			{
+				bool Ok = false;
+				ArgIdx++;
+				int NewValue = Arguments[ArgIdx].toFloat(&Ok);
+
+				if (Ok)
+					Value = NewValue;
+				else
+					printf("Invalid value specified for the '%s' argument.\n", Arguments[ArgIdx - 1].toLatin1().constData());
+			}
+			else
+				printf("Not enough parameters for the '%s' argument.\n", Arguments[ArgIdx].toLatin1().constData());
+		};
+
 		auto ParseVector2 = [&ArgIdx, &Arguments, NumArguments](float& Value1, float& Value2)
 		{
 			if (ArgIdx < NumArguments - 2 && Arguments[ArgIdx + 1][0] != '-' && Arguments[ArgIdx + 2][0] != '-')
@@ -295,6 +327,22 @@ bool lcApplication::Initialize(QList<QPair<QString, bool>>& LibraryPaths, bool&
 			Orthographic = true;
 		else if (Param == QLatin1String("--highlight"))
 			ImageHighlight = true;
+		else if (Param == QLatin1String("--shading"))
+		{
+			QString ShadingString;
+			ParseString(ShadingString, true);
+
+			if (ShadingString == QLatin1String("wireframe"))
+				mPreferences.mShadingMode = LC_SHADING_WIREFRAME;
+			else if (ShadingString == QLatin1String("flat"))
+				mPreferences.mShadingMode = LC_SHADING_FLAT;
+			else if (ShadingString == QLatin1String("default"))
+				mPreferences.mShadingMode = LC_SHADING_DEFAULT_LIGHTS;
+			else if (ShadingString == QLatin1String("full"))
+				mPreferences.mShadingMode = LC_SHADING_FULL;
+		}
+		else if (Param == QLatin1String("--line-width"))
+			ParseFloat(mPreferences.mLineWidth);
 		else if (Param == QLatin1String("-obj") || Param == QLatin1String("--export-wavefront"))
 		{
 			SaveWavefront = true;
@@ -321,6 +369,11 @@ bool lcApplication::Initialize(QList<QPair<QString, bool>>& LibraryPaths, bool&
 			ParseInteger(PartImagesHeight);
 		else if (Param == QLatin1String("-v") || Param == QLatin1String("--version"))
 		{
+#ifdef LC_CONTINUOUS_BUILD
+			printf("LeoCAD Continuous Build " QT_STRINGIFY(LC_CONTINUOUS_BUILD) "\n");
+#else
+			printf("LeoCAD Version " LC_VERSION_TEXT "\n");
+#endif
 			printf("LeoCAD Version " LC_VERSION_TEXT "\n");
 			printf("Compiled " __DATE__ "\n");
 
@@ -343,6 +396,8 @@ bool lcApplication::Initialize(QList<QPair<QString, bool>>& LibraryPaths, bool&
 			printf("  --camera-angles <latitude> <longitude>: Set the camera angles in degrees around the model.\n");
 			printf("  --orthographic: Make the view orthographic.\n");
 			printf("  --highlight: Highlight pieces in the steps they appear.\n");
+			printf("  --shading <wireframe|flat|default|full>: Select shading mode for rendering.\n");
+			printf("  --line-width <width>: Set the with of the edge lines.\n");
 			printf("  -obj, --export-wavefront <outfile.obj>: Export the model to Wavefront OBJ format.\n");
 			printf("  -3ds, --export-3ds <outfile.3ds>: Export the model to 3D Studio 3DS format.\n");
 			printf("  -dae, --export-collada <outfile.dae>: Export the model to COLLADA DAE format.\n");
@@ -371,9 +426,9 @@ bool lcApplication::Initialize(QList<QPair<QString, bool>>& LibraryPaths, bool&
 		QString Message;
 
 		if (mLibrary->LoadBuiltinPieces())
-			Message = tr("LeoCAD could not find a compatible Parts Library so only a small number of parts will be available.\n\nPlease visit http://www.leocad.org for information on how to download and install a library.");
+			Message = tr("LeoCAD could not find a compatible Parts Library so only a small number of parts will be available.\n\nPlease visit https://www.leocad.org for information on how to download and install a library.");
 		else
-			Message = tr("LeoCAD could not load Parts Library.\n\nPlease visit http://www.leocad.org for information on how to download and install a library.");
+			Message = tr("LeoCAD could not load Parts Library.\n\nPlease visit https://www.leocad.org for information on how to download and install a library.");
 
 		if (ShowWindow)
 			QMessageBox::information(gMainWindow, tr("LeoCAD"), Message);
@@ -386,6 +441,9 @@ bool lcApplication::Initialize(QList<QPair<QString, bool>>& LibraryPaths, bool&
 	Project* NewProject = new Project();
 	SetProject(NewProject);
 
+	if (ShowWindow && ProjectName.isEmpty() && lcGetProfileInt(LC_PROFILE_AUTOLOAD_MOSTRECENT))
+		ProjectName = lcGetProfileString(LC_PROFILE_RECENT_FILE1);
+
 	if (!ProjectName.isEmpty() && gMainWindow->OpenProject(ProjectName))
 	{
 		if (!ModelName.isEmpty())
@@ -438,8 +496,15 @@ bool lcApplication::Initialize(QList<QPair<QString, bool>>& LibraryPaths, bool&
 
 		if (SaveImage)
 		{
+			lcModel* ActiveModel;
+
+			if (ModelName.isEmpty())
+				ActiveModel = mProject->GetMainModel();
+			else
+				ActiveModel = mProject->GetActiveModel();
+
 			if (ImageName.isEmpty())
-				ImageName = mProject->GetImageFileName();
+				ImageName = mProject->GetImageFileName(true);
 
 			if (ImageEnd < ImageStart)
 				ImageEnd = ImageStart;
@@ -447,7 +512,7 @@ bool lcApplication::Initialize(QList<QPair<QString, bool>>& LibraryPaths, bool&
 				ImageStart = ImageEnd;
 
 			if ((ImageStart == 0) && (ImageEnd == 0))
-				ImageStart = ImageEnd = mProject->GetActiveModel()->GetCurrentStep();
+				ImageStart = ImageEnd = ActiveModel->GetCurrentStep();
 			else if ((ImageStart == 0) && (ImageEnd != 0))
 				ImageStart = ImageEnd;
 			else if ((ImageStart != 0) && (ImageEnd == 0))
@@ -469,7 +534,7 @@ bool lcApplication::Initialize(QList<QPair<QString, bool>>& LibraryPaths, bool&
 			else
 				Frame = ImageName;
 
-			lcGetActiveModel()->SaveStepImages(Frame, ImageStart != ImageEnd, CameraName == nullptr, ImageHighlight, ImageWidth, ImageHeight, ImageStart, ImageEnd);
+			ActiveModel->SaveStepImages(Frame, ImageStart != ImageEnd, CameraName == nullptr, ImageHighlight, ImageWidth, ImageHeight, ImageStart, ImageEnd);
 		}
 
 		if (SaveWavefront)
@@ -585,10 +650,12 @@ void lcApplication::ShowPreferencesDialog()
 
 	Options.Preferences = mPreferences;
 
-	Options.DefaultAuthor = lcGetProfileString(LC_PROFILE_DEFAULT_AUTHOR_NAME);
 	Options.LibraryPath = lcGetProfileString(LC_PROFILE_PARTS_LIBRARY);
+	Options.MinifigSettingsPath = lcGetProfileString(LC_PROFILE_MINIFIG_SETTINGS);
+	Options.ColorConfigPath = lcGetProfileString(LC_PROFILE_COLOR_CONFIG);
 	Options.POVRayPath = lcGetProfileString(LC_PROFILE_POVRAY_PATH);
 	Options.LGEOPath = lcGetProfileString(LC_PROFILE_POVRAY_LGEO_PATH);
+	Options.DefaultAuthor = lcGetProfileString(LC_PROFILE_DEFAULT_AUTHOR_NAME);
 	Options.CheckForUpdates = lcGetProfileInt(LC_PROFILE_CHECK_UPDATES);
 
 	Options.AASamples = CurrentAASamples;
@@ -609,6 +676,7 @@ void lcApplication::ShowPreferencesDialog()
 		return;
 
 	bool LibraryChanged = Options.LibraryPath != lcGetProfileString(LC_PROFILE_PARTS_LIBRARY);
+	bool ColorsChanged = Options.ColorConfigPath != lcGetProfileString(LC_PROFILE_COLOR_CONFIG);
 	bool AAChanged = CurrentAASamples != Options.AASamples;
 
 	mPreferences = Options.Preferences;
@@ -617,17 +685,15 @@ void lcApplication::ShowPreferencesDialog()
 
 	lcSetProfileString(LC_PROFILE_DEFAULT_AUTHOR_NAME, Options.DefaultAuthor);
 	lcSetProfileString(LC_PROFILE_PARTS_LIBRARY, Options.LibraryPath);
+	lcSetProfileString(LC_PROFILE_COLOR_CONFIG, Options.ColorConfigPath);
+	lcSetProfileString(LC_PROFILE_MINIFIG_SETTINGS, Options.MinifigSettingsPath);
 	lcSetProfileString(LC_PROFILE_POVRAY_PATH, Options.POVRayPath);
 	lcSetProfileString(LC_PROFILE_POVRAY_LGEO_PATH, Options.LGEOPath);
 	lcSetProfileInt(LC_PROFILE_CHECK_UPDATES, Options.CheckForUpdates);
 	lcSetProfileInt(LC_PROFILE_ANTIALIASING_SAMPLES, Options.AASamples);
 
-	if (LibraryChanged && AAChanged)
-		QMessageBox::information(gMainWindow, tr("LeoCAD"), tr("Parts library and Anti-aliasing changes will only take effect the next time you start LeoCAD."));
-	else if (LibraryChanged)
-		QMessageBox::information(gMainWindow, tr("LeoCAD"), tr("Parts library changes will only take effect the next time you start LeoCAD."));
-	else if (AAChanged)
-		QMessageBox::information(gMainWindow, tr("LeoCAD"), tr("Anti-aliasing changes will only take effect the next time you start LeoCAD."));
+	if (LibraryChanged || ColorsChanged || AAChanged)
+		QMessageBox::information(gMainWindow, tr("LeoCAD"), tr("Some changes will only take effect the next time you start LeoCAD."));
 
 	if (Options.CategoriesModified)
 	{
