@@ -14,6 +14,14 @@ enum lcShadingMode
 	LC_NUM_SHADING_MODES
 };
 
+enum class lcViewSphereLocation
+{
+	TOP_LEFT,
+	TOP_RIGHT,
+	BOTTOM_LEFT,
+	BOTTOM_RIGHT
+};
+
 class lcPreferences
 {
 public:
@@ -31,6 +39,12 @@ public:
 	int mGridLineSpacing;
 	quint32 mGridLineColor;
 	bool mFixedAxes;
+	lcViewSphereLocation mViewSphereLocation;
+	int mViewSphereSize;
+	quint32 mViewSphereColor;
+	quint32 mViewSphereTextColor;
+	quint32 mViewSphereHighlightColor;
+	bool mAutoLoadMostRecent;
 };
 
 class lcApplication : public QApplication
