@@ -26,7 +26,7 @@ enum
 
 struct lcColorGroup
 {
-	lcArray<int> Colors;
+	std::vector<int> Colors;
 	QString Name;
 };
 
@@ -34,6 +34,7 @@ enum lcInterfaceColor
 {
 	LC_COLOR_SELECTED,
 	LC_COLOR_FOCUSED,
+	LC_COLOR_DISABLED,
 	LC_COLOR_CAMERA,
 	LC_COLOR_LIGHT,
 	LC_COLOR_CONTROL_POINT,
@@ -43,7 +44,7 @@ enum lcInterfaceColor
 };
 
 extern lcVector4 gInterfaceColors[LC_NUM_INTERFACECOLORS];
-extern lcArray<lcColor> gColorList;
+extern std::vector<lcColor> gColorList;
 extern lcColorGroup gColorGroups[LC_NUM_COLORGROUPS];
 extern int gNumUserColors;
 extern int gEdgeColor;
@@ -71,7 +72,7 @@ inline quint32 lcGetColorCode(int ColorIndex)
 	return gColorList[ColorIndex].Code;
 }
 
-inline bool lcIsColorTranslucent(int ColorIndex)
+inline bool lcIsColorTranslucent(size_t ColorIndex)
 {
 	return gColorList[ColorIndex].Translucent;
 }
