@@ -34,6 +34,8 @@ lcContext::lcContext()
 
 	mTexture2D = 0;
 	mTexture2DMS = 0;
+	mTextureCubeMap = 0;
+	mPolygonOffset = LC_POLYGON_OFFSET_NONE;
 	mLineWidth = 1.0f;
 #ifndef LC_OPENGLES
 	mMatrixMode = GL_MODELVIEW;
@@ -47,11 +49,16 @@ lcContext::lcContext()
 	mViewMatrix = lcMatrix44Identity();
 	mProjectionMatrix = lcMatrix44Identity();
 	mViewProjectionMatrix = lcMatrix44Identity();
+	mHighlightParams[0] = lcVector4(0.0f, 0.0f, 0.0f, 0.0f);
+	mHighlightParams[1] = lcVector4(0.0f, 0.0f, 0.0f, 0.0f);
+	mHighlightParams[2] = lcVector4(0.0f, 0.0f, 0.0f, 0.0f);
+	mHighlightParams[3] = lcVector4(0.0f, 0.0f, 0.0f, 0.0f);
 	mColorDirty = false;
 	mWorldMatrixDirty = false;
 	mViewMatrixDirty = false;
 	mProjectionMatrixDirty = false;
 	mViewProjectionMatrixDirty = false;
+	mHighlightParamsDirty = false;
 
 	mMaterialType = LC_NUM_MATERIALS;
 }
@@ -62,223 +69,105 @@ lcContext::~lcContext()
 
 void lcContext::CreateShaderPrograms()
 {
+	const char* ShaderPrefix =
+	{
 #ifndef LC_OPENGLES
-#define LC_SHADER_VERSION "#version 110\n#define mediump\n"
-#define LC_VERTEX_INPUT "attribute "
-#define LC_VERTEX_OUTPUT "varying "
-#define LC_PIXEL_INPUT "varying "
-#define LC_PIXEL_OUTPUT
-#define LC_SHADER_PRECISION
+"#version 110\n"
+"#define mediump\n"
+"#define LC_VERTEX_INPUT attribute\n"
+"#define LC_VERTEX_OUTPUT varying\n"
+"#define LC_PIXEL_INPUT varying\n"
+"#define LC_PIXEL_OUTPUT\n"
+"#define LC_SHADER_PRECISION\n"
 #else
-#define LC_SHADER_VERSION "#version 300 es\n#define texture2D texture\n"
-#define LC_VERTEX_INPUT "in "
-#define LC_VERTEX_OUTPUT "out "
-#define LC_PIXEL_INPUT "in mediump "
-#define LC_PIXEL_OUTPUT "#define gl_FragColor FragColor\nout mediump vec4 gl_FragColor;\n"
-#define LC_SHADER_PRECISION "mediump "
+"#version 300 es\n"
+"#define texture2D texture\n"
+"#define LC_VERTEX_INPUT in\n"
+"#define LC_VERTEX_OUTPUT out\n"
+"#define LC_PIXEL_INPUT in mediump\n"
+"#define gl_FragColor FragColor\n"
+"#define LC_PIXEL_OUTPUT out mediump vec4 gl_FragColor;\n"
+"#define LC_SHADER_PRECISION mediump\n"
 #endif
-#define LC_PIXEL_FAKE_LIGHTING \
-	LC_SHADER_PRECISION "	vec3 Normal = normalize(PixelNormal);\n" \
-	LC_SHADER_PRECISION "	vec3 LightDirection = normalize(PixelPosition - LightPosition);" \
-	LC_SHADER_PRECISION "	vec3 VertexToEye = normalize(EyePosition - PixelPosition);\n" \
-	LC_SHADER_PRECISION "	vec3 LightReflect = normalize(reflect(-LightDirection, Normal));\n" \
-	LC_SHADER_PRECISION "	float Specular = abs(dot(VertexToEye, LightReflect));\n" \
-	"	Specular = min(pow(Specular, 8.0), 1.0) * 0.25;\n" \
-	LC_SHADER_PRECISION "	vec3 SpecularColor = vec3(Specular, Specular, Specular);\n" \
-	LC_SHADER_PRECISION "	float Diffuse = min(abs(dot(Normal, LightDirection)) * 0.6 + 0.65, 1.0);\n"
+
+"#define LC_PIXEL_FAKE_LIGHTING \\\n"
+"		LC_SHADER_PRECISION vec3 Normal = normalize(PixelNormal); \\\n"
+"		LC_SHADER_PRECISION vec3 LightDirection = normalize(PixelPosition - LightPosition); \\\n"
+"		LC_SHADER_PRECISION vec3 VertexToEye = normalize(EyePosition - PixelPosition); \\\n"
+"		LC_SHADER_PRECISION vec3 LightReflect = normalize(reflect(-LightDirection, Normal)); \\\n"
+"		LC_SHADER_PRECISION float Specular = abs(dot(VertexToEye, LightReflect)); \\\n"
+"		Specular = min(pow(Specular, 8.0), 1.0) * 0.25; \\\n"
+"		LC_SHADER_PRECISION vec3 SpecularColor = vec3(Specular, Specular, Specular); \\\n"
+"		LC_SHADER_PRECISION float Diffuse = min(abs(dot(Normal, LightDirection)) * 0.6 + 0.65, 1.0);\n"
+	};
 
 	const char* VertexShaders[LC_NUM_MATERIALS] =
 	{
-		// LC_MATERIAL_UNLIT_COLOR
-		LC_SHADER_VERSION
-		LC_VERTEX_INPUT "vec3 VertexPosition;\n"
-		"uniform mat4 WorldViewProjectionMatrix;\n"
-		"void main()\n"
-		"{\n"
-		"	gl_Position = WorldViewProjectionMatrix * vec4(VertexPosition, 1.0);\n"
-		"}\n",
-		// LC_MATERIAL_UNLIT_TEXTURE_MODULATE
-		LC_SHADER_VERSION
-		LC_VERTEX_INPUT "vec3 VertexPosition;\n"
-		LC_VERTEX_INPUT "vec2 VertexTexCoord;\n"
-		LC_VERTEX_OUTPUT "vec2 PixelTexCoord;\n"
-		"uniform mat4 WorldViewProjectionMatrix;\n"
-		"void main()\n"
-		"{\n"
-		"	gl_Position = WorldViewProjectionMatrix * vec4(VertexPosition, 1.0);\n"
-		"	PixelTexCoord = VertexTexCoord;\n"
-		"}\n",
-		// LC_MATERIAL_UNLIT_TEXTURE_DECAL
-		LC_SHADER_VERSION
-		LC_VERTEX_INPUT "vec3 VertexPosition;\n"
-		LC_VERTEX_INPUT "vec2 VertexTexCoord;\n"
-		LC_VERTEX_OUTPUT "vec2 PixelTexCoord;\n"
-		"uniform mat4 WorldViewProjectionMatrix;\n"
-		"void main()\n"
-		"{\n"
-		"	gl_Position = WorldViewProjectionMatrix * vec4(VertexPosition, 1.0);\n"
-		"	PixelTexCoord = VertexTexCoord;\n"
-		"}\n",
-		// LC_MATERIAL_UNLIT_VERTEX_COLOR
-		LC_SHADER_VERSION
-		LC_VERTEX_INPUT "vec3 VertexPosition;\n"
-		LC_VERTEX_INPUT "vec4 VertexColor;\n"
-		LC_VERTEX_OUTPUT "vec4 PixelColor;\n"
-		"uniform mat4 WorldViewProjectionMatrix;\n"
-		"void main()\n"
-		"{\n"
-		"	gl_Position = WorldViewProjectionMatrix * vec4(VertexPosition, 1.0);\n"
-		"	PixelColor = VertexColor;\n"
-		"}\n",
-		// LC_MATERIAL_FAKELIT_COLOR
-		LC_SHADER_VERSION
-		LC_VERTEX_INPUT "vec3 VertexPosition;\n"
-		LC_VERTEX_INPUT "vec3 VertexNormal;\n"
-		LC_VERTEX_OUTPUT "vec3 PixelPosition;\n"
-		LC_VERTEX_OUTPUT "vec3 PixelNormal;\n"
-		"uniform mat4 WorldViewProjectionMatrix;\n"
-		"uniform mat4 WorldMatrix;\n"
-		"void main()\n"
-		"{\n"
-		"	PixelPosition = (WorldMatrix * vec4(VertexPosition, 1.0)).xyz;\n"
-		"   PixelNormal = (WorldMatrix * vec4(VertexNormal, 0.0)).xyz;\n"
-		"	gl_Position = WorldViewProjectionMatrix * vec4(VertexPosition, 1.0);\n"
-		"}\n",
-		// LC_MATERIAL_FAKELIT_TEXTURE_DECAL
-		LC_SHADER_VERSION
-		LC_VERTEX_INPUT "vec3 VertexPosition;\n"
-		LC_VERTEX_INPUT "vec3 VertexNormal;\n"
-		LC_VERTEX_INPUT "vec2 VertexTexCoord;\n"
-		LC_VERTEX_OUTPUT "vec3 PixelPosition;\n"
-		LC_VERTEX_OUTPUT "vec3 PixelNormal;\n"
-		LC_VERTEX_OUTPUT "vec2 PixelTexCoord;\n"
-		"uniform mat4 WorldViewProjectionMatrix;\n"
-		"uniform mat4 WorldMatrix;\n"
-		"void main()\n"
-		"{\n"
-		"	PixelPosition = (WorldMatrix * vec4(VertexPosition, 1.0)).xyz;\n"
-		"   PixelNormal = (WorldMatrix * vec4(VertexNormal, 0.0)).xyz;\n"
-		"	gl_Position = WorldViewProjectionMatrix * vec4(VertexPosition, 1.0);\n"
-		"	PixelTexCoord = VertexTexCoord;\n"
-		"}\n"
+		":/resources/shaders/unlit_color_vs.glsl",            // LC_MATERIAL_UNLIT_COLOR
+		":/resources/shaders/unlit_texture_modulate_vs.glsl", // LC_MATERIAL_UNLIT_TEXTURE_MODULATE
+		":/resources/shaders/unlit_texture_decal_vs.glsl",    // LC_MATERIAL_UNLIT_TEXTURE_DECAL
+		":/resources/shaders/unlit_vertex_color_vs.glsl",     // LC_MATERIAL_UNLIT_VERTEX_COLOR
+		":/resources/shaders/unlit_view_sphere_vs.glsl",      // LC_MATERIAL_UNLIT_VIEW_SPHERE
+		":/resources/shaders/fakelit_color_vs.glsl",          // LC_MATERIAL_FAKELIT_COLOR
+		":/resources/shaders/fakelit_texture_decal_vs.glsl"   // LC_MATERIAL_FAKELIT_TEXTURE_DECAL
 	};
 
 	const char* FragmentShaders[LC_NUM_MATERIALS] =
 	{
-		// LC_MATERIAL_UNLIT_COLOR
-		LC_SHADER_VERSION
-		LC_PIXEL_OUTPUT
-		"uniform mediump vec4 MaterialColor;\n"
-		"void main()\n"
-		"{\n"
-		"	gl_FragColor = MaterialColor;\n"
-		"}\n",
-		// LC_MATERIAL_UNLIT_TEXTURE_MODULATE
-		LC_SHADER_VERSION
-		LC_PIXEL_INPUT "vec2 PixelTexCoord;\n"
-		LC_PIXEL_OUTPUT
-		"uniform mediump vec4 MaterialColor;\n"
-		"uniform sampler2D Texture;\n"
-		"void main()\n"
-		"{\n"
-		LC_SHADER_PRECISION "	vec4 TexelColor = texture2D(Texture, PixelTexCoord);"
-		"	gl_FragColor = vec4(MaterialColor.rgb, TexelColor.a * MaterialColor.a);\n"
-		"}\n",
-		// LC_MATERIAL_UNLIT_TEXTURE_DECAL
-		LC_SHADER_VERSION
-		LC_PIXEL_INPUT "vec2 PixelTexCoord;\n"
-		LC_PIXEL_OUTPUT
-		"uniform mediump vec4 MaterialColor;\n"
-		"uniform sampler2D Texture;\n"
-		"void main()\n"
-		"{\n"
-		LC_SHADER_PRECISION "	vec4 TexelColor = texture2D(Texture, PixelTexCoord);"
-		"	gl_FragColor = mix(MaterialColor, TexelColor, TexelColor.a);\n"
-		"}\n",
-		// LC_MATERIAL_UNLIT_VERTEX_COLOR
-		LC_SHADER_VERSION
-		LC_PIXEL_INPUT "vec4 PixelColor;\n"
-		LC_PIXEL_OUTPUT
-		"void main()\n"
-		"{\n"
-		"	gl_FragColor = PixelColor;\n"
-		"}\n",
-		// LC_MATERIAL_FAKELIT_COLOR
-		LC_SHADER_VERSION
-		LC_PIXEL_INPUT "vec3 PixelPosition;\n"
-		LC_PIXEL_INPUT "vec3 PixelNormal;\n"
-		LC_PIXEL_OUTPUT
-		"uniform mediump vec4 MaterialColor;\n"
-		"uniform mediump vec3 LightPosition;\n"
-		"uniform mediump vec3 EyePosition;\n"
-		"void main()\n"
-		"{\n"
-		LC_PIXEL_FAKE_LIGHTING
-		LC_SHADER_PRECISION "	vec3 DiffuseColor = MaterialColor.rgb * Diffuse;\n"
-		"	gl_FragColor = vec4(DiffuseColor + SpecularColor, MaterialColor.a);\n"
-		"}\n",
-		// LC_MATERIAL_FAKELIT_TEXTURE_DECAL
-		LC_SHADER_VERSION
-		LC_PIXEL_INPUT "vec3 PixelPosition;\n"
-		LC_PIXEL_INPUT "vec3 PixelNormal;\n"
-		LC_PIXEL_INPUT "vec2 PixelTexCoord;\n"
-		LC_PIXEL_OUTPUT
-		"uniform mediump vec4 MaterialColor;\n"
-		"uniform mediump vec3 LightPosition;\n"
-		"uniform mediump vec3 EyePosition;\n"
-		"uniform sampler2D Texture;\n"
-		"void main()\n"
-		"{\n"
-		LC_PIXEL_FAKE_LIGHTING
-		LC_SHADER_PRECISION "	vec4 TexelColor = texture2D(Texture, PixelTexCoord);"
-		LC_SHADER_PRECISION "	vec4 DiffuseColor = mix(MaterialColor, TexelColor, TexelColor.a);\n"
-		"	gl_FragColor = vec4(vec3(DiffuseColor) * Diffuse + SpecularColor, DiffuseColor.a);\n"
-		"}\n"
+		":/resources/shaders/unlit_color_ps.glsl",            // LC_MATERIAL_UNLIT_COLOR
+		":/resources/shaders/unlit_texture_modulate_ps.glsl", // LC_MATERIAL_UNLIT_TEXTURE_MODULATE
+		":/resources/shaders/unlit_texture_decal_ps.glsl",    // LC_MATERIAL_UNLIT_TEXTURE_DECAL
+		":/resources/shaders/unlit_vertex_color_ps.glsl",     // LC_MATERIAL_UNLIT_VERTEX_COLOR
+		":/resources/shaders/unlit_view_sphere_ps.glsl",      // LC_MATERIAL_UNLIT_VIEW_SPHERE
+		":/resources/shaders/fakelit_color_ps.glsl",          // LC_MATERIAL_FAKELIT_COLOR
+		":/resources/shaders/fakelit_texture_decal_ps.glsl"   // LC_MATERIAL_FAKELIT_TEXTURE_DECAL
+	};
+
+	auto LoadShader = [ShaderPrefix](const char* FileName, GLuint ShaderType) -> GLuint
+	{
+		QResource Resource(FileName);
+
+		if (!Resource.isValid())
+			return 0;
+
+		QByteArray Data;
+
+		if (Resource.isCompressed())
+			Data = qUncompress(Resource.data(), Resource.size());
+		else
+			Data = QByteArray::fromRawData((const char*)Resource.data(), Resource.size());
+
+		Data = ShaderPrefix + Data;
+		const char* Source = Data.constData();
+
+		GLuint Shader = glCreateShader(ShaderType);
+		glShaderSource(Shader, 1, &Source, nullptr);
+		glCompileShader(Shader);
+
+#ifndef QT_NO_DEBUG
+		GLint ShaderCompiled = 0;
+		glGetShaderiv(Shader, GL_COMPILE_STATUS, &ShaderCompiled);
+
+		if (ShaderCompiled == GL_FALSE)
+		{
+			GLint Length = 0;
+			glGetShaderiv(Shader, GL_INFO_LOG_LENGTH, &Length);
+
+			QByteArray InfoLog;
+			InfoLog.resize(Length);
+			glGetShaderInfoLog(Shader, Length, &Length, InfoLog.data());
+
+			qDebug() << InfoLog;
+		}
+#endif
+
+		return Shader;
 	};
 
 	for (int MaterialType = 0; MaterialType < LC_NUM_MATERIALS; MaterialType++)
 	{
-		GLuint VertexShader = glCreateShader(GL_VERTEX_SHADER);
-		glShaderSource(VertexShader, 1, &VertexShaders[MaterialType], nullptr);
-		glCompileShader(VertexShader);
-
-#ifndef QT_NO_DEBUG
-		GLint VertexShaderCompiled = 0;
-		glGetShaderiv(VertexShader, GL_COMPILE_STATUS, &VertexShaderCompiled);
-
-		if (VertexShaderCompiled == GL_FALSE)
-		{
-			GLint Length = 0;
-			glGetShaderiv(VertexShader, GL_INFO_LOG_LENGTH, &Length);
-
-			QByteArray InfoLog;
-			InfoLog.resize(Length);
-			glGetShaderInfoLog(VertexShader, Length, &Length, InfoLog.data());
-
-			qDebug() << InfoLog;
-		}
-#endif
-
-		GLuint FragmentShader = glCreateShader(GL_FRAGMENT_SHADER);
-		glShaderSource(FragmentShader, 1, &FragmentShaders[MaterialType], nullptr);
-		glCompileShader(FragmentShader);
-
-#ifndef QT_NO_DEBUG
-		GLint FragmentShaderCompiled = 0;
-		glGetShaderiv(FragmentShader, GL_COMPILE_STATUS, &FragmentShaderCompiled);
-
-		if (FragmentShaderCompiled == GL_FALSE)
-		{
-			GLint Length = 0;
-			glGetShaderiv(FragmentShader, GL_INFO_LOG_LENGTH, &Length);
-
-			QByteArray InfoLog;
-			InfoLog.resize(Length);
-			glGetShaderInfoLog(FragmentShader, Length, &Length, InfoLog.data());
-
-			qDebug() << InfoLog;
-		}
-#endif
+		GLuint VertexShader = LoadShader(VertexShaders[MaterialType], GL_VERTEX_SHADER);
+		GLuint FragmentShader = LoadShader(FragmentShaders[MaterialType], GL_FRAGMENT_SHADER);
 
 		GLuint Program = glCreateProgram();
 
@@ -299,6 +188,7 @@ void lcContext::CreateShaderPrograms()
 
 		GLint IsLinked = 0;
 		glGetProgramiv(Program, GL_LINK_STATUS, &IsLinked);
+
 		if (IsLinked == GL_FALSE)
 		{
 			GLint Length = 0;
@@ -318,6 +208,7 @@ void lcContext::CreateShaderPrograms()
 		mPrograms[MaterialType].MaterialColorLocation = glGetUniformLocation(Program, "MaterialColor");
 		mPrograms[MaterialType].LightPositionLocation = glGetUniformLocation(Program, "LightPosition");
 		mPrograms[MaterialType].EyePositionLocation = glGetUniformLocation(Program, "EyePosition");
+		mPrograms[MaterialType].HighlightParamsLocation = glGetUniformLocation(Program, "HighlightParams");
 	}
 }
 
@@ -346,10 +237,10 @@ void lcContext::SetDefaultState()
 	glEnable(GL_DEPTH_TEST);
 	glDepthFunc(GL_LEQUAL);
 
-	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-
-	glEnable(GL_POLYGON_OFFSET_FILL);
-	glPolygonOffset(0.5f, 0.1f);
+	if (gSupportsBlendFuncSeparate)
+		glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_ONE_MINUS_DST_ALPHA, GL_ONE);
+	else
+		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
 	if (gSupportsVertexBufferObject)
 	{
@@ -398,6 +289,11 @@ void lcContext::SetDefaultState()
 		mTexture2DMS = 0;
 	}
 #endif
+	glBindTexture(GL_TEXTURE_CUBE_MAP, 0);
+	mTextureCubeMap = 0;
+
+	glDisable(GL_POLYGON_OFFSET_FILL);
+	mPolygonOffset = LC_POLYGON_OFFSET_NONE;
 
 	glLineWidth(1.0f);
 	mLineWidth = 1.0f;
@@ -441,6 +337,7 @@ void lcContext::SetMaterial(lcMaterialType MaterialType)
 		mColorDirty = true;
 		mWorldMatrixDirty = true; // todo: change dirty to a bitfield and set the lighting constants dirty here
 		mViewMatrixDirty = true;
+		mHighlightParamsDirty = true;
 	}
 	else
 	{
@@ -478,6 +375,7 @@ void lcContext::SetMaterial(lcMaterialType MaterialType)
 			}
 			break;
 
+		case LC_MATERIAL_UNLIT_VIEW_SPHERE:
 		case LC_NUM_MATERIALS:
 			break;
 		}
@@ -490,6 +388,31 @@ void lcContext::SetViewport(int x, int y, int Width, int Height)
 	glViewport(x, y, Width, Height);
 }
 
+void lcContext::SetPolygonOffset(lcPolygonOffset PolygonOffset)
+{
+	if (mPolygonOffset == PolygonOffset)
+		return;
+
+	switch (PolygonOffset)
+	{
+	case LC_POLYGON_OFFSET_NONE:
+		glDisable(GL_POLYGON_OFFSET_FILL);
+		break;
+
+	case LC_POLYGON_OFFSET_OPAQUE:
+		glPolygonOffset(0.5f, 0.1f);
+		glEnable(GL_POLYGON_OFFSET_FILL);
+		break;
+
+	case LC_POLYGON_OFFSET_TRANSLUCENT:
+		glPolygonOffset(0.25f, 0.1f);
+		glEnable(GL_POLYGON_OFFSET_FILL);
+		break;
+	}
+
+	mPolygonOffset = PolygonOffset;
+}
+
 void lcContext::SetLineWidth(float LineWidth)
 {
 	if (LineWidth == mLineWidth)
@@ -507,6 +430,20 @@ void lcContext::SetSmoothShading(bool Smooth)
 #endif
 }
 
+void lcContext::BeginTranslucent()
+{
+	glEnable(GL_BLEND);
+	glDepthMask(GL_FALSE);
+	SetPolygonOffset(LC_POLYGON_OFFSET_TRANSLUCENT);
+}
+
+void lcContext::EndTranslucent()
+{
+	glDepthMask(GL_TRUE);
+	glDisable(GL_BLEND);
+	SetPolygonOffset(LC_POLYGON_OFFSET_OPAQUE);
+}
+
 void lcContext::BindTexture2D(GLuint Texture)
 {
 	if (mTexture2D == Texture)
@@ -530,6 +467,33 @@ void lcContext::BindTexture2DMS(GLuint Texture)
 #endif
 }
 
+void lcContext::BindTextureCubeMap(GLuint Texture)
+{
+	if (mTextureCubeMap == Texture)
+		return;
+
+	glBindTexture(GL_TEXTURE_CUBE_MAP, Texture);
+	mTextureCubeMap = Texture;
+}
+
+void lcContext::UnbindTexture2D(GLuint Texture)
+{
+	if (mTexture2D != Texture)
+		return;
+
+	glBindTexture(GL_TEXTURE_2D, 0);
+	mTexture2D = 0;
+}
+
+void lcContext::UnbindTextureCubeMap(GLuint Texture)
+{
+	if (mTextureCubeMap != Texture)
+		return;
+
+	glBindTexture(GL_TEXTURE_CUBE_MAP, 0);
+	mTextureCubeMap = 0;
+}
+
 void lcContext::SetColor(float Red, float Green, float Blue, float Alpha)
 {
 	SetColor(lcVector4(Red, Green, Blue, Alpha));
@@ -540,9 +504,10 @@ void lcContext::SetColorIndex(int ColorIndex)
 	SetColor(gColorList[ColorIndex].Value);
 }
 
-void lcContext::SetColorIndexTinted(int ColorIndex, lcInterfaceColor InterfaceColor)
+void lcContext::SetColorIndexTinted(int ColorIndex, lcInterfaceColor InterfaceColor, float Weight)
 {
-	SetColor((gColorList[ColorIndex].Value + gInterfaceColors[InterfaceColor]) * 0.5f);
+	lcVector3 Color(gColorList[ColorIndex].Value * Weight + gInterfaceColors[InterfaceColor] * (1.0f - Weight));
+	SetColor(lcVector4(Color, gColorList[ColorIndex].Value.w));
 }
 
 void lcContext::SetEdgeColorIndex(int ColorIndex)
@@ -576,7 +541,7 @@ lcFramebuffer lcContext::CreateFramebuffer(int Width, int Height, bool Depth, bo
 
 	if (gSupportsFramebufferObjectARB)
 	{
-		int Samples = (Multisample && gSupportsTexImage2DMultisample) ? QGLFormat::defaultFormat().samples() : 1;
+		int Samples = (Multisample && gSupportsTexImage2DMultisample && QGLFormat::defaultFormat().sampleBuffers()) ? QGLFormat::defaultFormat().samples() : 1;
 
 		glGenFramebuffers(1, &Framebuffer.mObject);
 		glBindFramebuffer(GL_FRAMEBUFFER, Framebuffer.mObject);
@@ -697,7 +662,7 @@ void lcContext::BindFramebuffer(GLuint FramebufferObject)
 
 std::pair<lcFramebuffer, lcFramebuffer> lcContext::CreateRenderFramebuffer(int Width, int Height)
 {
-	if (gSupportsFramebufferObjectARB && QGLFormat::defaultFormat().samples() > 1)
+	if (gSupportsFramebufferObjectARB && QGLFormat::defaultFormat().sampleBuffers() && QGLFormat::defaultFormat().samples() > 1)
 		return std::make_pair(CreateFramebuffer(Width, Height, true, true), CreateFramebuffer(Width, Height, false, false));
 	else
 		return std::make_pair(CreateFramebuffer(Width, Height, true, false), lcFramebuffer());
@@ -1252,6 +1217,15 @@ void lcContext::FlushState()
 			glUniform4fv(Program.MaterialColorLocation, 1, mColor);
 			mColorDirty = false;
 		}
+
+		if (mHighlightParamsDirty && Program.HighlightParamsLocation != -1)
+		{
+			lcMatrix44 InverseViewMatrix = lcMatrix44AffineInverse(mViewMatrix);
+			mHighlightParams[4] = InverseViewMatrix[2];
+
+			glUniform4fv(Program.HighlightParamsLocation, 5, mHighlightParams[0]);
+			mHighlightParamsDirty = false;
+		}
 	}
 	else
 	{
