@@ -9,6 +9,19 @@
 #include <QGLWidget>
 #include <QtGui>
 #include <QPrinter>
+#include <map>
+#include <vector>
+#include <array>
+#include <set>
+#include <functional>
+
+#ifndef Q_FALLTHROUGH
+#define Q_FALLTHROUGH();
+#endif
+
+#ifndef QT_STRINGIFY
+#define QT_STRINGIFY(s) ""
+#endif
 
 #if !defined(EGL_VERSION_1_0) && !defined(GL_ES_VERSION_2_0) && !defined(GL_ES_VERSION_3_0) && !defined(QT_OPENGL_ES)
 #undef GL_LINES_ADJACENCY_EXT
@@ -22,6 +35,7 @@
 
 // Old defines and declarations.
 #define LC_MAXPATH 1024
+#define LC_MAXNAME 1000
 
 #ifdef Q_OS_WIN
 char* strcasestr(const char *s, const char *find);
@@ -31,10 +45,10 @@ char* strlwr(char* string);
 #endif
 
 // Version number.
-#define LC_VERSION_MAJOR 18
-#define LC_VERSION_MINOR 02
-#define LC_VERSION_PATCH 0
-#define LC_VERSION_TEXT "18.02"
+#define LC_VERSION_MAJOR 19
+#define LC_VERSION_MINOR 07
+#define LC_VERSION_PATCH 1
+#define LC_VERSION_TEXT "19.07.1"
 
 // Forward declarations.
 class Project;
