@@ -1,5 +1,4 @@
-#ifndef _LC_GLWIDGET_H_
-#define _LC_GLWIDGET_H_
+#pragma once
 
 #include "lc_context.h"
 
@@ -11,7 +10,8 @@ enum LC_CURSOR_TYPE
 	LC_CURSOR_SPOTLIGHT,
 	LC_CURSOR_CAMERA,
 	LC_CURSOR_SELECT,
-	LC_CURSOR_SELECT_GROUP,
+	LC_CURSOR_SELECT_ADD,
+	LC_CURSOR_SELECT_REMOVE,
 	LC_CURSOR_MOVE,
 	LC_CURSOR_ROTATE,
 	LC_CURSOR_ROTATEX,
@@ -94,4 +94,3 @@ public:
 	bool mDeleteContext;
 };
 
-#endif // _LC_GLWIDGET_H_
