@@ -22,19 +22,16 @@ enum lcLibraryFolderType
 	LC_NUM_FOLDERTYPES
 };
 
+#define LC_LIBRARY_VERTEX_UNTEXTURED 0x1
+#define LC_LIBRARY_VERTEX_TEXTURED   0x2
+
 struct lcLibraryMeshVertex
-{
-	lcVector3 Position;
-	lcVector3 Normal;
-	float NormalWeight;
-};
-
-struct lcLibraryMeshVertexTextured
 {
 	lcVector3 Position;
 	lcVector3 Normal;
 	float NormalWeight;
 	lcVector2 TexCoord;
+	quint32 Usage;
 };
 
 class lcLibraryMeshSection
@@ -68,7 +65,35 @@ enum class lcLibraryTextureMapType
 struct lcLibraryTextureMap
 {
 	lcTexture* Texture;
-	lcVector4 Params[4];
+
+	union lcTextureMapParams
+	{
+		lcTextureMapParams()
+		{
+		}
+
+		struct lcTextureMapPlanarParams
+		{
+			lcVector4 Planes[2];
+		} Planar;
+
+		struct lcTextureMapCylindricalParams
+		{
+			lcVector4 FrontPlane;
+			float UpLength;
+			lcVector4 Plane1;
+			lcVector4 Plane2;
+		} Cylindrical;
+
+		struct lcTextureMapSphericalParams
+		{
+			lcVector4 FrontPlane;
+			lcVector3 Center;
+			lcVector4 Plane1;
+			lcVector4 Plane2;
+		} Spherical;
+	} Params;
+
 	float Angle1;
 	float Angle2;
 	lcLibraryTextureMapType Type;
@@ -96,6 +121,8 @@ class lcLibraryMeshData
 public:
 	lcLibraryMeshData()
 	{
+		mHasTextures = false;
+
 		for (int MeshDataIdx = 0; MeshDataIdx < LC_NUM_MESHDATA_TYPES; MeshDataIdx++)
 			mVertices[MeshDataIdx].SetGrow(1024);
 	}
@@ -131,13 +158,14 @@ public:
 
 	lcArray<lcLibraryMeshSection*> mSections[LC_NUM_MESHDATA_TYPES];
 	lcArray<lcLibraryMeshVertex> mVertices[LC_NUM_MESHDATA_TYPES];
-	lcArray<lcLibraryMeshVertexTextured> mTexturedVertices[LC_NUM_MESHDATA_TYPES];
+	bool mHasTextures;
 };
 
 class lcLibraryPrimitive
 {
 public:
-	lcLibraryPrimitive(const char* Name, lcZipFileType ZipFileType,quint32 ZipFileIndex, bool Stud, bool SubFile)
+	explicit lcLibraryPrimitive(QString&& FileName, const char* Name, lcZipFileType ZipFileType,quint32 ZipFileIndex, bool Stud, bool SubFile)
+		: mFileName(std::move(FileName))
 	{
 		strncpy(mName, Name, sizeof(mName));
 		mName[sizeof(mName) - 1] = 0;
@@ -149,13 +177,14 @@ public:
 		mSubFile = SubFile;
 	}
 
-	void SetZipFile(lcZipFileType ZipFileType,quint32 ZipFileIndex)
+	void SetZipFile(lcZipFileType ZipFileType, quint32 ZipFileIndex)
 	{
 		mZipFileType = ZipFileType;
 		mZipFileIndex = ZipFileIndex;
 	}
 
-	char mName[LC_MAXPATH];
+	QString mFileName;
+	char mName[LC_MAXNAME];
 	lcZipFileType mZipFileType;
 	quint32 mZipFileIndex;
 	lcPrimitiveState mState;
@@ -190,7 +219,7 @@ public:
 	bool LoadTexture(lcTexture* Texture);
 	void ReleaseTexture(lcTexture* Texture);
 	void QueueTextureUpload(lcTexture* Texture);
-	void UploadTextures();
+	void UploadTextures(lcContext* Context);
 
 	bool PieceInCategory(PieceInfo* Info, const char* CategoryKeywords) const;
 	void GetCategoryEntries(int CategoryIndex, bool GroupPieces, lcArray<PieceInfo*>& SinglePieces, lcArray<PieceInfo*>& GroupedPieces);
@@ -206,7 +235,7 @@ public:
 	void SetOfficialPieces()
 	{
 		if (mZipFiles[LC_ZIPFILE_OFFICIAL])
-			mNumOfficialPieces = mPieces.size();
+			mNumOfficialPieces = (int)mPieces.size();
 	}
 
 	bool ReadMeshData(lcFile& File, const lcMatrix44& CurrentTransform, quint32 CurrentColorCode, bool InvertWinding, lcArray<lcLibraryTextureMap>& TextureStack, lcLibraryMeshData& MeshData, lcMeshDataType MeshDataType, bool Optimize, Project* CurrentProject, bool SearchProjectFolder);
@@ -219,7 +248,7 @@ public:
 	std::map<std::string, lcLibraryPrimitive*> mPrimitives;
 	int mNumOfficialPieces;
 
-	lcArray<lcTexture*> mTextures;
+	std::vector<lcTexture*> mTextures;
 
 	QDir mLibraryDir;
 
