@@ -22,11 +22,38 @@
 #include "pieceinf.h"
 #include "lc_library.h"
 #include "lc_colors.h"
-#include <functional>
+
+#if LC_ENABLE_GAMEPAD
+#include <QtGamepad/QGamepad>
+#endif
 
 lcMainWindow* gMainWindow;
 #define LC_TAB_LAYOUT_VERSION 0x0001
 
+void lcTabBar::mousePressEvent(QMouseEvent* Event)
+{
+	if (Event->button() == Qt::MidButton)
+		mMousePressTab = tabAt(Event->pos());
+	else
+		QTabBar::mousePressEvent(Event);
+}
+
+void lcTabBar::mouseReleaseEvent(QMouseEvent* Event)
+{
+	if (Event->button() == Qt::MidButton && tabAt(Event->pos()) == mMousePressTab)
+		tabCloseRequested(mMousePressTab);
+	else
+		QTabBar::mouseReleaseEvent(Event);
+}
+
+lcTabWidget::lcTabWidget()
+	: QTabWidget()
+{
+	lcTabBar* TabBar = new lcTabBar(this);
+	setTabBar(TabBar);
+	TabBar->setDrawBase(false);
+}
+
 void lcModelTabWidget::ResetLayout()
 {
 	QLayout* TabLayout = layout();
@@ -50,7 +77,7 @@ void lcModelTabWidget::Clear()
 	ResetLayout();
 	mModel = nullptr;
 	for (View* View : mViews)
-		View->mModel = nullptr;
+		View->Clear();
 	mViews.RemoveAll();
 	mActiveView = nullptr;
 	lcQGLWidget* Widget = (lcQGLWidget*)layout()->itemAt(0)->widget();
@@ -75,12 +102,19 @@ lcMainWindow::lcMainWindow()
 	mRelativeTransform = true;
 	mCurrentPieceInfo = nullptr;
 	mSelectionMode = lcSelectionMode::SINGLE;
+	mModelTabWidget = nullptr;
 
 	memset(&mSearchOptions, 0, sizeof(mSearchOptions));
 
 	for (int FileIdx = 0; FileIdx < LC_MAX_RECENT_FILES; FileIdx++)
 		mRecentFiles[FileIdx] = lcGetProfileString((LC_PROFILE_KEY)(LC_PROFILE_RECENT_FILE1 + FileIdx));
 
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 8, 0))
+	connect(&mGamepadTimer, &QTimer::timeout, this, &lcMainWindow::UpdateGamepads);
+	mLastGamepadUpdate = QDateTime::currentDateTime();
+	mGamepadTimer.start(33);
+#endif
+
 	gMainWindow = this;
 }
 
@@ -281,7 +315,6 @@ void lcMainWindow::CreateActions()
 	mActions[LC_VIEW_TIME_LAST]->setIcon(QIcon(":/resources/time_last.png"));
 	mActions[LC_VIEW_TIME_ADD_KEYS]->setIcon(QIcon(":/resources/time_add_keys.png"));
 	mActions[LC_HELP_HOMEPAGE]->setIcon(QIcon(":/resources/help_homepage.png"));
-	mActions[LC_HELP_EMAIL]->setIcon(QIcon(":/resources/help_email.png"));
 
 	mActions[LC_EDIT_TRANSFORM_RELATIVE]->setCheckable(true);
 	mActions[LC_EDIT_SNAP_MOVE_TOGGLE]->setCheckable(true);
@@ -511,10 +544,13 @@ void lcMainWindow::CreateMenus()
 	PieceMenu->addAction(mActions[LC_PIECE_INSERT]);
 	PieceMenu->addAction(mActions[LC_PIECE_DELETE]);
 	PieceMenu->addAction(mActions[LC_PIECE_DUPLICATE]);
-	PieceMenu->addAction(mActions[LC_PIECE_RESET_PIVOT_POINT]);
 	PieceMenu->addAction(mActions[LC_PIECE_ARRAY]);
 	PieceMenu->addAction(mActions[LC_PIECE_MINIFIG_WIZARD]);
+	PieceMenu->addAction(mActions[LC_PIECE_RESET_PIVOT_POINT]);
+	PieceMenu->addAction(mActions[LC_PIECE_REMOVE_KEY_FRAMES]);
 	PieceMenu->addSeparator();
+	PieceMenu->addAction(mActions[LC_PIECE_EDIT_SELECTED_SUBMODEL]);
+	PieceMenu->addAction(mActions[LC_PIECE_EDIT_END_SUBMODEL]);
 	PieceMenu->addAction(mActions[LC_PIECE_VIEW_SELECTED_MODEL]);
 	PieceMenu->addAction(mActions[LC_PIECE_INLINE_SELECTED_MODELS]);
 	PieceMenu->addAction(mActions[LC_PIECE_MOVE_SELECTION_TO_MODEL]);
@@ -532,18 +568,20 @@ void lcMainWindow::CreateMenus()
 	QMenu* ModelMenu = menuBar()->addMenu(tr("Sub&model"));
 	ModelMenu->addAction(mActions[LC_MODEL_PROPERTIES]);
 	ModelMenu->addAction(mActions[LC_MODEL_NEW]);
+	ModelMenu->addAction(mActions[LC_MODEL_LIST]);
 	ModelMenu->addSeparator();
 	for (int ModelIdx = LC_MODEL_FIRST; ModelIdx <= LC_MODEL_LAST; ModelIdx++)
 		ModelMenu->addAction(mActions[ModelIdx]);
-	ModelMenu->addAction(mActions[LC_MODEL_LIST]);
 
 	QMenu* HelpMenu = menuBar()->addMenu(tr("&Help"));
 	HelpMenu->addAction(mActions[LC_HELP_HOMEPAGE]);
-	HelpMenu->addAction(mActions[LC_HELP_EMAIL]);
+	HelpMenu->addAction(mActions[LC_HELP_BUG_REPORT]);
 #if !LC_DISABLE_UPDATE_CHECK
 	HelpMenu->addAction(mActions[LC_HELP_UPDATES]);
 #endif
+#ifndef Q_OS_MACOS
 	HelpMenu->addSeparator();
+#endif
 	HelpMenu->addAction(mActions[LC_HELP_ABOUT]);
 }
 
@@ -752,7 +790,8 @@ void lcMainWindow::dragEnterEvent(QDragEnterEvent* Event)
 void lcMainWindow::dropEvent(QDropEvent* Event)
 {
 	const QMimeData* MimeData = Event->mimeData();
-	foreach (const QUrl &Url, MimeData->urls())
+	const QList<QUrl> Urls = MimeData->urls();
+	for (const QUrl& Url : Urls)
 		if (OpenProject(Url.toLocalFile()))
 			break;
 }
@@ -773,6 +812,34 @@ QMenu* lcMainWindow::createPopupMenu()
 	return Menu;
 }
 
+void lcMainWindow::UpdateGamepads()
+{
+#if LC_ENABLE_GAMEPAD
+	QDateTime Now = QDateTime::currentDateTime();
+	quint64 Elapsed = mLastGamepadUpdate.msecsTo(Now);
+	mLastGamepadUpdate = Now;
+
+	if (!gMainWindow)
+		return;
+
+	View* ActiveView = GetActiveView();
+	if (!ActiveView)
+		return;
+
+	const QList<int> Gamepads = QGamepadManager::instance()->connectedGamepads();
+	if (Gamepads.isEmpty())
+		return;
+
+	QGamepad Gamepad(Gamepads[0]);
+
+	float Scale = (float)Elapsed / 20.0f;
+	lcVector3 Distance(Scale * Gamepad.axisLeftX(), 0.0f, -Scale * Gamepad.axisLeftY());
+
+	if (fabsf(Distance.LengthSquared()) > 0.01f)
+		ActiveView->MoveCamera(Distance);
+#endif
+}
+
 void lcMainWindow::ModelTabContextMenuRequested(const QPoint& Point)
 {
 	QMenu* Menu = new QMenu;
@@ -785,6 +852,7 @@ void lcMainWindow::ModelTabContextMenuRequested(const QPoint& Point)
 		Menu->addAction(tr("Reset Views"), this, SLOT(ModelTabResetViews()));
 
 	Menu->exec(QCursor::pos());
+	delete Menu;
 }
 
 void lcMainWindow::ModelTabCloseOtherTabs()
@@ -824,7 +892,7 @@ void lcMainWindow::ClipboardChanged()
 	const QMimeData* MimeData = QApplication::clipboard()->mimeData();
 	QByteArray ClipboardData;
 
-	if (MimeData->hasFormat(MimeType))
+	if (MimeData && MimeData->hasFormat(MimeType))
 		ClipboardData = MimeData->data(MimeType);
 
 	gApplication->SetClipboard(ClipboardData);
@@ -858,18 +926,21 @@ void lcMainWindow::ProjectFileChanged(const QString& Path)
 
 	QString Text = tr("The file '%1' has been modified by another application, do you want to reload it?").arg(QDir::toNativeSeparators(Path));
 
+	Project* CurrentProject = lcGetActiveProject();
+
 	Ignore = true;
 
 	if (QMessageBox::question(this, tr("File Changed"), Text, QMessageBox::Yes | QMessageBox::No) != QMessageBox::Yes)
 	{
 		Ignore = false;
+		CurrentProject->MarkAsModified();
+		UpdateTitle();
 		return;
 	}
 
 	Ignore = false;
 
 	QFileInfo FileInfo(Path);
-	Project* CurrentProject = lcGetActiveProject();
 
 	if (FileInfo == QFileInfo(CurrentProject->GetFileName()))
 	{
@@ -882,17 +953,12 @@ void lcMainWindow::ProjectFileChanged(const QString& Path)
 			RestoreTabLayout(TabLayout);
 			UpdateAllViews();
 		}
-		else
-		{
-			QMessageBox::information(this, tr("LeoCAD"), tr("Error loading '%1'.").arg(Path));
-			delete NewProject;
-		}
 	}
 	else
 	{
 		PieceInfo* Info = lcGetPiecesLibrary()->FindPiece(FileInfo.fileName().toLatin1(), CurrentProject, false, true);
 
-		if (Info)
+		if (Info && Info->IsProject())
 			Info->GetProject()->Load(Path);
 	}
 }
@@ -1070,7 +1136,7 @@ void lcMainWindow::Print(QPrinter* Printer)
 
 void lcMainWindow::ShowSearchDialog()
 {
-	lcModel* Model = lcGetActiveModel();
+	lcModel* Model = GetActiveModel();
 
 	if (!mSearchOptions.SearchValid)
 	{
@@ -1079,7 +1145,7 @@ void lcMainWindow::ShowSearchDialog()
 			mSearchOptions.Info = ((lcPiece*)Focus)->mPieceInfo;
 	}
 
-	lcQFindDialog Dialog(this, &mSearchOptions);
+	lcQFindDialog Dialog(this, &mSearchOptions, Model);
 	if (Dialog.exec() == QDialog::Accepted)
 		Model->FindPiece(true, true);
 }
@@ -1150,8 +1216,8 @@ void lcMainWindow::SetSelectionMode(lcSelectionMode SelectionMode)
 
 QByteArray lcMainWindow::GetTabLayout()
 {
-	QByteArray TabLayout;
-	QDataStream DataStream(&TabLayout, QIODevice::WriteOnly);
+	QByteArray TabLayoutData;
+	QDataStream DataStream(&TabLayoutData, QIODevice::WriteOnly);
 
 	DataStream << (quint32)LC_TAB_LAYOUT_VERSION;
 	qint32 NumTabs = mModelTabWidget->count();
@@ -1207,7 +1273,7 @@ QByteArray lcMainWindow::GetTabLayout()
 		SaveWidget(TabLayout->itemAt(0)->widget());
 	}
 
-	return TabLayout;
+	return TabLayoutData;
 }
 
 void lcMainWindow::RestoreTabLayout(const QByteArray& TabLayout)
@@ -1248,7 +1314,7 @@ void lcMainWindow::RestoreTabLayout(const QByteArray& TabLayout)
 
 		QWidget* ActiveWidget = nullptr;
 
-		std::function<void(QWidget*)> LoadWidget = [&DataStream, &LoadWidget, Model, &ActiveWidget, this](QWidget* ParentWidget)
+		std::function<void(QWidget*)> LoadWidget = [&DataStream, &LoadWidget, &ActiveWidget, this](QWidget* ParentWidget)
 		{
 			qint32 WidgetType;
 			DataStream >> WidgetType;
@@ -1284,12 +1350,18 @@ void lcMainWindow::RestoreTabLayout(const QByteArray& TabLayout)
 					if (CurrentView)
 					{
 						lcCamera* Camera = CurrentView->mCamera;
-						Camera->m_fovy = FoV;
-						Camera->m_zNear = ZNear;
-						Camera->m_zFar = ZFar;
-						Camera->mPosition = Position;
-						Camera->mTargetPosition = TargetPosition;
-						Camera->mUpVector = UpVector;
+						if (!std::isnan(FoV))
+							Camera->m_fovy = FoV;
+						if (!std::isnan(ZNear))
+							Camera->m_zNear = ZNear;
+						if (!std::isnan(ZFar))
+							Camera->m_zFar = ZFar;
+						if (!Position.IsNan() && !TargetPosition.IsNan() && !UpVector.IsNan())
+						{
+							Camera->mPosition = Position;
+							Camera->mTargetPosition = TargetPosition;
+							Camera->mUpVector = UpVector;
+						}
 						Camera->UpdatePosition(1);
 					}
 				}
@@ -1360,6 +1432,12 @@ void lcMainWindow::RemoveAllModelTabs()
 	}
 }
 
+void lcMainWindow::CloseCurrentModelTab()
+{
+	if (mModelTabWidget->count() > 1)
+		delete mModelTabWidget->currentWidget();
+}
+
 void lcMainWindow::SetCurrentModelTab(lcModel* Model)
 {
 	lcModelTabWidget* EmptyWidget = nullptr;
@@ -1386,7 +1464,6 @@ void lcMainWindow::SetCurrentModelTab(lcModel* Model)
 	{
 		TabWidget = new lcModelTabWidget(Model);
 		mModelTabWidget->addTab(TabWidget, Model->GetProperties().mName);
-		mModelTabWidget->setCurrentWidget(TabWidget);
 
 		QGridLayout* CentralLayout = new QGridLayout(TabWidget);
 		CentralLayout->setContentsMargins(0, 0, 0, 0);
@@ -1394,12 +1471,13 @@ void lcMainWindow::SetCurrentModelTab(lcModel* Model)
 		NewView = new View(Model);
 		ViewWidget = new lcQGLWidget(TabWidget, NewView, true);
 		CentralLayout->addWidget(ViewWidget, 0, 0, 1, 1);
+
+		mModelTabWidget->setCurrentWidget(TabWidget);
 	}
 	else
 	{
 		TabWidget = EmptyWidget;
 		TabWidget->SetModel(Model);
-		mModelTabWidget->setCurrentWidget(TabWidget);
 
 		NewView = new View(Model);
 		ViewWidget = (lcQGLWidget*)TabWidget->layout()->itemAt(0)->widget();
@@ -1408,6 +1486,8 @@ void lcMainWindow::SetCurrentModelTab(lcModel* Model)
 		NewView->mWidth = ViewWidget->width();
 		NewView->mHeight = ViewWidget->height();
 		AddView(NewView);
+
+		mModelTabWidget->setCurrentWidget(TabWidget);
 	}
 
 	ViewWidget->show();
@@ -1433,7 +1513,7 @@ void lcMainWindow::ResetCameras()
 
 void lcMainWindow::AddView(View* View)
 {
-	lcModelTabWidget* TabWidget = GetTabWidgetForModel(View->mModel);
+	lcModelTabWidget* TabWidget = GetTabWidgetForModel(View->GetModel());
 
 	if (!TabWidget)
 		return;
@@ -1460,10 +1540,14 @@ void lcMainWindow::RemoveView(View* View)
 void lcMainWindow::SetActiveView(View* ActiveView)
 {
 	lcModelTabWidget* TabWidget = GetTabForView(ActiveView);
+	View* CurrentActiveView = TabWidget->GetActiveView();
 
-	if (!TabWidget || TabWidget->GetActiveView() == ActiveView)
+	if (!TabWidget || CurrentActiveView == ActiveView)
 		return;
 
+	if (CurrentActiveView)
+		CurrentActiveView->SetTopSubmodelActive();
+
 	TabWidget->SetActiveView(ActiveView);
 
 	UpdateCameraMenu();
@@ -1601,7 +1685,7 @@ void lcMainWindow::SplitView(Qt::Orientation Orientation)
 		Splitter = new QSplitter(Orientation, Parent);
 		Parent->layout()->addWidget(Splitter);
 		Splitter->addWidget(Focus);
-		Splitter->addWidget(new lcQGLWidget(mModelTabWidget->currentWidget(), new View(lcGetActiveModel()), true));
+		Splitter->addWidget(new lcQGLWidget(mModelTabWidget->currentWidget(), new View(GetCurrentTabModel()), true));
 	}
 	else
 	{
@@ -1612,7 +1696,7 @@ void lcMainWindow::SplitView(Qt::Orientation Orientation)
 		Splitter = new QSplitter(Orientation, Parent);
 		ParentSplitter->insertWidget(FocusIndex, Splitter);
 		Splitter->addWidget(Focus);
-		Splitter->addWidget(new lcQGLWidget(mModelTabWidget->currentWidget(), new View(lcGetActiveModel()), true));
+		Splitter->addWidget(new lcQGLWidget(mModelTabWidget->currentWidget(), new View(GetCurrentTabModel()), true));
 
 		ParentSplitter->setSizes(Sizes);
 	}
@@ -1756,11 +1840,13 @@ void lcMainWindow::RemoveRecentFile(int FileIndex)
 
 void lcMainWindow::UpdateSelectedObjects(bool SelectionChanged)
 {
-	int Flags;
+	int Flags = 0;
 	lcArray<lcObject*> Selection;
-	lcObject* Focus;
+	lcObject* Focus = nullptr;
 
-	lcGetActiveModel()->GetSelectionInformation(&Flags, Selection, &Focus);
+	lcModel* ActiveModel = GetActiveModel();
+	if (ActiveModel)
+		ActiveModel->GetSelectionInformation(&Flags, Selection, &Focus);
 
 	if (SelectionChanged)
 	{
@@ -1780,6 +1866,7 @@ void lcMainWindow::UpdateSelectedObjects(bool SelectionChanged)
 		mActions[LC_PIECE_DELETE]->setEnabled(Flags & LC_SEL_SELECTED);
 		mActions[LC_PIECE_DUPLICATE]->setEnabled(Flags & LC_SEL_SELECTED);
 		mActions[LC_PIECE_RESET_PIVOT_POINT]->setEnabled(Flags & LC_SEL_SELECTED);
+		mActions[LC_PIECE_REMOVE_KEY_FRAMES]->setEnabled(Flags & LC_SEL_SELECTED);
 		mActions[LC_PIECE_ARRAY]->setEnabled(Flags & LC_SEL_PIECE);
 		mActions[LC_PIECE_CONTROL_POINT_INSERT]->setEnabled(Flags & LC_SEL_CAN_ADD_CONTROL_POINT);
 		mActions[LC_PIECE_CONTROL_POINT_REMOVE]->setEnabled(Flags & LC_SEL_CAN_REMOVE_CONTROL_POINT);
@@ -1790,6 +1877,7 @@ void lcMainWindow::UpdateSelectedObjects(bool SelectionChanged)
 		mActions[LC_PIECE_VIEW_SELECTED_MODEL]->setEnabled(Flags & LC_SEL_MODEL_SELECTED);
 		mActions[LC_PIECE_MOVE_SELECTION_TO_MODEL]->setEnabled(Flags & LC_SEL_PIECE);
 		mActions[LC_PIECE_INLINE_SELECTED_MODELS]->setEnabled(Flags & LC_SEL_MODEL_SELECTED);
+		mActions[LC_PIECE_EDIT_SELECTED_SUBMODEL]->setEnabled(Flags & LC_SEL_MODEL_SELECTED);
 		mActions[LC_PIECE_GROUP]->setEnabled(Flags & LC_SEL_CAN_GROUP);
 		mActions[LC_PIECE_UNGROUP]->setEnabled(Flags & LC_SEL_GROUPED);
 		mActions[LC_PIECE_GROUP_ADD]->setEnabled((Flags & (LC_SEL_GROUPED | LC_SEL_FOCUS_GROUPED)) == LC_SEL_GROUPED);
@@ -1798,6 +1886,8 @@ void lcMainWindow::UpdateSelectedObjects(bool SelectionChanged)
 		mActions[LC_PIECE_SHOW_EARLIER]->setEnabled(Flags & LC_SEL_PIECE); // FIXME: disable if current step is 1
 		mActions[LC_PIECE_SHOW_LATER]->setEnabled(Flags & LC_SEL_PIECE);
 		mActions[LC_TIMELINE_MOVE_SELECTION]->setEnabled(Flags & LC_SEL_PIECE);
+
+		mActions[LC_PIECE_EDIT_END_SUBMODEL]->setEnabled(GetCurrentTabModel() != ActiveModel);
 	}
 
 	mPropertiesWidget->Update(Selection, Focus);
@@ -2123,7 +2213,6 @@ bool lcMainWindow::OpenProject(const QString& FileName)
 		return true;
 	}
 
-	QMessageBox::information(this, tr("LeoCAD"), tr("Error loading '%1'.").arg(LoadFileName));
 	delete NewProject;
 
 	return false;
@@ -2158,11 +2247,8 @@ void lcMainWindow::MergeProject()
 
 		UpdateModels();
 	}
-	else
-	{
-		QMessageBox::information(this, tr("LeoCAD"), tr("Error loading '%1'.").arg(LoadFileName));
-		delete NewProject;
-	}
+
+	delete NewProject;
 }
 
 void lcMainWindow::ImportLDD()
@@ -2204,18 +2290,25 @@ void lcMainWindow::ImportInventory()
 	else
 		delete NewProject;
 }
-
 bool lcMainWindow::SaveProject(const QString& FileName)
 {
-	QString SaveFileName;
+	QString SaveFileName = FileName;
 	Project* Project = lcGetActiveProject();
 
-	if (!FileName.isEmpty())
-		SaveFileName = FileName;
-	else
+	if (!SaveFileName.isEmpty() && Project->GetModels().GetSize() > 1 && QFileInfo(SaveFileName).suffix().toLower() != QLatin1String("mpd"))
+		SaveFileName.clear();
+
+	if (SaveFileName.isEmpty())
 	{
 		SaveFileName = Project->GetFileName();
 
+		if (Project->GetModels().GetSize() > 1 && QFileInfo(SaveFileName).suffix().toLower() != QLatin1String("mpd"))
+		{
+			int SuffixLength = QFileInfo(SaveFileName).suffix().length();
+			if (SuffixLength)
+				SaveFileName = SaveFileName.left(SaveFileName.length() - SuffixLength - 1);
+		}
+
 		if (SaveFileName.isEmpty())
 			SaveFileName = QFileInfo(QDir(lcGetProfileString(LC_PROFILE_PROJECTS_PATH)), Project->GetTitle()).absoluteFilePath();
 
@@ -2270,7 +2363,7 @@ bool lcMainWindow::SaveProjectIfModified()
 
 bool lcMainWindow::SetModelFromFocus()
 {
-	lcObject* FocusObject = lcGetActiveModel()->GetFocusObject();
+	lcObject* FocusObject = GetActiveModel()->GetFocusObject();
 
 	if (!FocusObject || !FocusObject->IsPiece())
 		return false;
@@ -2292,7 +2385,7 @@ void lcMainWindow::SetModelFromSelection()
 	if (SetModelFromFocus())
 		return;
 
-	lcModel* Model = lcGetActiveModel()->GetFirstSelectedSubmodel();
+	lcModel* Model = GetActiveModel()->GetFirstSelectedSubmodel();
 
 	if (Model)
 	{
@@ -2301,9 +2394,16 @@ void lcMainWindow::SetModelFromSelection()
 	}
 }
 
+lcModel* lcMainWindow::GetActiveModel() const
+{
+	View* ActiveView = GetActiveView();
+	return ActiveView ? ActiveView->GetActiveModel() : nullptr;
+}
+
 void lcMainWindow::HandleCommand(lcCommandId CommandId)
 {
 	View* ActiveView = GetActiveView();
+	lcModel* ActiveModel = ActiveView ? ActiveView->GetActiveModel() : nullptr;
 
 	switch (CommandId)
 	{
@@ -2396,23 +2496,28 @@ void lcMainWindow::HandleCommand(lcCommandId CommandId)
 		break;
 
 	case LC_EDIT_UNDO:
-		lcGetActiveModel()->UndoAction();
+		if (ActiveModel)
+			ActiveModel->UndoAction();
 		break;
 
 	case LC_EDIT_REDO:
-		lcGetActiveModel()->RedoAction();
+		if (ActiveModel)
+			ActiveModel->RedoAction();
 		break;
 
 	case LC_EDIT_CUT:
-		lcGetActiveModel()->Cut();
+		if (ActiveModel)
+			ActiveModel->Cut();
 		break;
 
 	case LC_EDIT_COPY:
-		lcGetActiveModel()->Copy();
+		if (ActiveModel)
+			ActiveModel->Copy();
 		break;
 
 	case LC_EDIT_PASTE:
-		lcGetActiveModel()->Paste();
+		if (ActiveModel)
+			ActiveModel->Paste();
 		break;
 
 	case LC_EDIT_FIND:
@@ -2420,31 +2525,38 @@ void lcMainWindow::HandleCommand(lcCommandId CommandId)
 		break;
 
 	case LC_EDIT_FIND_NEXT:
-		lcGetActiveModel()->FindPiece(false, true);
+		if (ActiveModel)
+			ActiveModel->FindPiece(false, true);
 		break;
 
 	case LC_EDIT_FIND_PREVIOUS:
-		lcGetActiveModel()->FindPiece(false, false);
+		if (ActiveModel)
+			ActiveModel->FindPiece(false, false);
 		break;
 
 	case LC_EDIT_SELECT_ALL:
-		lcGetActiveModel()->SelectAllPieces();
+		if (ActiveModel)
+			ActiveModel->SelectAllPieces();
 		break;
 
 	case LC_EDIT_SELECT_NONE:
-		lcGetActiveModel()->ClearSelection(true);
+		if (ActiveModel)
+			ActiveModel->ClearSelection(true);
 		break;
 
 	case LC_EDIT_SELECT_INVERT:
-		lcGetActiveModel()->InvertSelection();
+		if (ActiveModel)
+			ActiveModel->InvertSelection();
 		break;
 
 	case LC_EDIT_SELECT_BY_NAME:
-		lcGetActiveModel()->ShowSelectByNameDialog();
+		if (ActiveModel)
+			ActiveModel->ShowSelectByNameDialog();
 		break;
 
 	case LC_EDIT_SELECT_BY_COLOR:
-		lcGetActiveModel()->ShowSelectByColorDialog();
+		if (ActiveModel)
+			ActiveModel->ShowSelectByColorDialog();
 		break;
 
 	case LC_EDIT_SELECTION_SINGLE:
@@ -2483,6 +2595,10 @@ void lcMainWindow::HandleCommand(lcCommandId CommandId)
 		ToggleFullScreen();
 		break;
 
+	case LC_VIEW_CLOSE_CURRENT_TAB:
+		CloseCurrentModelTab();
+		break;
+
 	case LC_VIEW_SHADING_WIREFRAME:
 		SetShadingMode(LC_SHADING_WIREFRAME);
 		break;
@@ -2506,95 +2622,108 @@ void lcMainWindow::HandleCommand(lcCommandId CommandId)
 		break;
 
 	case LC_PIECE_INSERT:
-		lcGetActiveModel()->AddPiece();
+		if (ActiveModel)
+			ActiveModel->AddPiece();
 		break;
 
 	case LC_PIECE_DELETE:
-		lcGetActiveModel()->DeleteSelectedObjects();
+		if (ActiveModel)
+			ActiveModel->DeleteSelectedObjects();
 		break;
 
 	case LC_PIECE_DUPLICATE:
-		lcGetActiveModel()->DuplicateSelectedPieces();
+		if (ActiveModel)
+			ActiveModel->DuplicateSelectedPieces();
 		break;
 
 	case LC_PIECE_RESET_PIVOT_POINT:
-		lcGetActiveModel()->ResetSelectedPiecesPivotPoint();
+		if (ActiveModel)
+			ActiveModel->ResetSelectedPiecesPivotPoint();
+		break;
+
+	case LC_PIECE_REMOVE_KEY_FRAMES:
+		if (ActiveModel)
+			ActiveModel->RemoveSelectedPiecesKeyFrames();
 		break;
 
 	case LC_PIECE_CONTROL_POINT_INSERT:
-		lcGetActiveModel()->InsertControlPoint();
+		if (ActiveModel)
+			ActiveModel->InsertControlPoint();
 		break;
 
 	case LC_PIECE_CONTROL_POINT_REMOVE:
-		lcGetActiveModel()->RemoveFocusedControlPoint();
+		if (ActiveModel)
+			ActiveModel->RemoveFocusedControlPoint();
 		break;
 
 	case LC_PIECE_MOVE_PLUSX:
-		if (ActiveView)
-			lcGetActiveModel()->MoveSelectedObjects(ActiveView->GetMoveDirection(lcVector3(lcMax(GetMoveXYSnap(), 0.1f), 0.0f, 0.0f)), true, false, true, true);
+		if (ActiveModel)
+			ActiveModel->MoveSelectedObjects(ActiveView->GetMoveDirection(lcVector3(lcMax(GetMoveXYSnap(), 0.1f), 0.0f, 0.0f)), true, false, true, true);
 		break;
 
 	case LC_PIECE_MOVE_MINUSX:
-		if (ActiveView)
-			lcGetActiveModel()->MoveSelectedObjects(ActiveView->GetMoveDirection(lcVector3(-lcMax(GetMoveXYSnap(), 0.1f), 0.0f, 0.0f)), true, false, true, true);
+		if (ActiveModel)
+			ActiveModel->MoveSelectedObjects(ActiveView->GetMoveDirection(lcVector3(-lcMax(GetMoveXYSnap(), 0.1f), 0.0f, 0.0f)), true, false, true, true);
 		break;
 
 	case LC_PIECE_MOVE_PLUSY:
-		if (ActiveView)
-			lcGetActiveModel()->MoveSelectedObjects(ActiveView->GetMoveDirection(lcVector3(0.0f, lcMax(GetMoveXYSnap(), 0.1f), 0.0f)), true, false, true, true);
+		if (ActiveModel)
+			ActiveModel->MoveSelectedObjects(ActiveView->GetMoveDirection(lcVector3(0.0f, lcMax(GetMoveXYSnap(), 0.1f), 0.0f)), true, false, true, true);
 		break;
 
 	case LC_PIECE_MOVE_MINUSY:
-		if (ActiveView)
-			lcGetActiveModel()->MoveSelectedObjects(ActiveView->GetMoveDirection(lcVector3(0.0f, -lcMax(GetMoveXYSnap(), 0.1f), 0.0f)), true, false, true, true);
+		if (ActiveModel)
+			ActiveModel->MoveSelectedObjects(ActiveView->GetMoveDirection(lcVector3(0.0f, -lcMax(GetMoveXYSnap(), 0.1f), 0.0f)), true, false, true, true);
 		break;
 
 	case LC_PIECE_MOVE_PLUSZ:
-		if (ActiveView)
-			lcGetActiveModel()->MoveSelectedObjects(ActiveView->GetMoveDirection(lcVector3(0.0f, 0.0f, lcMax(GetMoveZSnap(), 0.1f))), true, false, true, true);
+		if (ActiveModel)
+			ActiveModel->MoveSelectedObjects(ActiveView->GetMoveDirection(lcVector3(0.0f, 0.0f, lcMax(GetMoveZSnap(), 0.1f))), true, false, true, true);
 		break;
 
 	case LC_PIECE_MOVE_MINUSZ:
-		if (ActiveView)
-			lcGetActiveModel()->MoveSelectedObjects(ActiveView->GetMoveDirection(lcVector3(0.0f, 0.0f, -lcMax(GetMoveZSnap(), 0.1f))), true, false, true, true);
+		if (ActiveModel)
+			ActiveModel->MoveSelectedObjects(ActiveView->GetMoveDirection(lcVector3(0.0f, 0.0f, -lcMax(GetMoveZSnap(), 0.1f))), true, false, true, true);
 		break;
 
 	case LC_PIECE_ROTATE_PLUSX:
-		if (ActiveView)
-			lcGetActiveModel()->RotateSelectedPieces(ActiveView->GetMoveDirection(lcVector3(lcMax(GetAngleSnap(), 1.0f), 0.0f, 0.0f)), true, false, true, true);
+		if (ActiveModel)
+			ActiveModel->RotateSelectedPieces(ActiveView->GetMoveDirection(lcVector3(lcMax(GetAngleSnap(), 1.0f), 0.0f, 0.0f)), true, false, true, true);
 		break;
 
 	case LC_PIECE_ROTATE_MINUSX:
-		if (ActiveView)
-			lcGetActiveModel()->RotateSelectedPieces(ActiveView->GetMoveDirection(-lcVector3(lcMax(GetAngleSnap(), 1.0f), 0.0f, 0.0f)), true, false, true, true);
+		if (ActiveModel)
+			ActiveModel->RotateSelectedPieces(ActiveView->GetMoveDirection(-lcVector3(lcMax(GetAngleSnap(), 1.0f), 0.0f, 0.0f)), true, false, true, true);
 		break;
 
 	case LC_PIECE_ROTATE_PLUSY:
-		if (ActiveView)
-			lcGetActiveModel()->RotateSelectedPieces(ActiveView->GetMoveDirection(lcVector3(0.0f, lcMax(GetAngleSnap(), 1.0f), 0.0f)), true, false, true, true);
+		if (ActiveModel)
+			ActiveModel->RotateSelectedPieces(ActiveView->GetMoveDirection(lcVector3(0.0f, lcMax(GetAngleSnap(), 1.0f), 0.0f)), true, false, true, true);
 		break;
 
 	case LC_PIECE_ROTATE_MINUSY:
-		if (ActiveView)
-			lcGetActiveModel()->RotateSelectedPieces(ActiveView->GetMoveDirection(lcVector3(0.0f, -lcMax(GetAngleSnap(), 1.0f), 0.0f)), true, false, true, true);
+		if (ActiveModel)
+			ActiveModel->RotateSelectedPieces(ActiveView->GetMoveDirection(lcVector3(0.0f, -lcMax(GetAngleSnap(), 1.0f), 0.0f)), true, false, true, true);
 		break;
 
 	case LC_PIECE_ROTATE_PLUSZ:
-		if (ActiveView)
-			lcGetActiveModel()->RotateSelectedPieces(ActiveView->GetMoveDirection(lcVector3(0.0f, 0.0f, lcMax(GetAngleSnap(), 1.0f))), true, false, true, true);
+		if (ActiveModel)
+			ActiveModel->RotateSelectedPieces(ActiveView->GetMoveDirection(lcVector3(0.0f, 0.0f, lcMax(GetAngleSnap(), 1.0f))), true, false, true, true);
 		break;
 
 	case LC_PIECE_ROTATE_MINUSZ:
-		if (ActiveView)
-			lcGetActiveModel()->RotateSelectedPieces(ActiveView->GetMoveDirection(lcVector3(0.0f, 0.0f, -lcMax(GetAngleSnap(), 1.0f))), true, false, true, true);
+		if (ActiveModel)
+			ActiveModel->RotateSelectedPieces(ActiveView->GetMoveDirection(lcVector3(0.0f, 0.0f, -lcMax(GetAngleSnap(), 1.0f))), true, false, true, true);
 		break;
 
 	case LC_PIECE_MINIFIG_WIZARD:
-		lcGetActiveModel()->ShowMinifigDialog();
+		if (ActiveModel)
+			ActiveModel->ShowMinifigDialog();
 		break;
 
 	case LC_PIECE_ARRAY:
-		lcGetActiveModel()->ShowArrayDialog();
+		if (ActiveModel)
+			ActiveModel->ShowArrayDialog();
 		break;
 
 	case LC_PIECE_VIEW_SELECTED_MODEL:
@@ -2602,55 +2731,85 @@ void lcMainWindow::HandleCommand(lcCommandId CommandId)
 		break;
 
 	case LC_PIECE_MOVE_SELECTION_TO_MODEL:
-		lcGetActiveModel()->MoveSelectionToModel(lcGetActiveProject()->CreateNewModel(false));
+		if (ActiveModel)
+			ActiveModel->MoveSelectionToModel(lcGetActiveProject()->CreateNewModel(false));
 		break;
 
 	case LC_PIECE_INLINE_SELECTED_MODELS:
-		lcGetActiveModel()->InlineSelectedModels();
+		if (ActiveModel)
+			ActiveModel->InlineSelectedModels();
+		break;
+
+	case LC_PIECE_EDIT_END_SUBMODEL:
+		if (ActiveView)
+		{
+			ActiveView->SetTopSubmodelActive();
+			if (ActiveModel)
+			{
+				std::vector<lcModel*> UpdatedModels;
+				ActiveModel->UpdatePieceInfo(UpdatedModels);
+			}
+		}
+		break;
+
+	case LC_PIECE_EDIT_SELECTED_SUBMODEL:
+		if (ActiveView)
+			ActiveView->SetSelectedSubmodelActive();
 		break;
 
 	case LC_PIECE_GROUP:
-		lcGetActiveModel()->GroupSelection();
+		if (ActiveModel)
+			ActiveModel->GroupSelection();
 		break;
 
 	case LC_PIECE_UNGROUP:
-		lcGetActiveModel()->UngroupSelection();
+		if (ActiveModel)
+			ActiveModel->UngroupSelection();
 		break;
 
 	case LC_PIECE_GROUP_ADD:
-		lcGetActiveModel()->AddSelectedPiecesToGroup();
+		if (ActiveModel)
+			ActiveModel->AddSelectedPiecesToGroup();
 		break;
 
 	case LC_PIECE_GROUP_REMOVE:
-		lcGetActiveModel()->RemoveFocusPieceFromGroup();
+		if (ActiveModel)
+			ActiveModel->RemoveFocusPieceFromGroup();
 		break;
 
 	case LC_PIECE_GROUP_EDIT:
-		lcGetActiveModel()->ShowEditGroupsDialog();
+		if (ActiveModel)
+			ActiveModel->ShowEditGroupsDialog();
 		break;
 
 	case LC_PIECE_HIDE_SELECTED:
-		lcGetActiveModel()->HideSelectedPieces();
+		if (ActiveModel)
+			ActiveModel->HideSelectedPieces();
 		break;
 
 	case LC_PIECE_HIDE_UNSELECTED:
-		lcGetActiveModel()->HideUnselectedPieces();
+		if (ActiveModel)
+			ActiveModel->HideUnselectedPieces();
 		break;
 
 	case LC_PIECE_UNHIDE_SELECTED:
-		lcGetActiveModel()->UnhideSelectedPieces();
+		if (ActiveModel)
+			ActiveModel->UnhideSelectedPieces();
 		break;
 
 	case LC_PIECE_UNHIDE_ALL:
-		lcGetActiveModel()->UnhideAllPieces();
+		if (ActiveModel)
+			ActiveModel->UnhideAllPieces();
 		break;
 
 	case LC_PIECE_SHOW_EARLIER:
-		lcGetActiveModel()->ShowSelectedPiecesEarlier();
+		if (ActiveModel)
+			ActiveModel->ShowSelectedPiecesEarlier();
 		break;
 
 	case LC_PIECE_SHOW_LATER:
-		lcGetActiveModel()->ShowSelectedPiecesLater();
+		if (ActiveModel)
+			ActiveModel->ShowSelectedPiecesLater();
 		break;
 
 	case LC_VIEW_PREFERENCES:
@@ -2659,12 +2818,12 @@ void lcMainWindow::HandleCommand(lcCommandId CommandId)
 
 	case LC_VIEW_ZOOM_IN:
 		if (ActiveView)
-			lcGetActiveModel()->Zoom(ActiveView->mCamera, 10.0f);
+			ActiveView->Zoom(10.0f);
 		break;
 
 	case LC_VIEW_ZOOM_OUT:
 		if (ActiveView)
-			lcGetActiveModel()->Zoom(ActiveView->mCamera, -10.0f);
+			ActiveView->Zoom(-10.0f);
 		break;
 
 	case LC_VIEW_ZOOM_EXTENTS:
@@ -2708,19 +2867,23 @@ void lcMainWindow::HandleCommand(lcCommandId CommandId)
 		break;
 
 	case LC_VIEW_TIME_NEXT:
-		lcGetActiveModel()->ShowNextStep();
+		if (ActiveModel)
+			ActiveModel->ShowNextStep();
 		break;
 
 	case LC_VIEW_TIME_PREVIOUS:
-		lcGetActiveModel()->ShowPreviousStep();
+		if (ActiveModel)
+			ActiveModel->ShowPreviousStep();
 		break;
 
 	case LC_VIEW_TIME_FIRST:
-		lcGetActiveModel()->ShowFirstStep();
+		if (ActiveModel)
+			ActiveModel->ShowFirstStep();
 		break;
 
 	case LC_VIEW_TIME_LAST:
-		lcGetActiveModel()->ShowLastStep();
+		if (ActiveModel)
+			ActiveModel->ShowLastStep();
 		break;
 
 	case LC_VIEW_TIME_INSERT:
@@ -2803,10 +2966,6 @@ void lcMainWindow::HandleCommand(lcCommandId CommandId)
 		lcGetActiveModel()->ShowPropertiesDialog();
 		break;
 
-	case LC_MODEL_EDIT_FOCUS:
-		SetModelFromFocus();
-		break;
-
 	case LC_MODEL_LIST:
 		lcGetActiveProject()->ShowModelListDialog();
 		break;
@@ -2839,11 +2998,11 @@ void lcMainWindow::HandleCommand(lcCommandId CommandId)
 		break;
 
 	case LC_HELP_HOMEPAGE:
-		QDesktopServices::openUrl(QUrl("http://www.leocad.org/"));
+		QDesktopServices::openUrl(QUrl("https://www.leocad.org/"));
 		break;
 
-	case LC_HELP_EMAIL:
-		QDesktopServices::openUrl(QUrl("mailto:leozide@gmail.com?subject=LeoCAD"));
+	case LC_HELP_BUG_REPORT:
+		QDesktopServices::openUrl(QUrl("https://github.com/leozide/leocad/issues"));
 		break;
 
 	case LC_HELP_UPDATES:
@@ -2910,7 +3069,8 @@ void lcMainWindow::HandleCommand(lcCommandId CommandId)
 		break;
 
 	case LC_EDIT_TRANSFORM:
-		lcGetActiveModel()->TransformSelectedObjects(GetTransformType(), GetTransformAmount());
+		if (ActiveModel)
+			ActiveModel->TransformSelectedObjects(GetTransformType(), GetTransformAmount());
 		break;
 
 	case LC_EDIT_TRANSFORM_ABSOLUTE_TRANSLATION:
