@@ -71,6 +71,11 @@ public:
 		return ((float*)this)[i];
 	}
 
+	bool IsNan() const
+	{
+		return std::isnan(x) || std::isnan(y);
+	}
+
 	float x, y;
 };
 
@@ -108,6 +113,11 @@ public:
 		return ((float*)this)[i];
 	}
 
+	bool IsNan() const
+	{
+		return std::isnan(x) || std::isnan(y) || std::isnan(z);
+	}
+
 	void Normalize();
 	float Length() const;
 	float LengthSquared() const;
@@ -152,6 +162,11 @@ public:
 		return ((float*)this)[i];
 	}
 
+	bool IsNan() const
+	{
+		return std::isnan(x) || std::isnan(y) || std::isnan(z) || std::isnan(w);
+	}
+
 	float x, y, z, w;
 };
 
