@@ -19,6 +19,7 @@
 #define LC_SEL_CAN_REMOVE_CONTROL_POINT 0x1000 // Can remove control points from focused piece
 
 class lcGLWidget;
+enum class lcRenderMeshState : int;
 
 enum class lcSelectionMode
 {
@@ -90,13 +91,6 @@ struct lcModelHistoryEntry
 	QString Description;
 };
 
-struct lcModelPartsEntry
-{
-	lcMatrix44 WorldMatrix;
-	PieceInfo* Info;
-	int ColorIndex;
-};
-
 class lcModel
 {
 public:
@@ -108,9 +102,10 @@ public:
 		return mSavedHistory != mUndoHistory[0];
 	}
 
+	bool GetPieceWorldMatrix(lcPiece* Piece, lcMatrix44& ParentWorldMatrix) const;
 	bool IncludesModel(const lcModel* Model) const;
 	void CreatePieceInfo(Project* Project);
-	void UpdatePieceInfo(lcArray<lcModel*>& UpdatedModels);
+	void UpdatePieceInfo(std::vector<lcModel*>& UpdatedModels);
 	void UpdateMesh();
 
 	PieceInfo* GetPieceInfo() const
@@ -190,6 +185,7 @@ public:
 	void DeleteAllCameras();
 	void DeleteSelectedObjects();
 	void ResetSelectedPiecesPivotPoint();
+	void RemoveSelectedPiecesKeyFrames();
 	void InsertControlPoint();
 	void RemoveFocusedControlPoint();
 	void ShowSelectedPiecesEarlier();
@@ -219,7 +215,7 @@ public:
 
 	void SetSaved()
 	{
-		if (mUndoHistory.IsEmpty())
+		if (mUndoHistory.empty())
 			SaveCheckpoint(QString());
 
 		mSavedHistory = mUndoHistory[0];
@@ -230,8 +226,8 @@ public:
 	void Paste();
 	void DuplicateSelectedPieces();
 
-	void GetScene(lcScene& Scene, lcCamera* ViewCamera, bool DrawInterface, bool Highlight) const;
-	void SubModelAddRenderMeshes(lcScene& Scene, const lcMatrix44& WorldMatrix, int DefaultColorIndex, bool Focused, bool Selected) const;
+	void GetScene(lcScene& Scene, lcCamera* ViewCamera, bool DrawInterface, bool Highlight, lcPiece* ActiveSubmodelInstance, const lcMatrix44& ActiveSubmodelTransform) const;
+	void AddSubModelRenderMeshes(lcScene& Scene, const lcMatrix44& WorldMatrix, int DefaultColorIndex, lcRenderMeshState RenderMeshState, bool ParentActive) const;
 	void DrawBackground(lcGLWidget* Widget);
 	void SaveStepImages(const QString& BaseName, bool AddStepSuffix, bool Zoom, bool Highlight, int Width, int Height, lcStep Start, lcStep End);
 
@@ -253,7 +249,7 @@ public:
 	bool GetPiecesBoundingBox(lcVector3& Min, lcVector3& Max) const;
 	void GetPartsList(int DefaultColorIndex, bool IncludeSubmodels, lcPartsList& PartsList) const;
 	void GetPartsListForStep(lcStep Step, int DefaultColorIndex, lcPartsList& PartsList) const;
-	void GetModelParts(const lcMatrix44& WorldMatrix, int DefaultColorIndex, lcArray<lcModelPartsEntry>& ModelParts) const;
+	void GetModelParts(const lcMatrix44& WorldMatrix, int DefaultColorIndex, std::vector<lcModelPartsEntry>& ModelParts) const;
 	void GetSelectionInformation(int* Flags, lcArray<lcObject*>& Selection, lcObject** Focus) const;
 	lcArray<lcObject*> GetSelectionModePieces(lcPiece* SelectedPiece) const;
 
@@ -369,8 +365,8 @@ protected:
 	QStringList mFileLines;
 
 	lcModelHistoryEntry* mSavedHistory;
-	lcArray<lcModelHistoryEntry*> mUndoHistory;
-	lcArray<lcModelHistoryEntry*> mRedoHistory;
+	std::vector<lcModelHistoryEntry*> mUndoHistory;
+	std::vector<lcModelHistoryEntry*> mRedoHistory;
 
 	Q_DECLARE_TR_FUNCTIONS(lcModel);
 };
