@@ -67,7 +67,7 @@ void lcPartSelectionListModel::ClearRequests()
 {
 	lcPiecesLibrary* Library = lcGetPiecesLibrary();
 
-	foreach(int RequestIdx, mRequestedPreviews)
+	for (int RequestIdx : mRequestedPreviews)
 	{
 		PieceInfo* Info = mParts[RequestIdx].first;
 		Library->ReleasePieceInfo(Info);
@@ -82,7 +82,7 @@ void lcPartSelectionListModel::Redraw()
 
 	beginResetModel();
 
-	for (int PartIdx = 0; PartIdx < mParts.size(); PartIdx++)
+	for (size_t PartIdx = 0; PartIdx < mParts.size(); PartIdx++)
 		mParts[PartIdx].second = QPixmap();
 
 	endResetModel();
@@ -130,13 +130,13 @@ void lcPartSelectionListModel::SetCategory(int CategoryIndex)
 	{
 		Library->GetParts(SingleParts);
 
-		lcModel* CurrentModel = lcGetActiveModel();
+		lcModel* ActiveModel = gMainWindow->GetActiveModel();
 
 		for (int PartIdx = 0; PartIdx < SingleParts.GetSize(); )
 		{
 			PieceInfo* Info = SingleParts[PartIdx];
 
-			if (!Info->IsModel() || !Info->GetModel()->IncludesModel(CurrentModel))
+			if (!Info->IsModel() || !Info->GetModel()->IncludesModel(ActiveModel))
 				PartIdx++;
 			else
 				SingleParts.RemoveIndex(PartIdx);
@@ -168,14 +168,14 @@ void lcPartSelectionListModel::SetModelsCategory()
 	mParts.clear();
 
 	const lcArray<lcModel*>& Models = lcGetActiveProject()->GetModels();
-	lcModel* CurrentModel = lcGetActiveModel();
+	lcModel* ActiveModel = gMainWindow->GetActiveModel();
 
 	for (int ModelIdx = 0; ModelIdx < Models.GetSize(); ModelIdx++)
 	{
 		lcModel* Model = Models[ModelIdx];
 
-		if (!Model->IncludesModel(CurrentModel))
-			mParts.append(QPair<PieceInfo*, QPixmap>(Model->GetPieceInfo(), QPixmap()));
+		if (!Model->IncludesModel(ActiveModel))
+			mParts.emplace_back(QPair<PieceInfo*, QPixmap>(Model->GetPieceInfo(), QPixmap()));
 	}
 
 	endResetModel();
@@ -191,12 +191,12 @@ void lcPartSelectionListModel::SetCurrentModelCategory()
 
 	mParts.clear();
 
-	lcModel* CurrentModel = lcGetActiveModel();
+	lcModel* ActiveModel = gMainWindow->GetActiveModel();
 	lcPartsList PartsList;
-	CurrentModel->GetPartsList(gDefaultColor, true, PartsList);
+	ActiveModel->GetPartsList(gDefaultColor, true, PartsList);
 
 	for (const auto& PartIt : PartsList)
-		mParts.append(QPair<PieceInfo*, QPixmap>((PieceInfo*)PartIt.first, QPixmap()));
+		mParts.emplace_back(QPair<PieceInfo*, QPixmap>((PieceInfo*)PartIt.first, QPixmap()));
 
 	endResetModel();
 
@@ -207,7 +207,7 @@ void lcPartSelectionListModel::SetFilter(const QString& Filter)
 {
 	mFilter = Filter.toLatin1();
 
-	for (int PartIdx = 0; PartIdx < mParts.size(); PartIdx++)
+	for (size_t PartIdx = 0; PartIdx < mParts.size(); PartIdx++)
 	{
 		PieceInfo* Info = mParts[PartIdx].first;
 		bool Visible;
@@ -238,7 +238,7 @@ void lcPartSelectionListModel::SetFilter(const QString& Filter)
 			Visible = strcasestr(Description, mFilter) || strcasestr(Info->mFileName, mFilter);
 		}
 
-		mListView->setRowHidden(PartIdx, !Visible);
+		mListView->setRowHidden((int)PartIdx, !Visible);
 	}
 }
 
@@ -246,12 +246,12 @@ int lcPartSelectionListModel::rowCount(const QModelIndex& Parent) const
 {
 	Q_UNUSED(Parent);
 
-	return mParts.size();
+	return (int)mParts.size();
 }
 
 QVariant lcPartSelectionListModel::data(const QModelIndex& Index, int Role) const
 {
-	int InfoIndex = Index.row();
+	size_t InfoIndex = Index.row();
 
 	if (Index.isValid() && InfoIndex < mParts.size())
 	{
@@ -304,7 +304,7 @@ void lcPartSelectionListModel::RequestPreview(int InfoIndex)
 	if (!mIconSize || !mParts[InfoIndex].second.isNull())
 		return;
 
-	if (mRequestedPreviews.indexOf(InfoIndex) != -1)
+	if (std::find(mRequestedPreviews.begin(), mRequestedPreviews.end(), InfoIndex) != mRequestedPreviews.end())
 		return;
 
 	PieceInfo* Info = mParts[InfoIndex].first;
@@ -313,17 +313,21 @@ void lcPartSelectionListModel::RequestPreview(int InfoIndex)
 	if (Info->mState == LC_PIECEINFO_LOADED)
 		DrawPreview(InfoIndex);
 	else
-		mRequestedPreviews.append(InfoIndex);
+		mRequestedPreviews.push_back(InfoIndex);
 }
 
 void lcPartSelectionListModel::PartLoaded(PieceInfo* Info)
 {
-	for (int PartIdx = 0; PartIdx < mParts.size(); PartIdx++)
+	for (size_t PartIdx = 0; PartIdx < mParts.size(); PartIdx++)
 	{
 		if (mParts[PartIdx].first == Info)
 		{
-			if (mRequestedPreviews.removeOne(PartIdx))
-				DrawPreview(PartIdx);
+			auto PreviewIt = std::find(mRequestedPreviews.begin(), mRequestedPreviews.end(), PartIdx);
+			if (PreviewIt != mRequestedPreviews.end())
+			{
+				mRequestedPreviews.erase(PreviewIt);
+				DrawPreview((int)PartIdx);
+			}
 			break;
 		}
 	}
@@ -368,9 +372,10 @@ void lcPartSelectionListModel::DrawPreview(int InfoIndex)
 	Context->SetProjectionMatrix(ProjectionMatrix);
 
 	lcScene Scene;
+	Scene.SetAllowWireframe(false);
 	Scene.Begin(ViewMatrix);
 
-	Info->AddRenderMeshes(Scene, lcMatrix44Identity(), mColorIndex, false, false, false);
+	Info->AddRenderMeshes(Scene, lcMatrix44Identity(), mColorIndex, lcRenderMeshState::NORMAL, false);
 
 	Scene.End();
 
@@ -384,7 +389,9 @@ void lcPartSelectionListModel::DrawPreview(int InfoIndex)
 	Context->ClearResources();
 
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 2, 0))
-	emit dataChanged(index(InfoIndex, 0), index(InfoIndex, 0), QVector<int>() << Qt::DecorationRole);
+	QVector<int> Roles;
+	Roles.append(Qt::DecorationRole);
+	emit dataChanged(index(InfoIndex, 0), index(InfoIndex, 0), Roles);
 #else
 	emit dataChanged(index(InfoIndex, 0), index(InfoIndex, 0));
 #endif
@@ -409,7 +416,7 @@ void lcPartSelectionListModel::SetIconSize(int Size)
 
 	beginResetModel();
 
-	for (int PartIdx = 0; PartIdx < mParts.size(); PartIdx++)
+	for (size_t PartIdx = 0; PartIdx < mParts.size(); PartIdx++)
 		mParts[PartIdx].second = QPixmap();
 
 	endResetModel();
@@ -507,7 +514,8 @@ void lcPartSelectionListView::CustomContextMenuRequested(QPoint Pos)
 		FixedColor->setChecked(mListModel->IsColorLocked());
 	}
 
-	Menu->popup(viewport()->mapToGlobal(Pos));
+	Menu->exec(viewport()->mapToGlobal(Pos));
+	delete Menu;
 }
 
 void lcPartSelectionListView::SetNoIcons()
@@ -572,6 +580,14 @@ void lcPartSelectionListView::SetIconSize(int Size)
 	lcSetProfileInt(LC_PROFILE_PARTS_LIST_ICONS, Size);
 	mListModel->SetIconSize(Size);
 	UpdateViewMode();
+
+	int Width = Size + 2 * frameWidth() + 6;
+	if (verticalScrollBar())
+		Width += verticalScrollBar()->sizeHint().width();
+	int Height = Size + 2 * frameWidth() + 2;
+	if (horizontalScrollBar())
+		Height += horizontalScrollBar()->sizeHint().height();
+	setMinimumSize(Width, Height);
 }
 
 void lcPartSelectionListView::startDrag(Qt::DropActions SupportedActions)
@@ -791,7 +807,7 @@ void lcPartSelectionWidget::UpdateCategories()
 	for (int CategoryIdx = 0; CategoryIdx < gCategories.GetSize(); CategoryIdx++)
 		new QTreeWidgetItem(mCategoriesWidget, QStringList(gCategories[CategoryIdx].Name));
 
-	mModelsCategoryItem = new QTreeWidgetItem(mCategoriesWidget, QStringList(tr("Models")));
+	mModelsCategoryItem = new QTreeWidgetItem(mCategoriesWidget, QStringList(tr("Submodels")));
 
 	if (CurrentIndex != -1)
 		mCategoriesWidget->setCurrentItem(mCategoriesWidget->topLevelItem(CurrentIndex));
