@@ -14,10 +14,17 @@ enum LC_PROFILE_KEY
 	LC_PROFILE_GRID_LINE_SPACING,
 	LC_PROFILE_GRID_LINE_COLOR,
 	LC_PROFILE_ANTIALIASING_SAMPLES,
+	LC_PROFILE_VIEW_SPHERE_LOCATION,
+	LC_PROFILE_VIEW_SPHERE_SIZE,
+	LC_PROFILE_VIEW_SPHERE_COLOR,
+	LC_PROFILE_VIEW_SPHERE_TEXT_COLOR,
+	LC_PROFILE_VIEW_SPHERE_HIGHLIGHT_COLOR,
 
 	LC_PROFILE_CHECK_UPDATES,
 	LC_PROFILE_PROJECTS_PATH,
 	LC_PROFILE_PARTS_LIBRARY,
+	LC_PROFILE_MINIFIG_SETTINGS,
+	LC_PROFILE_COLOR_CONFIG,
 	LC_PROFILE_KEYBOARD_SHORTCUTS,
 	LC_PROFILE_MOUSE_SHORTCUTS,
 	LC_PROFILE_CATEGORIES,
@@ -25,6 +32,7 @@ enum LC_PROFILE_KEY
 	LC_PROFILE_RECENT_FILE2,
 	LC_PROFILE_RECENT_FILE3,
 	LC_PROFILE_RECENT_FILE4,
+	LC_PROFILE_AUTOLOAD_MOSTRECENT,
 	LC_PROFILE_AUTOSAVE_INTERVAL,
 	LC_PROFILE_MOUSE_SENSITIVITY,
 	LC_PROFILE_IMAGE_WIDTH,
@@ -98,8 +106,6 @@ public:
 	} mDefault;
 };
 
-extern lcProfileEntry gProfileEntries[LC_NUM_PROFILE_KEYS];
-
 void lcRemoveProfileKey(LC_PROFILE_KEY Key);
 
 int lcGetDefaultProfileInt(LC_PROFILE_KEY Key);
