@@ -10,11 +10,15 @@
 lcScene::lcScene()
 	: mRenderMeshes(0, 1024), mOpaqueMeshes(0, 1024), mTranslucentMeshes(0, 1024), mInterfaceObjects(0, 1024)
 {
+	mActiveSubmodelInstance = nullptr;
+	mAllowWireframe = true;
 }
 
 void lcScene::Begin(const lcMatrix44& ViewMatrix)
 {
 	mViewMatrix = ViewMatrix;
+	mActiveSubmodelInstance = nullptr;
+	mDrawInterface = false;
 	mRenderMeshes.RemoveAll();
 	mOpaqueMeshes.RemoveAll();
 	mTranslucentMeshes.RemoveAll();
@@ -29,14 +33,14 @@ void lcScene::End()
 		return mRenderMeshes[Index1].Mesh <  mRenderMeshes[Index2].Mesh;
 	};
 
-	std::sort(&mOpaqueMeshes[0], &mOpaqueMeshes[0] + mOpaqueMeshes.GetSize(), OpaqueMeshCompare);
+	std::sort(mOpaqueMeshes.begin(), mOpaqueMeshes.end(), OpaqueMeshCompare);
 
 	auto TranslucentMeshCompare = [this](int Index1, int Index2)
 	{
 		return mRenderMeshes[Index1].Distance <  mRenderMeshes[Index2].Distance;
 	};
 
-	std::sort(&mTranslucentMeshes[0], &mTranslucentMeshes[0] + mTranslucentMeshes.GetSize(), TranslucentMeshCompare);
+	std::sort(mTranslucentMeshes.begin(), mTranslucentMeshes.end(), TranslucentMeshCompare);
 }
 
 void lcScene::AddMesh(lcMesh* Mesh, const lcMatrix44& WorldMatrix, int ColorIndex, lcRenderMeshState State, int Flags)
@@ -66,6 +70,11 @@ void lcScene::DrawRenderMeshes(lcContext* Context, int PrimitiveTypes, bool Enab
 {
 	const lcArray<int>& Meshes = DrawTranslucent ? mTranslucentMeshes : mOpaqueMeshes;
 
+	if (DrawTranslucent)
+		Context->BeginTranslucent();
+	else
+		Context->SetPolygonOffset(LC_POLYGON_OFFSET_OPAQUE);
+
 	for (int MeshIndex : Meshes)
 	{
 		const lcRenderMesh& RenderMesh = mRenderMeshes[MeshIndex];
@@ -94,17 +103,21 @@ void lcScene::DrawRenderMeshes(lcContext* Context, int PrimitiveTypes, bool Enab
 
 				switch (RenderMesh.State)
 				{
-				case LC_RENDERMESH_NONE:
-				case LC_RENDERMESH_HIGHLIGHT:
+				case lcRenderMeshState::NORMAL:
+				case lcRenderMeshState::HIGHLIGHT:
 					Context->SetColorIndex(ColorIndex);
 					break;
 
-				case LC_RENDERMESH_SELECTED:
-					Context->SetColorIndexTinted(ColorIndex, LC_COLOR_SELECTED);
+				case lcRenderMeshState::SELECTED:
+					Context->SetColorIndexTinted(ColorIndex, LC_COLOR_SELECTED, 0.5f);
 					break;
 
-				case LC_RENDERMESH_FOCUSED:
-					Context->SetColorIndexTinted(ColorIndex, LC_COLOR_FOCUSED);
+				case lcRenderMeshState::FOCUSED:
+					Context->SetColorIndexTinted(ColorIndex, LC_COLOR_FOCUSED, 0.5f);
+					break;
+
+				case lcRenderMeshState::DISABLED:
+					Context->SetColorIndexTinted(ColorIndex, LC_COLOR_DISABLED, 0.25f);
 					break;
 				}
 			}
@@ -112,24 +125,28 @@ void lcScene::DrawRenderMeshes(lcContext* Context, int PrimitiveTypes, bool Enab
 			{
 				switch (RenderMesh.State)
 				{
-				case LC_RENDERMESH_NONE:
+				case lcRenderMeshState::NORMAL:
 					if (ColorIndex == gEdgeColor)
 						Context->SetEdgeColorIndex(RenderMesh.ColorIndex);
 					else
 						Context->SetColorIndex(ColorIndex);
 					break;
 
-				case LC_RENDERMESH_SELECTED:
+				case lcRenderMeshState::SELECTED:
 					Context->SetInterfaceColor(LC_COLOR_SELECTED);
 					break;
 
-				case LC_RENDERMESH_FOCUSED:
+				case lcRenderMeshState::FOCUSED:
 					Context->SetInterfaceColor(LC_COLOR_FOCUSED);
 					break;
 
-				case LC_RENDERMESH_HIGHLIGHT:
+				case lcRenderMeshState::HIGHLIGHT:
 					Context->SetInterfaceColor(LC_COLOR_HIGHLIGHT);
 					break;
+
+				case lcRenderMeshState::DISABLED:
+					Context->SetInterfaceColor(LC_COLOR_DISABLED);
+					break;
 				}
 			}
 			else if (Section->PrimitiveType == LC_MESH_CONDITIONAL_LINES)
@@ -215,19 +232,25 @@ void lcScene::DrawRenderMeshes(lcContext* Context, int PrimitiveTypes, bool Enab
 		}
 #endif
 	}
+
+	if (DrawTranslucent)
+		Context->EndTranslucent();
 }
 
 void lcScene::Draw(lcContext* Context) const
 {
 	// TODO: find a better place for these updates
 	lcGetPiecesLibrary()->UpdateBuffers(Context);
-	lcGetPiecesLibrary()->UploadTextures();
+	lcGetPiecesLibrary()->UploadTextures(Context);
 
 	Context->SetViewMatrix(mViewMatrix);
 
 	const bool DrawConditional = false;
+	const lcPreferences& Preferences = lcGetPreferences();
 
-	lcShadingMode ShadingMode = lcGetPreferences().mShadingMode;
+	lcShadingMode ShadingMode = Preferences.mShadingMode;
+	if (ShadingMode == LC_SHADING_WIREFRAME && !mAllowWireframe)
+		ShadingMode = LC_SHADING_FLAT;
 
 	if (ShadingMode == LC_SHADING_WIREFRAME)
 	{
@@ -253,7 +276,7 @@ void lcScene::Draw(lcContext* Context) const
 	}
 	else if (ShadingMode == LC_SHADING_FLAT)
 	{
-		bool DrawLines = lcGetPreferences().mDrawEdgeLines;
+		bool DrawLines = Preferences.mDrawEdgeLines && Preferences.mLineWidth != 0.0f;
 		Context->BindTexture2D(0);
 
 		Context->SetMaterial(LC_MATERIAL_UNLIT_COLOR);
@@ -271,16 +294,8 @@ void lcScene::Draw(lcContext* Context) const
 		DrawRenderMeshes(Context, UntexturedPrimitives, false, false, false);
 
 		if (!mTranslucentMeshes.IsEmpty())
-		{
-			glEnable(GL_BLEND);
-			glDepthMask(GL_FALSE);
-
 			DrawRenderMeshes(Context, LC_MESH_TRIANGLES, false, true, false);
 
-			glDepthMask(GL_TRUE);
-			glDisable(GL_BLEND);
-		}
-
 		if (mHasTexture)
 		{
 			Context->SetMaterial(LC_MATERIAL_UNLIT_TEXTURE_DECAL);
@@ -291,22 +306,14 @@ void lcScene::Draw(lcContext* Context) const
 			DrawRenderMeshes(Context, LC_MESH_TEXTURED_TRIANGLES, false, false, true);
 
 			if (!mTranslucentMeshes.IsEmpty())
-			{
-				glEnable(GL_BLEND); // todo: remove GL calls
-				glDepthMask(GL_FALSE);
-
 				DrawRenderMeshes(Context, LC_MESH_TEXTURED_TRIANGLES, false, true, true);
 
-				glDepthMask(GL_TRUE);
-				glDisable(GL_BLEND);
-			}
-
 			Context->BindTexture2D(0);
 		}
 	}
 	else
 	{
-		bool DrawLines = lcGetPreferences().mDrawEdgeLines;
+		bool DrawLines = Preferences.mDrawEdgeLines && Preferences.mLineWidth != 0.0f;
 		Context->BindTexture2D(0);
 
 		if (DrawLines)
@@ -324,16 +331,8 @@ void lcScene::Draw(lcContext* Context) const
 		DrawRenderMeshes(Context, LC_MESH_TRIANGLES, true, false, false);
 
 		if (!mTranslucentMeshes.IsEmpty())
-		{
-			glEnable(GL_BLEND);
-			glDepthMask(GL_FALSE);
-
 			DrawRenderMeshes(Context, LC_MESH_TRIANGLES, true, true, false);
 
-			glDepthMask(GL_TRUE);
-			glDisable(GL_BLEND);
-		}
-
 		if (mHasTexture)
 		{
 			if (DrawLines)
@@ -346,16 +345,8 @@ void lcScene::Draw(lcContext* Context) const
 			DrawRenderMeshes(Context, LC_MESH_TEXTURED_TRIANGLES, true, false, true);
 
 			if (!mTranslucentMeshes.IsEmpty())
-			{
-				glEnable(GL_BLEND);
-				glDepthMask(GL_FALSE);
-
 				DrawRenderMeshes(Context, LC_MESH_TEXTURED_TRIANGLES, true, true, true);
 
-				glDepthMask(GL_TRUE);
-				glDisable(GL_BLEND);
-			}
-
 			Context->BindTexture2D(0);
 		}
 	}
@@ -364,5 +355,5 @@ void lcScene::Draw(lcContext* Context) const
 void lcScene::DrawInterfaceObjects(lcContext* Context) const
 {
 	for (const lcObject* Object : mInterfaceObjects)
-		Object->DrawInterface(Context);
+		Object->DrawInterface(Context, *this);
 }
