@@ -1,7 +1,6 @@
 #include "lc_global.h"
 #include "lc_timelinewidget.h"
 #include "lc_model.h"
-#include "project.h"
 #include "piece.h"
 #include "pieceinf.h"
 #include "lc_mainwindow.h"
@@ -32,7 +31,7 @@ void lcTimelineWidget::CustomMenuRequested(QPoint Pos)
 {
 	QMenu* Menu = new QMenu(this);
 
-	lcObject* FocusObject = lcGetActiveModel()->GetFocusObject();
+	lcObject* FocusObject = gMainWindow->GetActiveModel()->GetFocusObject();
 
 	if (FocusObject && FocusObject->IsPiece())
 	{
@@ -40,7 +39,9 @@ void lcTimelineWidget::CustomMenuRequested(QPoint Pos)
 
 		if (Piece->mPieceInfo->IsModel())
 		{
-			Menu->addAction(gMainWindow->mActions[LC_MODEL_EDIT_FOCUS]);
+			Menu->addAction(gMainWindow->mActions[LC_PIECE_EDIT_SELECTED_SUBMODEL]);
+			Menu->addAction(gMainWindow->mActions[LC_PIECE_VIEW_SELECTED_MODEL]);
+			Menu->addAction(gMainWindow->mActions[LC_PIECE_INLINE_SELECTED_MODELS]);
 			Menu->addSeparator();
 		}
 	}
@@ -57,7 +58,8 @@ void lcTimelineWidget::CustomMenuRequested(QPoint Pos)
 	Menu->addAction(gMainWindow->mActions[LC_PIECE_UNHIDE_SELECTED]);
 	Menu->addAction(gMainWindow->mActions[LC_PIECE_UNHIDE_ALL]);
 
-	Menu->popup(viewport()->mapToGlobal(Pos));
+	Menu->exec(viewport()->mapToGlobal(Pos));
+	delete Menu;
 }
 
 void lcTimelineWidget::Update(bool Clear, bool UpdateItems)
@@ -65,17 +67,18 @@ void lcTimelineWidget::Update(bool Clear, bool UpdateItems)
 	if (mIgnoreUpdates)
 		return;
 
-	lcModel* Model = lcGetActiveModel();
+	lcModel* Model = gMainWindow->GetActiveModel();
+
+	bool Blocked = blockSignals(true);
 
 	if (!Model)
 	{
 		mItems.clear();
 		clear();
+		blockSignals(Blocked);
 		return;
 	}
 
-	bool Blocked = blockSignals(true);
-
 	if (Clear)
 	{
 		mItems.clear();
@@ -112,10 +115,8 @@ void lcTimelineWidget::Update(bool Clear, bool UpdateItems)
 	int PieceItemIndex = 0;
 	lcStep Step = 0;
 
-	for (int PieceIdx = 0; PieceIdx != Pieces.GetSize(); PieceIdx++)
+	for (lcPiece* Piece : Pieces)
 	{
-		lcPiece* Piece = Pieces[PieceIdx];
-
 		while (Step != Piece->GetStepShow())
 		{
 			if (StepItem)
@@ -275,7 +276,7 @@ void lcTimelineWidget::InsertStep()
 	if (Step == -1)
 		return;
 
-	lcGetActiveModel()->InsertStep(Step + 1);
+	gMainWindow->GetActiveModel()->InsertStep(Step + 1);
 }
 
 void lcTimelineWidget::RemoveStep()
@@ -293,7 +294,7 @@ void lcTimelineWidget::RemoveStep()
 	if (Step == -1)
 		return;
 
-	lcGetActiveModel()->RemoveStep(Step + 1);
+	gMainWindow->GetActiveModel()->RemoveStep(Step + 1);
 }
 
 void lcTimelineWidget::MoveSelection()
@@ -313,7 +314,7 @@ void lcTimelineWidget::MoveSelection()
 
 	QList<QTreeWidgetItem*> SelectedItems = selectedItems();
 
-	foreach(QTreeWidgetItem* PieceItem, SelectedItems)
+	for (QTreeWidgetItem* PieceItem : SelectedItems)
 	{
 		QTreeWidgetItem* Parent = PieceItem->parent();
 
@@ -342,15 +343,16 @@ void lcTimelineWidget::SetCurrentStep()
 	if (Step == -1)
 		return;
 
-	lcGetActiveModel()->SetCurrentStep(Step + 1);
+	gMainWindow->GetActiveModel()->SetCurrentStep(Step + 1);
 }
 
 void lcTimelineWidget::ItemSelectionChanged()
 {
 	lcArray<lcObject*> Selection;
 	lcStep LastStep = 1;
+	QList<QTreeWidgetItem*> SelectedItems = selectedItems();
 
-	foreach (QTreeWidgetItem* PieceItem, selectedItems())
+	for (QTreeWidgetItem* PieceItem : SelectedItems)
 	{
 		lcPiece* Piece = (lcPiece*)PieceItem->data(0, Qt::UserRole).value<uintptr_t>();
 		if (Piece)
@@ -367,7 +369,7 @@ void lcTimelineWidget::ItemSelectionChanged()
 
 	bool Blocked = blockSignals(true);
 	mIgnoreUpdates = true;
-	lcModel* Model = lcGetActiveModel();
+	lcModel* Model = gMainWindow->GetActiveModel();
 	if (LastStep > Model->GetCurrentStep())
 		Model->SetCurrentStep(LastStep);
 	Model->SetSelectionAndFocus(Selection, CurrentPiece, LC_PIECE_SECTION_POSITION, false);
@@ -415,6 +417,6 @@ void lcTimelineWidget::UpdateModel()
 	}
 
 	mIgnoreUpdates = true;
-	lcGetActiveModel()->SetPieceSteps(PieceSteps);
+	gMainWindow->GetActiveModel()->SetPieceSteps(PieceSteps);
 	mIgnoreUpdates = false;
 }
