@@ -2,7 +2,6 @@
 
 #include "lc_glwidget.h"
 #include "lc_math.h"
-#include "lc_array.h"
 
 enum LC_MFW_TYPES
 {
@@ -41,12 +40,27 @@ struct lcMinifig
 	lcMatrix44 Matrices[LC_MFW_NUMITEMS];
 };
 
+struct lcMinifigTemplate
+{
+	QString Parts[LC_MFW_NUMITEMS];
+	int Colors[LC_MFW_NUMITEMS];
+	float Angles[LC_MFW_NUMITEMS];
+};
+
 class MinifigWizard : public lcGLWidget
 {
 public:
 	MinifigWizard();
 	~MinifigWizard();
 
+	const std::map<QString, lcMinifigTemplate>& GetTemplates() const
+	{
+		return mTemplates;
+	}
+
+	void SaveTemplate(const QString& TemplateName, const lcMinifigTemplate& Template);
+	void DeleteTemplate(const QString& TemplateName);
+
 	void OnDraw();
 	void OnLeftButtonDown();
 	void OnLeftButtonUp();
@@ -64,7 +78,7 @@ public:
 
 	void ParseSettings(lcFile& Settings);
 
-	lcArray<lcMinifigPieceInfo> mSettings[LC_MFW_NUMITEMS];
+	std::vector<lcMinifigPieceInfo> mSettings[LC_MFW_NUMITEMS];
 
 	lcMinifig mMinifig;
 
@@ -76,5 +90,13 @@ public:
 	float mRotateX;
 	float mRotateZ;
 	bool mAutoZoom;
+
+protected:
+	void LoadSettings();
+	void LoadTemplates();
+	void SaveTemplates();
+
+	std::map<QString, lcMinifigTemplate> mTemplates;
+	static const char* mSectionNames[LC_MFW_NUMITEMS];
 };
 
