@@ -30,9 +30,6 @@ lcPiece::lcPiece(PieceInfo* Info)
 	mGroup = nullptr;
 	mFileLine = -1;
 	mPivotMatrix = lcMatrix44Identity();
-
-	ChangeKey(mPositionKeys, lcVector3(0.0f, 0.0f, 0.0f), 1, true);
-	ChangeKey(mRotationKeys, lcMatrix33Identity(), 1, true);
 }
 
 lcPiece::lcPiece(const lcPiece& Other)
@@ -390,8 +387,10 @@ void lcPiece::Initialize(const lcMatrix44& WorldMatrix, lcStep Step)
 {
 	mStepShow = Step;
 
-	ChangeKey(mPositionKeys, WorldMatrix.GetTranslation(), 1, true);
-	ChangeKey(mRotationKeys, lcMatrix33(WorldMatrix), 1, true);
+	if (mPositionKeys.IsEmpty())
+		ChangeKey(mPositionKeys, WorldMatrix.GetTranslation(), 1, true);
+	if (mRotationKeys.IsEmpty())
+		ChangeKey(mRotationKeys, lcMatrix33(WorldMatrix), 1, true);
 
 	UpdatePosition(Step);
 }
@@ -510,7 +509,7 @@ void lcPiece::BoxTest(lcObjectBoxTest& ObjectBoxTest) const
 		ObjectBoxTest.Objects.Add(const_cast<lcPiece*>(this));
 }
 
-void lcPiece::DrawInterface(lcContext* Context) const
+void lcPiece::DrawInterface(lcContext* Context, const lcScene& Scene) const
 {
 	float LineWidth = lcGetPreferences().mLineWidth;
 	Context->SetLineWidth(2.0f * LineWidth);
@@ -555,8 +554,9 @@ void lcPiece::DrawInterface(lcContext* Context) const
 		{ Min[0], Min[1], Min[2] }, { Min[0], Min[1], Min[2] + Edge[2] },
 	};
 
+	lcMatrix44 WorldMatrix = Scene.ApplyActiveSubmodelTransform(mModelWorld);
 	Context->SetMaterial(LC_MATERIAL_UNLIT_COLOR);
-	Context->SetWorldMatrix(mModelWorld);
+	Context->SetWorldMatrix(WorldMatrix);
 
 	if (IsFocused(LC_PIECE_SECTION_POSITION))
 		Context->SetInterfaceColor(LC_COLOR_FOCUSED);
@@ -582,7 +582,7 @@ void lcPiece::DrawInterface(lcContext* Context) const
 			0, 1, 1, 2, 2, 3, 3, 0, 4, 5, 5, 6, 6, 7, 7, 4, 0, 4, 1, 5, 2, 6, 3, 7
 		};
 
-		Context->SetWorldMatrix(lcMul(mPivotMatrix, mModelWorld));
+		Context->SetWorldMatrix(lcMul(mPivotMatrix, WorldMatrix));
 
 		Context->SetVertexBufferPointer(Verts);
 		Context->SetVertexFormatPosition(3);
@@ -596,17 +596,17 @@ void lcPiece::DrawInterface(lcContext* Context) const
 		float Verts[8 * 3];
 		float* CurVert = Verts;
 
-		lcVector3 Min(-LC_PIECE_CONTROL_POINT_SIZE, -LC_PIECE_CONTROL_POINT_SIZE, -LC_PIECE_CONTROL_POINT_SIZE);
-		lcVector3 Max(LC_PIECE_CONTROL_POINT_SIZE, LC_PIECE_CONTROL_POINT_SIZE, LC_PIECE_CONTROL_POINT_SIZE);
+		lcVector3 CubeMin(-LC_PIECE_CONTROL_POINT_SIZE, -LC_PIECE_CONTROL_POINT_SIZE, -LC_PIECE_CONTROL_POINT_SIZE);
+		lcVector3 CubeMax(LC_PIECE_CONTROL_POINT_SIZE, LC_PIECE_CONTROL_POINT_SIZE, LC_PIECE_CONTROL_POINT_SIZE);
 
-		*CurVert++ = Min[0]; *CurVert++ = Min[1]; *CurVert++ = Min[2];
-		*CurVert++ = Min[0]; *CurVert++ = Max[1]; *CurVert++ = Min[2];
-		*CurVert++ = Max[0]; *CurVert++ = Max[1]; *CurVert++ = Min[2];
-		*CurVert++ = Max[0]; *CurVert++ = Min[1]; *CurVert++ = Min[2];
-		*CurVert++ = Min[0]; *CurVert++ = Min[1]; *CurVert++ = Max[2];
-		*CurVert++ = Min[0]; *CurVert++ = Max[1]; *CurVert++ = Max[2];
-		*CurVert++ = Max[0]; *CurVert++ = Max[1]; *CurVert++ = Max[2];
-		*CurVert++ = Max[0]; *CurVert++ = Min[1]; *CurVert++ = Max[2];
+		*CurVert++ = CubeMin[0]; *CurVert++ = CubeMin[1]; *CurVert++ = CubeMin[2];
+		*CurVert++ = CubeMin[0]; *CurVert++ = CubeMax[1]; *CurVert++ = CubeMin[2];
+		*CurVert++ = CubeMax[0]; *CurVert++ = CubeMax[1]; *CurVert++ = CubeMin[2];
+		*CurVert++ = CubeMax[0]; *CurVert++ = CubeMin[1]; *CurVert++ = CubeMin[2];
+		*CurVert++ = CubeMin[0]; *CurVert++ = CubeMin[1]; *CurVert++ = CubeMax[2];
+		*CurVert++ = CubeMin[0]; *CurVert++ = CubeMax[1]; *CurVert++ = CubeMax[2];
+		*CurVert++ = CubeMax[0]; *CurVert++ = CubeMax[1]; *CurVert++ = CubeMax[2];
+		*CurVert++ = CubeMax[0]; *CurVert++ = CubeMin[1]; *CurVert++ = CubeMax[2];
 
 		const GLushort Indices[36] =
 		{
@@ -619,7 +619,7 @@ void lcPiece::DrawInterface(lcContext* Context) const
 
 		for (int ControlPointIdx = 0; ControlPointIdx < mControlPoints.GetSize(); ControlPointIdx++)
 		{
-			Context->SetWorldMatrix(lcMul(mControlPoints[ControlPointIdx].Transform, mModelWorld));
+			Context->SetWorldMatrix(lcMul(mControlPoints[ControlPointIdx].Transform, WorldMatrix));
 
 			Context->SetVertexBufferPointer(Verts);
 			Context->SetVertexFormatPosition(3);
@@ -638,41 +638,64 @@ void lcPiece::DrawInterface(lcContext* Context) const
 	}
 }
 
-void lcPiece::AddRenderMeshes(lcScene& Scene, bool DrawInterface, bool Highlight) const
+void lcPiece::RemoveKeyFrames()
 {
-	bool Focused, Selected;
+	mPositionKeys.RemoveAll();
+	ChangeKey(mPositionKeys, mModelWorld.GetTranslation(), 1, true);
 
-	if (DrawInterface)
+	mRotationKeys.RemoveAll();
+	ChangeKey(mRotationKeys, lcMatrix33(mModelWorld), 1, true);
+}
+
+void lcPiece::AddMainModelRenderMeshes(lcScene& Scene, bool Highlight) const
+{
+	lcRenderMeshState RenderMeshState = lcRenderMeshState::NORMAL;
+	bool ParentActive = false;
+
+	if (Scene.GetDrawInterface())
 	{
-		Focused = IsFocused();
-		Selected = IsSelected();
-	}
-	else
-	{
-		Focused = false;
-		Selected = false;
+		lcPiece* ActiveSubmodelInstance = Scene.GetActiveSubmodelInstance();
+
+		if (!ActiveSubmodelInstance)
+			RenderMeshState = IsFocused() ? lcRenderMeshState::FOCUSED : (IsSelected() ? lcRenderMeshState::SELECTED : lcRenderMeshState::NORMAL);
+		else if (ActiveSubmodelInstance == this)
+			ParentActive = true;
+		else
+			RenderMeshState = lcRenderMeshState::DISABLED;
 	}
+	else if (Highlight)
+		RenderMeshState = lcRenderMeshState::HIGHLIGHT;
 
 	if (!mMesh)
-		mPieceInfo->AddRenderMeshes(Scene, mModelWorld, mColorIndex, Focused, Selected, Highlight);
+		mPieceInfo->AddRenderMeshes(Scene, mModelWorld, mColorIndex, RenderMeshState, ParentActive);
 	else
-		Scene.AddMesh(mMesh, mModelWorld, mColorIndex, Focused ? LC_RENDERMESH_FOCUSED : (Selected ? LC_RENDERMESH_SELECTED : LC_RENDERMESH_NONE), mPieceInfo->mFlags);
+		Scene.AddMesh(mMesh, mModelWorld, mColorIndex, RenderMeshState, mPieceInfo->mFlags);
 
-	if (Selected)
+	if (RenderMeshState == lcRenderMeshState::FOCUSED || RenderMeshState == lcRenderMeshState::SELECTED)
 		Scene.AddInterfaceObject(this);
 }
 
-void lcPiece::SubModelAddRenderMeshes(lcScene& Scene, const lcMatrix44& WorldMatrix, int DefaultColorIndex, bool Focused, bool Selected) const
+void lcPiece::AddSubModelRenderMeshes(lcScene& Scene, const lcMatrix44& WorldMatrix, int DefaultColorIndex, lcRenderMeshState RenderMeshState, bool ParentActive) const
 {
 	int ColorIndex = mColorIndex;
 
 	if (ColorIndex == gDefaultColor)
 		ColorIndex = DefaultColorIndex;
 
+	lcPiece* ActiveSubmodelInstance = Scene.GetActiveSubmodelInstance();
+
+	if (ActiveSubmodelInstance == this)
+		RenderMeshState = lcRenderMeshState::NORMAL;
+	else if (ParentActive)
+		RenderMeshState = IsFocused() ? lcRenderMeshState::FOCUSED : (IsSelected() ? lcRenderMeshState::SELECTED : lcRenderMeshState::NORMAL);
+
 	if (!mMesh)
-		mPieceInfo->AddRenderMeshes(Scene, lcMul(mModelWorld, WorldMatrix), ColorIndex, Focused, Selected, false);
+		mPieceInfo->AddRenderMeshes(Scene, lcMul(mModelWorld, WorldMatrix), ColorIndex, RenderMeshState, ActiveSubmodelInstance == this);
 	else
-		Scene.AddMesh(mMesh, lcMul(mModelWorld, WorldMatrix), ColorIndex, Focused ? LC_RENDERMESH_FOCUSED : (Selected ? LC_RENDERMESH_SELECTED : LC_RENDERMESH_NONE), mPieceInfo->mFlags);
+		Scene.AddMesh(mMesh, lcMul(mModelWorld, WorldMatrix), ColorIndex, RenderMeshState, mPieceInfo->mFlags);
+
+	if (ParentActive && (RenderMeshState == lcRenderMeshState::FOCUSED || RenderMeshState == lcRenderMeshState::SELECTED))
+		Scene.AddInterfaceObject(this);
 }
 
 void lcPiece::MoveSelected(lcStep Step, bool AddKey, const lcVector3& Distance)
@@ -827,7 +850,7 @@ const char* lcPiece::GetName() const
 	return mPieceInfo->m_strDescription;
 }
 
-bool lcPiece::IsVisible(lcStep Step)
+bool lcPiece::IsVisible(lcStep Step) const
 {
 	if (mState & LC_PIECE_HIDDEN)
 		return false;
@@ -835,6 +858,27 @@ bool lcPiece::IsVisible(lcStep Step)
 	return (mStepShow <= Step) && (mStepHide > Step || mStepHide == LC_STEP_MAX);
 }
 
+bool lcPiece::IsVisibleInSubModel() const
+{
+	return (mStepHide == LC_STEP_MAX) && !(mState & LC_PIECE_HIDDEN);
+}
+
+void lcPiece::GetModelParts(const lcMatrix44& WorldMatrix, int DefaultColorIndex, std::vector<lcModelPartsEntry>& ModelParts) const
+{
+	if (!IsVisibleInSubModel())
+		return;
+
+	int ColorIndex = mColorIndex;
+
+	if (ColorIndex == gDefaultColor)
+		ColorIndex = DefaultColorIndex;
+
+	if (!mMesh)
+		mPieceInfo->GetModelParts(lcMul(mModelWorld, WorldMatrix), ColorIndex, ModelParts);
+	else
+		ModelParts.emplace_back(lcModelPartsEntry{ WorldMatrix, mPieceInfo, mMesh, DefaultColorIndex });
+}
+
 const lcBoundingBox& lcPiece::GetBoundingBox() const
 {
 	if (!mMesh)
