@@ -22,6 +22,14 @@ enum lcPieceInfoState
 	LC_PIECEINFO_LOADED
 };
 
+struct lcModelPartsEntry
+{
+	lcMatrix44 WorldMatrix;
+	const PieceInfo* Info;
+	lcMesh* Mesh;
+	int ColorIndex;
+};
+
 class lcSynthInfo;
 
 class PieceInfo
@@ -136,19 +144,20 @@ public:
 
 	void ZoomExtents(float FoV, float AspectRatio, lcMatrix44& ProjectionMatrix, lcMatrix44& ViewMatrix) const;
 	void AddRenderMesh(lcScene& Scene);
-	void AddRenderMeshes(lcScene& Scene, const lcMatrix44& WorldMatrix, int ColorIndex, bool Focused, bool Selected, bool Highlight) const;
+	void AddRenderMeshes(lcScene& Scene, const lcMatrix44& WorldMatrix, int ColorIndex, lcRenderMeshState RenderMeshState, bool ParentActive) const;
 
 	void CreatePlaceholder(const char* Name);
 
 	void SetPlaceholder();
 	void SetModel(lcModel* Model, bool UpdateMesh, Project* CurrentProject, bool SearchProjectFolder);
 	void CreateProject(Project* Project, const char* PieceName);
+	bool GetPieceWorldMatrix(lcPiece* Piece, lcMatrix44& WorldMatrix) const;
 	bool IncludesModel(const lcModel* Model) const;
 	bool MinIntersectDist(const lcVector3& Start, const lcVector3& End, float& MinDistance) const;
 	bool BoxTest(const lcMatrix44& WorldMatrix, const lcVector4 Planes[6]) const;
 	void GetPartsList(int DefaultColorIndex, bool IncludeSubmodels, lcPartsList& PartsList) const;
-	void GetModelParts(const lcMatrix44& WorldMatrix, int DefaultColorIndex, lcArray<lcModelPartsEntry>& ModelParts) const;
-	void UpdateBoundingBox(lcArray<lcModel*>& UpdatedModels);
+	void GetModelParts(const lcMatrix44& WorldMatrix, int DefaultColorIndex, std::vector<lcModelPartsEntry>& ModelParts) const;
+	void UpdateBoundingBox(std::vector<lcModel*>& UpdatedModels);
 
 	void Load();
 	void Unload();
