@@ -2,7 +2,6 @@
 #include "lc_math.h"
 #include "lc_mesh.h"
 #include <locale.h>
-#include <array>
 #include "pieceinf.h"
 #include "camera.h"
 #include "project.h"
@@ -121,9 +120,9 @@ QString Project::GetTitle() const
 	return mModels.GetSize() == 1 ? tr("New Model.ldr") : tr("New Model.mpd");
 }
 
-QString Project::GetImageFileName() const
+QString Project::GetImageFileName(bool AllowCurrentFolder) const
 {
-	QString FileName = QDir::toNativeSeparators(GetFileName());
+	QString FileName = GetFileName();
 
 	if (!FileName.isEmpty())
 	{
@@ -132,9 +131,22 @@ QString Project::GetImageFileName() const
 			FileName = FileName.left(FileName.length() - Extension.length() - 1);
 	}
 	else
-		FileName = QLatin1String("image");
+	{
+		if (AllowCurrentFolder)
+			FileName = QLatin1String("image");
+		else
+		{
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
+			QStringList cachePathList = QStandardPaths::standardLocations(QStandardPaths::DocumentsLocation);
+			FileName = cachePathList.first();
+#else
+			FileName = QDesktopServices::storageLocation(QDesktopServices::DocumentsLocation);
+#endif
+			FileName = QDir(FileName).absoluteFilePath(QLatin1String("image"));
+		}
+	}
 
-	return FileName + lcGetProfileString(LC_PROFILE_IMAGE_EXTENSION);
+	return QDir::toNativeSeparators(FileName) + lcGetProfileString(LC_PROFILE_IMAGE_EXTENSION);
 }
 
 void Project::SetActiveModel(int ModelIndex)
@@ -145,8 +157,8 @@ void Project::SetActiveModel(int ModelIndex)
 	for (int ModelIdx = 0; ModelIdx < mModels.GetSize(); ModelIdx++)
 		mModels[ModelIdx]->SetActive(ModelIdx == ModelIndex);
 
-	lcArray<lcModel*> UpdatedModels;
-	UpdatedModels.AllocGrow(mModels.GetSize());
+	std::vector<lcModel*> UpdatedModels;
+	UpdatedModels.reserve(mModels.GetSize());
 
 	for (int ModelIdx = 0; ModelIdx < mModels.GetSize(); ModelIdx++)
 		mModels[ModelIdx]->UpdatePieceInfo(UpdatedModels);
@@ -179,11 +191,11 @@ QString Project::GetNewModelName(QWidget* ParentWidget, const QString& DialogTit
 
 		for (int ModelIdx = 0; ModelIdx < ExistingModels.size(); ModelIdx++)
 		{
-			const QString& Name = ExistingModels[ModelIdx];
+			const QString& ExistingName = ExistingModels[ModelIdx];
 
-			if (Name.startsWith(Prefix))
+			if (ExistingName.startsWith(Prefix))
 			{
-				QString NumberString = Name.mid(Prefix.length());
+				QString NumberString = ExistingName.mid(Prefix.length());
 				QTextStream Stream(&NumberString);
 				int Number;
 				Stream >> Number;
@@ -322,12 +334,19 @@ void Project::ShowModelListDialog()
 
 	mModels = NewModels;
 
-	SetActiveModel(Dialog.mActiveModel);
 	gMainWindow->UpdateTitle();
+	gMainWindow->UpdateModels();
+
+	int ModelIndex = Dialog.GetActiveModelIndex();
+	if (ModelIndex != -1)
+		SetActiveModel(ModelIndex);
 }
 
 void Project::SetFileName(const QString& FileName)
 {
+	if (mFileName == FileName)
+		return;
+
 	if (!mFileName.isEmpty())
 		mFileWatcher.removePath(mFileName);
 
@@ -360,7 +379,7 @@ bool Project::Load(const QString& FileName)
 	else if (Extension == QLatin1String("lcd") || Extension == QLatin1String("leocad"))
 		LoadDAT = false;
 	else
-		LoadDAT = memcmp(FileData, "LeoCAD ", 7);
+		LoadDAT = FileData.size() > 7 && memcmp(FileData, "LeoCAD ", 7);
 
 	if (LoadDAT)
 	{
@@ -409,7 +428,10 @@ bool Project::Load(const QString& FileName)
 	}
 
 	if (mModels.IsEmpty())
+	{
+		QMessageBox::warning(gMainWindow, tr("Error"), tr("Error loading file '%1':\nFile format is not recognized.").arg(FileName));
 		return false;
+	}
 
 	if (mModels.GetSize() == 1)
 	{
@@ -422,8 +444,8 @@ bool Project::Load(const QString& FileName)
 		}
 	}
 
-	lcArray<lcModel*> UpdatedModels;
-	UpdatedModels.AllocGrow(mModels.GetSize());
+	std::vector<lcModel*> UpdatedModels;
+	UpdatedModels.reserve(mModels.GetSize());
 
 	for (lcModel* Model : mModels)
 	{
@@ -452,8 +474,11 @@ bool Project::Save(const QString& FileName)
 	bool Success = Save(Stream);
 	File.close();
 
-	SetFileName(FileName);
-	mModified = false;
+	if (Success)
+	{
+		SetFileName(FileName);
+		mModified = false;
+	}
 
 	return Success;
 }
@@ -462,10 +487,8 @@ bool Project::Save(QTextStream& Stream)
 {
 	bool MPD = mModels.GetSize() > 1;
 
-	for (int ModelIdx = 0; ModelIdx < mModels.GetSize(); ModelIdx++)
+	for (lcModel* Model : mModels)
 	{
-		lcModel* Model = mModels[ModelIdx];
-
 		if (MPD)
 			Stream << QLatin1String("0 FILE ") << Model->GetProperties().mName << QLatin1String("\r\n");
 
@@ -481,9 +504,8 @@ bool Project::Save(QTextStream& Stream)
 
 void Project::Merge(Project* Other)
 {
-	for (int ModelIdx = 0; ModelIdx < Other->mModels.GetSize(); ModelIdx++)
+	for (lcModel* Model : Other->mModels)
 	{
-		lcModel* Model = Other->mModels[ModelIdx];
 		QString Name = Model->GetProperties().mName;
 
 		for (;;)
@@ -509,6 +531,7 @@ void Project::Merge(Project* Other)
 		mModels.Add(Model);
 	}
 
+	Other->mModels.RemoveAll();
 	mModified = true;
 }
 
@@ -532,24 +555,19 @@ bool Project::ImportLDD(const QString& FileName)
 		Model->SetSaved();
 	}
 	else
-		delete Model;
-
-	if (mModels.IsEmpty())
-		return false;
-
-	if (mModels.GetSize() == 1)
 	{
-		lcModel* Model = mModels[0];
-
-		if (Model->GetProperties().mName.isEmpty())
-			Model->SetName(QFileInfo(FileName).completeBaseName());
+		delete Model;
+		return false;
 	}
 
+	if (Model->GetProperties().mName.isEmpty())
+		Model->SetName(QFileInfo(FileName).completeBaseName());
+
 	for (int ModelIdx = 0; ModelIdx < mModels.GetSize(); ModelIdx++)
 		mModels[ModelIdx]->CreatePieceInfo(this);
 
-	lcArray<lcModel*> UpdatedModels;
-	UpdatedModels.AllocGrow(mModels.GetSize());
+	std::vector<lcModel*> UpdatedModels;
+	UpdatedModels.reserve(mModels.GetSize());
 
 	for (int ModelIdx = 0; ModelIdx < mModels.GetSize(); ModelIdx++)
 		mModels[ModelIdx]->UpdatePieceInfo(UpdatedModels);
@@ -573,24 +591,19 @@ bool Project::ImportInventory(const QByteArray& Inventory, const QString& Name,
 		Model->SetSaved();
 	}
 	else
-		delete Model;
-
-	if (mModels.IsEmpty())
-		return false;
-
-	if (mModels.GetSize() == 1)
 	{
-		lcModel* Model = mModels[0];
-
-		Model->SetName(Name);
-		Model->SetDescription(Description);
+		delete Model;
+		return false;
 	}
 
+	Model->SetName(Name);
+	Model->SetDescription(Description);
+
 	for (int ModelIdx = 0; ModelIdx < mModels.GetSize(); ModelIdx++)
 		mModels[ModelIdx]->CreatePieceInfo(this);
 
-	lcArray<lcModel*> UpdatedModels;
-	UpdatedModels.AllocGrow(mModels.GetSize());
+	std::vector<lcModel*> UpdatedModels;
+	UpdatedModels.reserve(mModels.GetSize());
 
 	for (int ModelIdx = 0; ModelIdx < mModels.GetSize(); ModelIdx++)
 		mModels[ModelIdx]->UpdatePieceInfo(UpdatedModels);
@@ -600,20 +613,24 @@ bool Project::ImportInventory(const QByteArray& Inventory, const QString& Name,
 	return true;
 }
 
-void Project::GetModelParts(lcArray<lcModelPartsEntry>& ModelParts)
+std::vector<lcModelPartsEntry> Project::GetModelParts()
 {
-	if (mModels.IsEmpty())
-		return;
+	std::vector<lcModelPartsEntry> ModelParts;
 
-	for (int ModelIdx = 0; ModelIdx < mModels.GetSize(); ModelIdx++)
-		mModels[ModelIdx]->CalculateStep(LC_STEP_MAX);
+	if (mModels.IsEmpty())
+		return ModelParts;
+
+	for (lcModel* Model : mModels)
+		Model->CalculateStep(LC_STEP_MAX);
 
 	mModels[0]->GetModelParts(lcMatrix44Identity(), gDefaultColor, ModelParts);
 
 	SetActiveModel(mModels.FindIndex(mActiveModel));
+
+	return ModelParts;
 }
 
-bool Project::ExportModel(const QString& FileName, lcModel* Model)
+bool Project::ExportModel(const QString& FileName, lcModel* Model) const
 {
 	QFile File(FileName);
 
@@ -657,11 +674,9 @@ QString Project::GetExportFileName(const QString& FileName, const QString& Defau
 
 void Project::Export3DStudio(const QString& FileName)
 {
-	lcArray<lcModelPartsEntry> ModelParts;
+	std::vector<lcModelPartsEntry> ModelParts = GetModelParts();
 
-	GetModelParts(ModelParts);
-
-	if (ModelParts.IsEmpty())
+	if (ModelParts.empty())
 	{
 		QMessageBox::information(gMainWindow, tr("LeoCAD"), tr("Nothing to export."));
 		return;
@@ -699,11 +714,11 @@ void Project::Export3DStudio(const QString& FileName)
 	const int MaterialNameLength = 11;
 	char MaterialName[32];
 
-	for (int ColorIdx = 0; ColorIdx < gColorList.GetSize(); ColorIdx++)
+	for (size_t ColorIdx = 0; ColorIdx < gColorList.size(); ColorIdx++)
 	{
 		lcColor* Color = &gColorList[ColorIdx];
 
-		sprintf(MaterialName, "Material%03d", ColorIdx);
+		sprintf(MaterialName, "Material%03d", (int)ColorIdx);
 
 		long MaterialStart = File.GetPosition();
 		File.WriteU16(0xAFFF); // CHK_MAT_ENTRY
@@ -948,10 +963,9 @@ void Project::Export3DStudio(const QString& FileName)
 	}
 
 	int NumPieces = 0;
-	for (int PartIdx = 0; PartIdx < ModelParts.GetSize(); PartIdx++)
+	for (const lcModelPartsEntry& ModelPart : ModelParts)
 	{
-		PieceInfo* Info = ModelParts[PartIdx].Info;
-		lcMesh* Mesh = Info->GetMesh();
+		lcMesh* Mesh = !ModelPart.Mesh ? ModelPart.Info->GetMesh() : ModelPart.Mesh;
 
 		if (!Mesh || Mesh->mIndexType == GL_UNSIGNED_INT)
 			continue;
@@ -975,7 +989,7 @@ void Project::Export3DStudio(const QString& FileName)
 		File.WriteU16(Mesh->mNumVertices);
 
 		lcVertex* Verts = (lcVertex*)Mesh->mVertexData;
-		const lcMatrix44& ModelWorld = ModelParts[PartIdx].WorldMatrix;
+		const lcMatrix44& ModelWorld = ModelPart.WorldMatrix;
 
 		for (int VertexIdx = 0; VertexIdx < Mesh->mNumVertices; VertexIdx++)
 		{
@@ -1044,7 +1058,7 @@ void Project::Export3DStudio(const QString& FileName)
 			if (Section->PrimitiveType != LC_MESH_TRIANGLES && Section->PrimitiveType != LC_MESH_TEXTURED_TRIANGLES)
 				continue;
 
-			int MaterialIndex = Section->ColorIndex == gDefaultColor ? ModelParts[PartIdx].ColorIndex : Section->ColorIndex;
+			int MaterialIndex = Section->ColorIndex == gDefaultColor ? ModelPart.ColorIndex : Section->ColorIndex;
 
 			File.WriteU16(0x4130); // CHK_MSH_MAT_GROUP
 			File.WriteU32(6 + MaterialNameLength + 1 + 2 + 2 * Section->NumIndices / 3);
@@ -1148,12 +1162,8 @@ void Project::ExportBrickLink()
 			sprintf(Line, "    <ITEMID>%s</ITEMID>\n", FileName);
 			BrickLinkFile.WriteLine(Line);
 
-			int Count = ColorIt.second;
-			if (Count > 1)
-			{
-				sprintf(Line, "    <MINQTY>%d</MINQTY>\n", Count);
-				BrickLinkFile.WriteLine(Line);
-			}
+			sprintf(Line, "    <MINQTY>%d</MINQTY>\n", ColorIt.second);
+			BrickLinkFile.WriteLine(Line);
 
 			int Color = lcGetBrickLinkColor(ColorIt.first);
 			if (Color)
@@ -1171,11 +1181,9 @@ void Project::ExportBrickLink()
 
 void Project::ExportCOLLADA(const QString& FileName)
 {
-	lcArray<lcModelPartsEntry> ModelParts;
+	std::vector<lcModelPartsEntry> ModelParts = GetModelParts();
 
-	GetModelParts(ModelParts);
-
-	if (ModelParts.IsEmpty())
+	if (ModelParts.empty())
 	{
 		QMessageBox::information(gMainWindow, tr("LeoCAD"), tr("Nothing to export."));
 		return;
@@ -1256,18 +1264,27 @@ void Project::ExportCOLLADA(const QString& FileName)
 
 	Stream << "</library_materials>\r\n";
 	Stream << "<library_geometries>\r\n";
-	QSet<PieceInfo*> AddedPieces;
+	std::set<lcMesh*> AddedMeshes;
 
-	for (const lcModelPartsEntry& Entry : ModelParts)
+	auto GetMeshID = [](const lcModelPartsEntry& ModelPart)
 	{
-		PieceInfo* Info = Entry.Info;
-
-		if (AddedPieces.contains(Info))
-			continue;
-		AddedPieces.insert(Info);
-
+		const PieceInfo* Info = ModelPart.Info;
 		QString ID = QString(Info->mFileName).replace('.', '_');
-		lcMesh* Mesh = Info->GetMesh();
+
+		if (ModelPart.Mesh)
+			ID += "_" + QString::number((quintptr)ModelPart.Mesh, 16);
+
+		return ID;
+	};
+
+	for (const lcModelPartsEntry& ModelPart : ModelParts)
+	{
+		lcMesh* Mesh = !ModelPart.Mesh ? ModelPart.Info->GetMesh() : ModelPart.Mesh;
+
+		if (!AddedMeshes.insert(Mesh).second)
+			continue;
+
+		QString ID = GetMeshID(ModelPart);
 
 		if (!Mesh)
 			Mesh = gPlaceholderMesh;
@@ -1377,15 +1394,14 @@ void Project::ExportCOLLADA(const QString& FileName)
 	Stream << "<library_visual_scenes>\r\n";
 	Stream << "\t<visual_scene id=\"DefaultScene\">\r\n";
 
-	for (const lcModelPartsEntry& Entry : ModelParts)
+	for (const lcModelPartsEntry& ModelPart : ModelParts)
 	{
-		PieceInfo* Info = Entry.Info;
-		QString ID = QString(Info->mFileName).replace('.', '_');
+		QString ID = GetMeshID(ModelPart);
 
 		Stream << "\t\t<node>\r\n";
 		Stream << "\t\t\t<matrix>\r\n";
 
-		const lcMatrix44& Matrix = Entry.WorldMatrix;
+		const lcMatrix44& Matrix = ModelPart.WorldMatrix;
 		Stream << QString("\t\t\t\t%1 %2 %3 %4\r\n").arg(QString::number(Matrix[0][0]), QString::number(Matrix[1][0]), QString::number(Matrix[2][0]), QString::number(Matrix[3][0]));
 		Stream << QString("\t\t\t\t%1 %2 %3 %4\r\n").arg(QString::number(Matrix[0][1]), QString::number(Matrix[1][1]), QString::number(Matrix[2][1]), QString::number(Matrix[3][1]));
 		Stream << QString("\t\t\t\t%1 %2 %3 %4\r\n").arg(QString::number(Matrix[0][2]), QString::number(Matrix[1][2]), QString::number(Matrix[2][2]), QString::number(Matrix[3][2]));
@@ -1396,7 +1412,7 @@ void Project::ExportCOLLADA(const QString& FileName)
 		Stream << "\t\t\t\t<bind_material>\r\n";
 		Stream << "\t\t\t\t\t<technique_common>\r\n";
 
-		lcMesh* Mesh = Info->GetMesh();
+		lcMesh* Mesh = !ModelPart.Mesh ? ModelPart.Info->GetMesh() : ModelPart.Mesh;
 
 		if (!Mesh)
 			Mesh = gPlaceholderMesh;
@@ -1411,7 +1427,7 @@ void Project::ExportCOLLADA(const QString& FileName)
 			const char* SourceColorName = gColorList[Section->ColorIndex].SafeName;
 			const char* TargetColorName;
 			if (Section->ColorIndex == gDefaultColor)
-				TargetColorName = gColorList[Entry.ColorIndex].SafeName;
+				TargetColorName = gColorList[ModelPart.ColorIndex].SafeName;
 			else
 				TargetColorName = gColorList[Section->ColorIndex].SafeName;
 
@@ -1509,7 +1525,7 @@ QImage Project::CreatePartsListImage(lcModel* Model, lcStep Step)
 		}
 	}
 
-	auto ImageCompare = [this](const lcPartsListImage& Image1, const lcPartsListImage& Image2)
+	auto ImageCompare = [](const lcPartsListImage& Image1, const lcPartsListImage& Image2)
 	{
 		if (Image1.ColorIndex != Image2.ColorIndex)
 			return Image1.ColorIndex < Image2.ColorIndex;
@@ -1554,7 +1570,7 @@ QImage Project::CreatePartsListImage(lcModel* Model, lcStep Step)
 		lcScene Scene;
 		Scene.Begin(ViewMatrix);
 
-		Image.Info->AddRenderMeshes(Scene, lcMatrix44Identity(), Image.ColorIndex, false, false, false);
+		Image.Info->AddRenderMeshes(Scene, lcMatrix44Identity(), Image.ColorIndex, lcRenderMeshState::NORMAL, true);
 
 		Scene.End();
 
@@ -1650,10 +1666,10 @@ QImage Project::CreatePartsListImage(lcModel* Model, lcStep Step)
 		CurrentHeight += NextHeightIncrease;
 	}
 
-	QImage Image(MaxWidth + 40, CurrentHeight + 40, QImage::Format_ARGB32);
-	Image.fill(QColor(255, 255, 255, 0));
+	QImage PainterImage(MaxWidth + 40, CurrentHeight + 40, QImage::Format_ARGB32);
+	PainterImage.fill(QColor(255, 255, 255, 0));
 
-	QPainter Painter(&Image);
+	QPainter Painter(&PainterImage);
 	Painter.setFont(Font);
 
 	for (lcPartsListImage& Image : Images)
@@ -1665,12 +1681,12 @@ QImage Project::CreatePartsListImage(lcModel* Model, lcStep Step)
 
 	Painter.end();
 
-	return Image;
+	return PainterImage;
 }
 
 void Project::CreateHTMLPieceList(QTextStream& Stream, lcModel* Model, lcStep Step, bool Images)
 {
-	std::vector<int> ColorsUsed(gColorList.GetSize(), 0);
+	std::vector<int> ColorsUsed(gColorList.size(), 0);
 	int NumColors = 0;
 
 	lcPartsList PartsList;
@@ -1686,7 +1702,7 @@ void Project::CreateHTMLPieceList(QTextStream& Stream, lcModel* Model, lcStep St
 
 	Stream << QLatin1String("<br><table border=1><tr><td><center>Piece</center></td>\r\n");
 
-	for (int ColorIdx = 0; ColorIdx < gColorList.GetSize(); ColorIdx++)
+	for (size_t ColorIdx = 0; ColorIdx < gColorList.size(); ColorIdx++)
 	{
 		if (ColorsUsed[ColorIdx])
 		{
@@ -1807,7 +1823,7 @@ void Project::ExportHTML(const lcHTMLExportOptions& Options)
 			if (Options.PartsListEnd)
 				AddPartsListImage(Stream, Model, 0, BaseName);
 
-			Stream << QLatin1String("</CENTER>\r\n<BR><HR><BR><B><I>Created by <A HREF=\"http://www.leocad.org\">LeoCAD</A></I></B><BR></HTML>\r\n");
+			Stream << QLatin1String("</CENTER>\r\n<BR><HR><BR><B><I>Created by <A HREF=\"https://www.leocad.org\">LeoCAD</A></I></B><BR></HTML>\r\n");
 		}
 		else
 		{
@@ -1832,7 +1848,7 @@ void Project::ExportHTML(const lcHTMLExportOptions& Options)
 				if (Options.PartsListEnd)
 					Stream << QString::fromLatin1("<A HREF=\"%1-pieces.html\">Pieces Used</A><BR>\r\n").arg(BaseName);
 
-				Stream << QLatin1String("</CENTER>\r\n<BR><HR><BR><B><I>Created by <A HREF=\"http://www.leocad.org\">LeoCAD</A></B></I><BR></HTML>\r\n");
+				Stream << QLatin1String("</CENTER>\r\n<BR><HR><BR><B><I>Created by <A HREF=\"https://www.leocad.org\">LeoCAD</A></B></I><BR></HTML>\r\n");
 			}
 
 			for (lcStep Step = 1; Step <= LastStep; Step++)
@@ -1942,7 +1958,7 @@ void Project::ExportHTML(const lcHTMLExportOptions& Options)
 				lcScene Scene;
 				Scene.Begin(ViewMatrix);
 
-				Info->AddRenderMeshes(Scene, lcMatrix44Identity(), Options.PartImagesColor, false, false, false);
+				Info->AddRenderMeshes(Scene, lcMatrix44Identity(), Options.PartImagesColor, lcRenderMeshState::NORMAL, true);
 
 				Scene.End();
 
@@ -2012,17 +2028,15 @@ void Project::ExportHTML(const lcHTMLExportOptions& Options)
 			}
 		}
 
-		Stream << QLatin1String("</CENTER>\r\n<BR><HR><BR><B><I>Created by <A HREF=\"http://www.leocad.org\">LeoCAD</A></B></I><BR></HTML>\r\n");
+		Stream << QLatin1String("</CENTER>\r\n<BR><HR><BR><B><I>Created by <A HREF=\"https://www.leocad.org\">LeoCAD</A></B></I><BR></HTML>\r\n");
 	}
 }
 
 bool Project::ExportPOVRay(const QString& FileName)
 {
-	lcArray<lcModelPartsEntry> ModelParts;
+	std::vector<lcModelPartsEntry> ModelParts = GetModelParts();
 
-	GetModelParts(ModelParts);
-
-	if (ModelParts.IsEmpty())
+	if (ModelParts.empty())
 	{
 		QMessageBox::information(gMainWindow, tr("LeoCAD"), tr("Nothing to export."));
 		return false;
@@ -2046,8 +2060,8 @@ bool Project::ExportPOVRay(const QString& FileName)
 	char Line[1024];
 
 	lcPiecesLibrary* Library = lcGetPiecesLibrary();
-	std::map<PieceInfo*, std::pair<char[LC_PIECE_NAME_LEN], int>> PieceTable;
-	int NumColors = gColorList.GetSize();
+	std::map<const PieceInfo*, std::pair<char[LC_PIECE_NAME_LEN], int>> PieceTable;
+	size_t NumColors = gColorList.size();
 	std::vector<std::array<char, LC_MAX_COLOR_NAME>> ColorTable(NumColors);
 
 	enum
@@ -2137,7 +2151,7 @@ bool Project::ExportPOVRay(const QString& FileName)
 			if (sscanf(Line,"%d%s%s", &Code, Name, Flags) != 3)
 				continue;
 
-			int Color = lcGetColorIndex(Code);
+			size_t Color = lcGetColorIndex(Code);
 			if (Color >= NumColors)
 				continue;
 
@@ -2145,42 +2159,41 @@ bool Project::ExportPOVRay(const QString& FileName)
 		}
 	}
 
+	std::set<lcMesh*> AddedMeshes;
+
 	if (!LGEOPath.isEmpty())
 	{
 		POVFile.WriteLine("#include \"lg_defs.inc\"\n#include \"lg_color.inc\"\n\n");
 
-		for (int PartIdx = 0; PartIdx < ModelParts.GetSize(); PartIdx++)
+		for (const lcModelPartsEntry& ModelPart : ModelParts)
 		{
-			PieceInfo* Info = ModelParts[PartIdx].Info;
+			if (ModelPart.Mesh)
+				continue;
 
-			for (int CheckIdx = 0; CheckIdx < ModelParts.GetSize(); CheckIdx++)
+			auto Search = PieceTable.find(ModelPart.Info);
+			if (Search == PieceTable.end())
+				continue;
+
+			lcMesh* Mesh = ModelPart.Info->GetMesh();
+
+			if (!Mesh)
+				continue;
+
+			if (!AddedMeshes.insert(Mesh).second)
+				continue;
+
+			const std::pair<char[LC_PIECE_NAME_LEN], int>& Entry = Search->second;
+			if (Entry.first[0])
 			{
-				if (ModelParts[CheckIdx].Info != Info)
-					continue;
-
-				if (CheckIdx != PartIdx)
-					break;
-
-				auto Search = PieceTable.find(Info);
-
-				if (Search != PieceTable.end())
-				{
-					const std::pair<char[LC_PIECE_NAME_LEN], int>& Entry = Search->second;
-					if (Entry.first[0])
-					{
-						sprintf(Line, "#include \"%s.inc\"\n", Entry.first);
-						POVFile.WriteLine(Line);
-					}
-				}
-
-				break;
+				sprintf(Line, "#include \"%s.inc\"\n", Entry.first);
+				POVFile.WriteLine(Line);
 			}
 		}
 
 		POVFile.WriteLine("\n");
 	}
 
-	for (int ColorIdx = 0; ColorIdx < gColorList.GetSize(); ColorIdx++)
+	for (size_t ColorIdx = 0; ColorIdx < gColorList.size(); ColorIdx++)
 	{
 		lcColor* Color = &gColorList[ColorIdx];
 
@@ -2203,30 +2216,45 @@ bool Project::ExportPOVRay(const QString& FileName)
 
 	POVFile.WriteLine("\n");
 
-	lcArray<const char*> ColorTablePointer;
-	ColorTablePointer.SetSize(NumColors);
-	for (int ColorIdx = 0; ColorIdx < NumColors; ColorIdx++)
+	std::vector<const char*> ColorTablePointer;
+	ColorTablePointer.resize(NumColors);
+	for (size_t ColorIdx = 0; ColorIdx < NumColors; ColorIdx++)
 		ColorTablePointer[ColorIdx] = ColorTable[ColorIdx].data();
 
-	for (int PartIdx = 0; PartIdx < ModelParts.GetSize(); PartIdx++)
+	auto GetMeshName = [](const lcModelPartsEntry& ModelPart, char* Name)
 	{
-		PieceInfo* Info = ModelParts[PartIdx].Info;
-		lcMesh* Mesh = Info->GetMesh();
-		std::pair<char[LC_PIECE_NAME_LEN], int>& Entry = PieceTable[Info];
+		strcpy(Name, ModelPart.Info->mFileName);
 
-		if (!Mesh || Entry.first[0])
+		for (char* c = Name; *c; c++)
+			if (*c == '-' || *c == '.')
+				*c = '_';
+
+		if (ModelPart.Mesh)
+		{
+			char Suffix[32];
+			sprintf(Suffix, "_%p", ModelPart.Mesh);
+			strcat(Name, Suffix);
+		}
+	};
+
+	for (const lcModelPartsEntry& ModelPart : ModelParts)
+	{
+		lcMesh* Mesh = !ModelPart.Mesh ? ModelPart.Info->GetMesh() : ModelPart.Mesh;
+
+		if (!AddedMeshes.insert(Mesh).second)
+			continue;
+
+		if (!Mesh)
 			continue;
 
 		char Name[LC_PIECE_NAME_LEN];
-		char* Ptr;
+		GetMeshName(ModelPart, Name);
 
-		strcpy(Name, Info->mFileName);
-		while ((Ptr = strchr(Name, '-')))
-			*Ptr = '_';
-		while ((Ptr = strchr(Name, '.')))
-			*Ptr = '_';
-
-		sprintf(Entry.first, "lc_%s", Name);
+		if (!ModelPart.Mesh)
+		{
+			std::pair<char[LC_PIECE_NAME_LEN], int>& Entry = PieceTable[ModelPart.Info];
+			sprintf(Entry.first, "lc_%s", Name);
+		}
 
 		Mesh->ExportPOVRay(POVFile, Name, &ColorTablePointer[0]);
 
@@ -2277,28 +2305,41 @@ bool Project::ExportPOVRay(const QString& FileName)
 	sprintf(Line, "light_source{ <%f, %f, %f>\n  color rgb 0.5\n  area_light 200, 200, 10, 10\n  jitter\n}\n\n", 2.0f * Radius + Center.x, 0.0f * Radius + Center.y, -2.0f * Radius + Center.z);
 	POVFile.WriteLine(Line);
 
-	for (int PartIdx = 0; PartIdx < ModelParts.GetSize(); PartIdx++)
+	for (const lcModelPartsEntry& ModelPart : ModelParts)
 	{
-		std::pair<char[LC_PIECE_NAME_LEN], int>& Entry = PieceTable[ModelParts[PartIdx].Info];
-		int Color;
-
-		Color = ModelParts[PartIdx].ColorIndex;
+		int Color = ModelPart.ColorIndex;
 		const char* Suffix = lcIsColorTranslucent(Color) ? "_clear" : "";
+		const float* f = ModelPart.WorldMatrix;
 
-		const float* f = ModelParts[PartIdx].WorldMatrix;
-
-		if (Entry.second & LGEO_PIECE_SLOPE)
+		if (!ModelPart.Mesh)
 		{
-			sprintf(Line, "merge {\n object {\n  %s%s\n  texture { %s }\n }\n"
-					" object {\n  %s_slope\n  texture { %s normal { bumps 0.3 scale 0.02 } }\n }\n"
-					" matrix <%.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f>\n}\n",
-					Entry.first, Suffix, ColorTable[Color].data(), Entry.first, ColorTable[Color].data(),
-					-f[5], -f[4], -f[6], -f[1], -f[0], -f[2], f[9], f[8], f[10], f[13] / 25.0f, f[12] / 25.0f, f[14] / 25.0f);
+			std::pair<char[LC_PIECE_NAME_LEN], int>& Entry = PieceTable[ModelPart.Info];
+
+			if (Entry.second & LGEO_PIECE_SLOPE)
+			{
+				sprintf(Line, "merge {\n object {\n  %s%s\n  texture { %s }\n }\n"
+						" object {\n  %s_slope\n  texture { %s normal { bumps 0.3 scale 0.02 } }\n }\n"
+						" matrix <%.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f>\n}\n",
+						Entry.first, Suffix, ColorTable[Color].data(), Entry.first, ColorTable[Color].data(),
+						-f[5], -f[4], -f[6], -f[1], -f[0], -f[2], f[9], f[8], f[10], f[13] / 25.0f, f[12] / 25.0f, f[14] / 25.0f);
+			}
+			else
+			{
+				if (!ModelPart.Info || !ModelPart.Info->GetMesh())
+					continue;
+
+				sprintf(Line, "object {\n %s%s\n texture { %s }\n matrix <%.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f>\n}\n",
+						Entry.first, Suffix, ColorTable[Color].data(), -f[5], -f[4], -f[6], -f[1], -f[0], -f[2], f[9], f[8], f[10], f[13] / 25.0f, f[12] / 25.0f, f[14] / 25.0f);
+
+			}
 		}
 		else
 		{
-			sprintf(Line, "object {\n %s%s\n texture { %s }\n matrix <%.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f>\n}\n",
-					Entry.first, Suffix, ColorTable[Color].data(), -f[5], -f[4], -f[6], -f[1], -f[0], -f[2], f[9], f[8], f[10], f[13] / 25.0f, f[12] / 25.0f, f[14] / 25.0f);
+			char Name[LC_PIECE_NAME_LEN];
+			GetMeshName(ModelPart, Name);
+
+			sprintf(Line, "object {\n lc_%s%s\n texture { %s }\n matrix <%.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f, %.4f>\n}\n",
+					Name, Suffix, ColorTable[Color].data(), -f[5], -f[4], -f[6], -f[1], -f[0], -f[2], f[9], f[8], f[10], f[13] / 25.0f, f[12] / 25.0f, f[14] / 25.0f);
 		}
 
 		POVFile.WriteLine(Line);
@@ -2311,11 +2352,9 @@ bool Project::ExportPOVRay(const QString& FileName)
 
 void Project::ExportWavefront(const QString& FileName)
 {
-	lcArray<lcModelPartsEntry> ModelParts;
+	std::vector<lcModelPartsEntry> ModelParts = GetModelParts();
 
-	GetModelParts(ModelParts);
-
-	if (ModelParts.IsEmpty())
+	if (ModelParts.empty())
 	{
 		QMessageBox::information(gMainWindow, tr("LeoCAD"), tr("Nothing to export."));
 		return;
@@ -2353,24 +2392,23 @@ void Project::ExportWavefront(const QString& FileName)
 	}
 
 	MaterialFile.WriteLine("# Colors used by LeoCAD\n\n");
-	for (int ColorIdx = 0; ColorIdx < gColorList.GetSize(); ColorIdx++)
+	for (const lcColor& Color : gColorList)
 	{
-		lcColor* Color = &gColorList[ColorIdx];
-		if (Color->Translucent)
-			sprintf(Line, "newmtl %s\nKd %.2f %.2f %.2f\nD %.2f\n\n", Color->SafeName, Color->Value[0], Color->Value[1], Color->Value[2], Color->Value[3]);
+		if (Color.Translucent)
+			sprintf(Line, "newmtl %s\nKd %.2f %.2f %.2f\nD %.2f\n\n", Color.SafeName, Color.Value[0], Color.Value[1], Color.Value[2], Color.Value[3]);
 		else
-			sprintf(Line, "newmtl %s\nKd %.2f %.2f %.2f\n\n", Color->SafeName, Color->Value[0], Color->Value[1], Color->Value[2]);
+			sprintf(Line, "newmtl %s\nKd %.2f %.2f %.2f\n\n", Color.SafeName, Color.Value[0], Color.Value[1], Color.Value[2]);
 		MaterialFile.WriteLine(Line);
 	}
 
-	for (int PartIdx = 0; PartIdx < ModelParts.GetSize(); PartIdx++)
+	for (const lcModelPartsEntry& ModelPart : ModelParts)
 	{
-		lcMesh* Mesh = ModelParts[PartIdx].Info->GetMesh();
+		lcMesh* Mesh = !ModelPart.Mesh ? ModelPart.Info->GetMesh() : ModelPart.Mesh;
 
 		if (!Mesh)
 			continue;
 
-		const lcMatrix44& ModelWorld = ModelParts[PartIdx].WorldMatrix;
+		const lcMatrix44& ModelWorld = ModelPart.WorldMatrix;
 		lcVertex* Verts = (lcVertex*)Mesh->mVertexData;
 
 		for (int VertexIdx = 0; VertexIdx < Mesh->mNumVertices; VertexIdx++)
@@ -2383,14 +2421,14 @@ void Project::ExportWavefront(const QString& FileName)
 		OBJFile.WriteLine("#\n\n");
 	}
 
-	for (int PartIdx = 0; PartIdx < ModelParts.GetSize(); PartIdx++)
+	for (const lcModelPartsEntry& ModelPart : ModelParts)
 	{
-		lcMesh* Mesh = ModelParts[PartIdx].Info->GetMesh();
+		lcMesh* Mesh = !ModelPart.Mesh ? ModelPart.Info->GetMesh() : ModelPart.Mesh;
 
 		if (!Mesh)
 			continue;
 
-		const lcMatrix44& ModelWorld = ModelParts[PartIdx].WorldMatrix;
+		const lcMatrix44& ModelWorld = ModelPart.WorldMatrix;
 		lcVertex* Verts = (lcVertex*)Mesh->mVertexData;
 
 		for (int VertexIdx = 0; VertexIdx < Mesh->mNumVertices; VertexIdx++)
@@ -2403,18 +2441,17 @@ void Project::ExportWavefront(const QString& FileName)
 		OBJFile.WriteLine("#\n\n");
 	}
 
-	for (int PartIdx = 0; PartIdx < ModelParts.GetSize(); PartIdx++)
+	int NumPieces = 0;
+	for (const lcModelPartsEntry& ModelPart : ModelParts)
 	{
-		PieceInfo* Info = ModelParts[PartIdx].Info;
-
-		sprintf(Line, "g Piece%.3d\n", PartIdx);
+		sprintf(Line, "g Piece%.3d\n", NumPieces++);
 		OBJFile.WriteLine(Line);
 
-		lcMesh* Mesh = Info->GetMesh();
+		lcMesh* Mesh = !ModelPart.Mesh ? ModelPart.Info->GetMesh() : ModelPart.Mesh;
 
 		if (Mesh)
 		{
-			Mesh->ExportWavefrontIndices(OBJFile, ModelParts[PartIdx].ColorIndex, vert);
+			Mesh->ExportWavefrontIndices(OBJFile, ModelPart.ColorIndex, vert);
 			vert += Mesh->mNumVertices;
 		}
 	}
@@ -2442,10 +2479,15 @@ void Project::UpdatePieceInfo(PieceInfo* Info) const
 {
 	if (!mModels.IsEmpty())
 	{
-		lcArray<lcModel*> UpdatedModels;
+		std::vector<lcModel*> UpdatedModels;
 		mModels[0]->UpdatePieceInfo(UpdatedModels);
 
 		lcBoundingBox BoundingBox = mModels[0]->GetPieceInfo()->GetBoundingBox();
 		Info->SetBoundingBox(BoundingBox.Min, BoundingBox.Max);
 	}
 }
+
+void Project::MarkAsModified()
+{
+	mModified = true;
+}
