@@ -76,6 +76,7 @@ public:
 	}
 
 	bool IsModified() const;
+	void MarkAsModified();
 	QString GetTitle() const;
 
 	QString GetFileName() const
@@ -83,7 +84,7 @@ public:
 		return mFileName;
 	}
 
-	QString GetImageFileName() const;
+	QString GetImageFileName(bool AllowCurrentFolder) const;
 
 	void SetActiveModel(int ModelIndex);
 	void SetActiveModel(const QString& ModelName);
@@ -100,7 +101,7 @@ public:
 	bool ImportInventory(const QByteArray& Inventory, const QString& Name, const QString& Description);
 
 	void SaveImage();
-	bool ExportModel(const QString& FileName, lcModel* Model);
+	bool ExportModel(const QString& FileName, lcModel* Model) const;
 	void Export3DStudio(const QString& FileName);
 	void ExportBrickLink();
 	void ExportCOLLADA(const QString& FileName);
@@ -113,7 +114,7 @@ public:
 
 protected:
 	QString GetExportFileName(const QString& FileName, const QString& DefaultExtension, const QString& DialogTitle, const QString& DialogFilter) const;
-	void GetModelParts(lcArray<lcModelPartsEntry>& ModelParts);
+	std::vector<lcModelPartsEntry> GetModelParts();
 	QImage CreatePartsListImage(lcModel* Model, lcStep Step);
 	void CreateHTMLPieceList(QTextStream& Stream, lcModel* Model, lcStep Step, bool Images);
 	void SetFileName(const QString& FileName);
