@@ -13,8 +13,10 @@ lcVertexBuffer View::mRotateMoveVertexBuffer;
 lcIndexBuffer View::mRotateMoveIndexBuffer;
 
 View::View(lcModel* Model)
+	: mViewSphere(this)
 {
 	mModel = Model;
+	mActiveSubmodelInstance = nullptr;
 	mCamera = nullptr;
 	mHighlight = false;
 	memset(mGridSettings, 0, sizeof(mGridSettings));
@@ -42,6 +44,53 @@ View::~View()
 		delete mCamera;
 }
 
+lcModel* View::GetActiveModel() const
+{
+	return !mActiveSubmodelInstance ? mModel : mActiveSubmodelInstance->mPieceInfo->GetModel();
+}
+
+void View::SetTopSubmodelActive()
+{
+	lcModel* ActiveModel = GetActiveModel();
+
+	if (mActiveSubmodelInstance)
+	{
+		ActiveModel->SetActive(false);
+		mActiveSubmodelInstance = nullptr;
+	}
+
+	GetActiveModel()->UpdateInterface();
+}
+
+void View::SetSelectedSubmodelActive()
+{
+	lcModel* ActiveModel = GetActiveModel();
+	lcObject* Object = ActiveModel->GetFocusObject();
+
+	if (mActiveSubmodelInstance)
+	{
+		ActiveModel->SetActive(false);
+		mActiveSubmodelInstance = nullptr;
+	}
+
+	if (Object && Object->IsPiece())
+	{
+		lcPiece* Piece = (lcPiece*)Object;
+
+		if (Piece->mPieceInfo->IsModel())
+		{
+			mActiveSubmodelTransform = lcMatrix44Identity();
+			mModel->GetPieceWorldMatrix(Piece, mActiveSubmodelTransform);
+			mActiveSubmodelInstance = Piece;
+			ActiveModel = mActiveSubmodelInstance->mPieceInfo->GetModel();
+			ActiveModel->SetActive(true);
+			RemoveCamera();
+		}
+	}
+
+	GetActiveModel()->UpdateInterface();
+}
+
 void View::CreateResources(lcContext* Context)
 {
 	gGridTexture = new lcTexture;
@@ -280,6 +329,18 @@ void View::SetViewpoint(lcViewpoint Viewpoint)
 	gMainWindow->UpdateCurrentCamera(-1);
 }
 
+void View::SetViewpoint(const lcVector3& Position)
+{
+	if (!mCamera || !mCamera->IsSimple())
+		mCamera = new lcCamera(true);
+
+	mCamera->SetViewpoint(Position);
+	ZoomExtents();
+	Redraw();
+
+	gMainWindow->UpdateCurrentCamera(-1);
+}
+
 void View::SetCameraAngles(float Latitude, float Longitude)
 {
 	if (!mCamera || !mCamera->IsSimple())
@@ -428,6 +489,8 @@ void View::ShowContextMenu() const
 
 	Popup->addSeparator();
 
+	Popup->addAction(Actions[LC_PIECE_EDIT_SELECTED_SUBMODEL]);
+	Popup->addAction(Actions[LC_PIECE_EDIT_END_SUBMODEL]);
 	Popup->addAction(Actions[LC_PIECE_VIEW_SELECTED_MODEL]);
 	Popup->addAction(Actions[LC_PIECE_INLINE_SELECTED_MODELS]);
 	Popup->addAction(Actions[LC_PIECE_MOVE_SELECTION_TO_MODEL]);
@@ -448,6 +511,7 @@ void View::ShowContextMenu() const
 	Popup->addAction(Actions[LC_VIEW_RESET_VIEWS]);
 
 	Popup->exec(QCursor::pos());
+	delete Popup;
 }
 
 lcVector3 View::GetMoveDirection(const lcVector3& Direction) const
@@ -501,15 +565,16 @@ lcVector3 View::GetMoveDirection(const lcVector3& Direction) const
 lcMatrix44 View::GetPieceInsertPosition(bool IgnoreSelected, PieceInfo* Info) const
 {
 	lcPiece* HitPiece = (lcPiece*)FindObjectUnderPointer(true, IgnoreSelected).Object;
+	lcModel* ActiveModel = GetActiveModel();
 
 	if (HitPiece)
 	{
 		lcVector3 Position(0, 0, HitPiece->GetBoundingBox().Max.z - Info->GetBoundingBox().Min.z);
 
 		if (gMainWindow->GetRelativeTransform())
-			Position = lcMul31(mModel->SnapPosition(Position), HitPiece->mModelWorld);
+			Position = lcMul31(ActiveModel->SnapPosition(Position), HitPiece->mModelWorld);
 		else
-			Position = mModel->SnapPosition(lcMul31(Position, HitPiece->mModelWorld));
+			Position = ActiveModel->SnapPosition(lcMul31(Position, HitPiece->mModelWorld));
 
 		lcMatrix44 WorldMatrix = HitPiece->mModelWorld;
 		WorldMatrix.SetTranslation(Position);
@@ -517,15 +582,23 @@ lcMatrix44 View::GetPieceInsertPosition(bool IgnoreSelected, PieceInfo* Info) co
 		return WorldMatrix;
 	}
 
-	lcVector3 ClickPoints[2] = { lcVector3((float)mInputState.x, (float)mInputState.y, 0.0f), lcVector3((float)mInputState.x, (float)mInputState.y, 1.0f) };
-	UnprojectPoints(ClickPoints, 2);
+	std::array<lcVector3, 2> ClickPoints = {{ lcVector3((float)mInputState.x, (float)mInputState.y, 0.0f), lcVector3((float)mInputState.x, (float)mInputState.y, 1.0f) }};
+	UnprojectPoints(ClickPoints.data(), 2);
+
+	if (ActiveModel != mModel)
+	{
+		lcMatrix44 InverseMatrix = lcMatrix44AffineInverse(mActiveSubmodelTransform);
+
+		for (lcVector3& Point : ClickPoints)
+			Point = lcMul31(Point, InverseMatrix);
+	}
 
 	lcVector3 Intersection;
 
 	const lcBoundingBox& BoundingBox = Info->GetBoundingBox();
 	if (lcLineSegmentPlaneIntersection(&Intersection, ClickPoints[0], ClickPoints[1], lcVector4(0, 0, 1, BoundingBox.Min.z)))
 	{
-		Intersection = mModel->SnapPosition(Intersection);
+		Intersection = ActiveModel->SnapPosition(Intersection);
 		return lcMatrix44Translation(Intersection);
 	}
 
@@ -567,7 +640,17 @@ lcObjectSection View::FindObjectUnderPointer(bool PiecesOnly, bool IgnoreSelecte
 	ObjectRayTest.ObjectSection.Object = nullptr;
 	ObjectRayTest.ObjectSection.Section = 0;;
 
-	mModel->RayTest(ObjectRayTest);
+	lcModel* ActiveModel = GetActiveModel();
+
+	if (ActiveModel != mModel)
+	{
+		lcMatrix44 InverseMatrix = lcMatrix44AffineInverse(mActiveSubmodelTransform);
+
+		ObjectRayTest.Start = lcMul31(ObjectRayTest.Start, InverseMatrix);
+		ObjectRayTest.End = lcMul31(ObjectRayTest.End, InverseMatrix);
+	}
+
+	ActiveModel->RayTest(ObjectRayTest);
 
 	return ObjectRayTest.ObjectSection;
 }
@@ -598,17 +681,27 @@ lcArray<lcObject*> View::FindObjectsInBox(float x1, float y1, float x2, float y2
 		Bottom = y1;
 	}
 
-	lcVector3 Corners[6] =
-	{
+	std::array<lcVector3, 6> Corners =
+	{{
 		lcVector3(Left, Top, 0),
 		lcVector3(Left, Bottom, 0),
 		lcVector3(Right, Bottom, 0),
 		lcVector3(Right, Top, 0),
 		lcVector3(Left, Top, 1),
 		lcVector3(Right, Bottom, 1)
-	};
+	}};
 
-	UnprojectPoints(Corners, 6);
+	UnprojectPoints(Corners.data(), (int)Corners.size());
+
+	lcModel* ActiveModel = GetActiveModel();
+
+	if (ActiveModel != mModel)
+	{
+		lcMatrix44 InverseMatrix = lcMatrix44AffineInverse(mActiveSubmodelTransform);
+
+		for (lcVector3& Point : Corners)
+			Point = lcMul31(Point, InverseMatrix);
+	}
 
 	lcVector3 PlaneNormals[6];
 	PlaneNormals[0] = lcNormalize(lcCross(Corners[4] - Corners[0], Corners[1] - Corners[0])); // Left
@@ -627,7 +720,7 @@ lcArray<lcObject*> View::FindObjectsInBox(float x1, float y1, float x2, float y2
 	ObjectBoxTest.Planes[4] = lcVector4(PlaneNormals[4], -lcDot(PlaneNormals[4], Corners[0]));
 	ObjectBoxTest.Planes[5] = lcVector4(PlaneNormals[5], -lcDot(PlaneNormals[5], Corners[5]));
 
-	mModel->BoxTest(ObjectBoxTest);
+	ActiveModel->BoxTest(ObjectBoxTest);
 
 	return ObjectBoxTest.Objects;
 }
@@ -638,7 +731,7 @@ bool View::BeginRenderToImage(int Width, int Height)
 	glGetIntegerv(GL_MAX_TEXTURE_SIZE, &MaxTexture);
 
 	MaxTexture = qMin(MaxTexture, 2048);
-	MaxTexture /= QGLFormat::defaultFormat().samples();
+	MaxTexture /= QGLFormat::defaultFormat().sampleBuffers() ? QGLFormat::defaultFormat().samples() : 1;
 
 	int TileWidth = qMin(Width, MaxTexture);
 	int TileHeight = qMin(Height, MaxTexture);
@@ -666,14 +759,21 @@ void View::OnDraw()
 
 	bool DrawInterface = mWidget != nullptr;
 
-	mModel->GetScene(mScene, mCamera, DrawInterface, mHighlight);
+	mModel->GetScene(mScene, mCamera, DrawInterface, mHighlight, mActiveSubmodelInstance, mActiveSubmodelTransform);
 
 	if (DrawInterface && mTrackTool == LC_TRACKTOOL_INSERT)
 	{
 		PieceInfo* Info = gMainWindow->GetCurrentPieceInfo();
 
 		if (Info)
-			Info->AddRenderMeshes(mScene, GetPieceInsertPosition(false, gMainWindow->GetCurrentPieceInfo()), gMainWindow->mColorIndex, true, true, false);
+		{
+			lcMatrix44 WorldMatrix = GetPieceInsertPosition(false, gMainWindow->GetCurrentPieceInfo());
+
+			if (GetActiveModel() != mModel)
+				WorldMatrix = lcMul(WorldMatrix, mActiveSubmodelTransform);
+
+			Info->AddRenderMeshes(mScene, WorldMatrix, gMainWindow->mColorIndex, lcRenderMeshState::FOCUSED, true);
+		}
 	}
 
 	int TotalTileRows = 1;
@@ -771,18 +871,20 @@ void View::OnDraw()
 			DrawAxes();
 
 		lcTool Tool = gMainWindow->GetTool();
+		lcModel* ActiveModel = GetActiveModel();
 
-		if ((Tool == LC_TOOL_SELECT || Tool == LC_TOOL_MOVE) && mTrackButton == LC_TRACKBUTTON_NONE && mModel->AnyObjectsSelected())
+		if ((Tool == LC_TOOL_SELECT || Tool == LC_TOOL_MOVE) && mTrackButton == LC_TRACKBUTTON_NONE && ActiveModel->AnyObjectsSelected())
 			DrawSelectMoveOverlay();
 		else if (GetCurrentTool() == LC_TOOL_MOVE && mTrackButton != LC_TRACKBUTTON_NONE)
 			DrawSelectMoveOverlay();
-		else if ((Tool == LC_TOOL_ROTATE || (Tool == LC_TOOL_SELECT && mTrackButton != LC_TRACKBUTTON_NONE && mTrackTool >= LC_TRACKTOOL_ROTATE_X && mTrackTool <= LC_TRACKTOOL_ROTATE_XYZ)) && mModel->AnyPiecesSelected())
+		else if ((Tool == LC_TOOL_ROTATE || (Tool == LC_TOOL_SELECT && mTrackButton != LC_TRACKBUTTON_NONE && mTrackTool >= LC_TRACKTOOL_ROTATE_X && mTrackTool <= LC_TRACKTOOL_ROTATE_XYZ)) && ActiveModel->AnyPiecesSelected())
 			DrawRotateOverlay();
 		else if ((mTrackTool == LC_TRACKTOOL_SELECT || mTrackTool == LC_TRACKTOOL_ZOOM_REGION) && mTrackButton == LC_TRACKBUTTON_LEFT)
 			DrawSelectZoomRegionOverlay();
 		else if (Tool == LC_TOOL_ROTATE_VIEW && mTrackButton == LC_TRACKBUTTON_NONE)
 			DrawRotateViewOverlay();
 
+		mViewSphere.Draw();
 		DrawViewport();
 	}
 
@@ -799,11 +901,15 @@ void View::DrawSelectMoveOverlay()
 
 	lcVector3 OverlayCenter;
 	lcMatrix33 RelativeRotation;
-	mModel->GetMoveRotateTransform(OverlayCenter, RelativeRotation);
-	bool AnyPiecesSelected = mModel->AnyPiecesSelected();
+	lcModel* ActiveModel = GetActiveModel();
+	ActiveModel->GetMoveRotateTransform(OverlayCenter, RelativeRotation);
+	bool AnyPiecesSelected = ActiveModel->AnyPiecesSelected();
 
 	lcMatrix44 WorldMatrix = lcMatrix44(RelativeRotation, OverlayCenter);
 
+	if (ActiveModel != mModel)
+		WorldMatrix = lcMul(WorldMatrix, mActiveSubmodelTransform);
+
 	const float OverlayScale = GetOverlayScale();
 	WorldMatrix = lcMul(lcMatrix44Scale(lcVector3(OverlayScale, OverlayScale, OverlayScale)), WorldMatrix);
 
@@ -813,7 +919,7 @@ void View::DrawSelectMoveOverlay()
 	mContext->SetVertexBuffer(mRotateMoveVertexBuffer);
 	mContext->SetVertexFormatPosition(3);
 
-	lcObject* Focus = mModel->GetFocusObject();
+	lcObject* Focus = ActiveModel->GetFocusObject();
 	quint32 AllowedTransforms = Focus ? Focus->GetAllowedTransforms() : LC_OBJECT_TRANSFORM_MOVE_X | LC_OBJECT_TRANSFORM_MOVE_Y | LC_OBJECT_TRANSFORM_MOVE_Z | LC_OBJECT_TRANSFORM_ROTATE_X | LC_OBJECT_TRANSFORM_ROTATE_Y | LC_OBJECT_TRANSFORM_ROTATE_Z;
 
 	if (mTrackButton == LC_TRACKBUTTON_NONE || (mTrackTool >= LC_TRACKTOOL_MOVE_X && mTrackTool <= LC_TRACKTOOL_MOVE_XYZ))
@@ -989,8 +1095,9 @@ void View::DrawRotateOverlay()
 
 	lcVector3 OverlayCenter;
 	lcMatrix33 RelativeRotation;
-	mModel->GetMoveRotateTransform(OverlayCenter, RelativeRotation);
-	lcVector3 MouseToolDistance = mModel->SnapRotation(mModel->GetMouseToolDistance());
+	lcModel* ActiveModel = GetActiveModel();
+	ActiveModel->GetMoveRotateTransform(OverlayCenter, RelativeRotation);
+	lcVector3 MouseToolDistance = ActiveModel->SnapRotation(ActiveModel->GetMouseToolDistance());
 	bool HasAngle = false;
 
 	// Draw a disc showing the rotation amount.
@@ -1632,7 +1739,7 @@ void View::DrawAxes()
 	mContext->SetColor(0.0f, 0.0f, 0.0f, 1.0f);
 	mContext->DrawIndexedPrimitives(GL_LINES, 6, GL_UNSIGNED_SHORT, 0);
 
-	mContext->SetColor(0.8f, 0.0f, 0.0f, 1.0f),
+	mContext->SetColor(0.8f, 0.0f, 0.0f, 1.0f);
 	mContext->DrawIndexedPrimitives(GL_TRIANGLES, 24, GL_UNSIGNED_SHORT, 6 * 2);
 	mContext->SetColor(0.0f, 0.8f, 0.0f, 1.0f);
 	mContext->DrawIndexedPrimitives(GL_TRIANGLES, 24, GL_UNSIGNED_SHORT, (6 + 24) * 2);
@@ -1783,7 +1890,8 @@ float View::GetOverlayScale() const
 {
 	lcVector3 OverlayCenter;
 	lcMatrix33 RelativeRotation;
-	mModel->GetMoveRotateTransform(OverlayCenter, RelativeRotation);
+	lcModel* ActiveModel = GetActiveModel();
+	ActiveModel->GetMoveRotateTransform(OverlayCenter, RelativeRotation);
 
 	lcVector3 ScreenPos = ProjectPoint(OverlayCenter);
 	ScreenPos[0] += 10.0f;
@@ -1803,17 +1911,19 @@ void View::EndDrag(bool Accept)
 {
 	if (Accept)
 	{
+		lcModel* ActiveModel = GetActiveModel();
+
 		switch (mDragState)
 		{
 		case lcDragState::NONE:
 			break;
 
 		case lcDragState::PIECE:
-			mModel->InsertPieceToolClicked(GetPieceInsertPosition(false, gMainWindow->GetCurrentPieceInfo()));
+			ActiveModel->InsertPieceToolClicked(GetPieceInsertPosition(false, gMainWindow->GetCurrentPieceInfo()));
 			break;
 
 		case lcDragState::COLOR:
-			mModel->PaintToolClicked(FindObjectUnderPointer(true, false).Object);
+			ActiveModel->PaintToolClicked(FindObjectUnderPointer(true, false).Object);
 			break;
 		}
 	}
@@ -1832,22 +1942,39 @@ void View::SetProjection(bool Ortho)
 		gMainWindow->UpdatePerspective();
 	}
 	else
-		mModel->SetCameraOrthographic(mCamera, Ortho);
+	{
+		lcModel* ActiveModel = GetActiveModel();
+		if (ActiveModel)
+			ActiveModel->SetCameraOrthographic(mCamera, Ortho);
+	}
 }
 
 void View::LookAt()
 {
-	mModel->LookAt(mCamera);
+	lcModel* ActiveModel = GetActiveModel();
+	if (ActiveModel)
+		ActiveModel->LookAt(mCamera);
 }
 
 void View::ZoomExtents()
 {
-	mModel->ZoomExtents(mCamera, (float)mWidth / (float)mHeight);
+	lcModel* ActiveModel = GetActiveModel();
+	if (ActiveModel)
+		ActiveModel->ZoomExtents(mCamera, (float)mWidth / (float)mHeight);
 }
 
 void View::MoveCamera(const lcVector3& Direction)
 {
-	mModel->MoveCamera(mCamera, Direction);
+	lcModel* ActiveModel = GetActiveModel();
+	if (ActiveModel)
+		ActiveModel->MoveCamera(mCamera, Direction);
+}
+
+void View::Zoom(float Amount)
+{
+	lcModel* ActiveModel = GetActiveModel();
+	if (ActiveModel)
+		ActiveModel->Zoom(mCamera, Amount);
 }
 
 void View::UpdateTrackTool()
@@ -1858,6 +1985,7 @@ void View::UpdateTrackTool()
 	int y = mInputState.y;
 	bool Redraw = false;
 	mTrackToolFromOverlay = false;
+	lcModel* ActiveModel = GetActiveModel();
 
 	switch (CurrentTool)
 	{
@@ -1893,9 +2021,15 @@ void View::UpdateTrackTool()
 
 			lcVector3 OverlayCenter;
 			lcMatrix33 RelativeRotation;
-			if (!mModel->GetMoveRotateTransform(OverlayCenter, RelativeRotation))
+			if (!ActiveModel->GetMoveRotateTransform(OverlayCenter, RelativeRotation))
 				break;
 
+			lcMatrix44 WorldMatrix = lcMatrix44(RelativeRotation, OverlayCenter);
+
+			if (ActiveModel != mModel)
+				WorldMatrix = lcMul(WorldMatrix, mActiveSubmodelTransform);
+			OverlayCenter = WorldMatrix.GetTranslation();
+
 			lcVector3 PlaneNormals[3] =
 			{
 				lcVector3(1.0f, 0.0f, 0.0f),
@@ -1904,7 +2038,7 @@ void View::UpdateTrackTool()
 			};
 
 			for (int i = 0; i < 3; i++)
-				PlaneNormals[i] = lcMul(PlaneNormals[i], RelativeRotation);
+				PlaneNormals[i] = lcMul30(PlaneNormals[i], WorldMatrix);
 
 			lcVector3 StartEnd[2] = { lcVector3((float)x, (float)y, 0.0f), lcVector3((float)x, (float)y, 1.0f) };
 			UnprojectPoints(StartEnd, 2);
@@ -1912,7 +2046,7 @@ void View::UpdateTrackTool()
 			const lcVector3& End = StartEnd[1];
 			float ClosestIntersectionDistance = FLT_MAX;
 
-			lcObject* Focus = mModel->GetFocusObject();
+			lcObject* Focus = ActiveModel->GetFocusObject();
 			int ControlPointIndex = -1;
 			if (Focus && Focus->IsPiece())
 			{
@@ -1954,7 +2088,7 @@ void View::UpdateTrackTool()
 					}
 				}
 
-				if (CurrentTool == LC_TOOL_SELECT && Proj1 > OverlayRotateArrowStart && Proj1 < OverlayRotateArrowEnd && Proj2 > OverlayRotateArrowStart && Proj2 < OverlayRotateArrowEnd && mModel->AnyPiecesSelected())
+				if (CurrentTool == LC_TOOL_SELECT && Proj1 > OverlayRotateArrowStart && Proj1 < OverlayRotateArrowEnd && Proj2 > OverlayRotateArrowStart && Proj2 < OverlayRotateArrowEnd && ActiveModel->AnyPiecesSelected())
 				{
 					lcTrackTool PlaneModes[] = { LC_TRACKTOOL_ROTATE_X, LC_TRACKTOOL_ROTATE_Y, LC_TRACKTOOL_ROTATE_Z };
 
@@ -2064,7 +2198,7 @@ void View::UpdateTrackTool()
 
 			lcVector3 OverlayCenter;
 			lcMatrix33 RelativeRotation;
-			if (!mModel->GetMoveRotateTransform(OverlayCenter, RelativeRotation))
+			if (!ActiveModel->GetMoveRotateTransform(OverlayCenter, RelativeRotation))
 				break;
 
 			// Calculate the distance from the mouse pointer to the center of the sphere.
@@ -2362,6 +2496,13 @@ bool View::IsTrackToolAllowed(lcTrackTool TrackTool, quint32 AllowedTransforms)
 	return false;
 }
 
+void View::StartOrbitTracking()
+{
+	mTrackTool = LC_TRACKTOOL_ORBIT_XY;
+	OnUpdateCursor();
+	OnButtonDown(LC_TRACKBUTTON_LEFT);
+}
+
 void View::StartTracking(lcTrackButton TrackButton)
 {
 	mTrackButton = TrackButton;
@@ -2369,6 +2510,7 @@ void View::StartTracking(lcTrackButton TrackButton)
 	mMouseDownX = mInputState.x;
 	mMouseDownY = mInputState.y;
 	lcTool Tool = GetCurrentTool();
+	lcModel* ActiveModel = GetActiveModel();
 
 	switch (Tool)
 	{
@@ -2385,7 +2527,7 @@ void View::StartTracking(lcTrackButton TrackButton)
 			};
 
 			UnprojectPoints(PositionTarget, 2);
-			mModel->BeginSpotLightTool(PositionTarget[0], PositionTarget[1]);
+			ActiveModel->BeginSpotLightTool(PositionTarget[0], PositionTarget[1]);
 		}
 		break;
 
@@ -2398,7 +2540,7 @@ void View::StartTracking(lcTrackButton TrackButton)
 			};
 
 			UnprojectPoints(PositionTarget, 2);
-			mModel->BeginCameraTool(PositionTarget[0], PositionTarget[1]);
+			ActiveModel->BeginCameraTool(PositionTarget[0], PositionTarget[1]);
 		}
 		break;
 
@@ -2407,7 +2549,7 @@ void View::StartTracking(lcTrackButton TrackButton)
 
 	case LC_TOOL_MOVE:
 	case LC_TOOL_ROTATE:
-		mModel->BeginMouseTool();
+		ActiveModel->BeginMouseTool();
 		break;
 
 	case LC_TOOL_ERASER:
@@ -2418,7 +2560,7 @@ void View::StartTracking(lcTrackButton TrackButton)
 	case LC_TOOL_PAN:
 	case LC_TOOL_ROTATE_VIEW:
 	case LC_TOOL_ROLL:
-		mModel->BeginMouseTool();
+		ActiveModel->BeginMouseTool();
 		break;
 
 	case LC_TOOL_ZOOM_REGION:
@@ -2437,6 +2579,7 @@ void View::StopTracking(bool Accept)
 		return;
 
 	lcTool Tool = GetCurrentTool();
+	lcModel* ActiveModel = GetActiveModel();
 
 	switch (Tool)
 	{
@@ -2446,7 +2589,7 @@ void View::StopTracking(bool Accept)
 
 	case LC_TOOL_SPOTLIGHT:
 	case LC_TOOL_CAMERA:
-		mModel->EndMouseTool(Tool, Accept);
+		ActiveModel->EndMouseTool(Tool, Accept);
 		break;
 
 	case LC_TOOL_SELECT:
@@ -2455,17 +2598,17 @@ void View::StopTracking(bool Accept)
 			lcArray<lcObject*> Objects = FindObjectsInBox(mMouseDownX, mMouseDownY, mInputState.x, mInputState.y);
 
 			if (mInputState.Modifiers & Qt::ControlModifier)
-				mModel->AddToSelection(Objects, true, true);
+				ActiveModel->AddToSelection(Objects, true, true);
 			else if (mInputState.Modifiers & Qt::ShiftModifier)
-				mModel->RemoveFromSelection(Objects);
+				ActiveModel->RemoveFromSelection(Objects);
 			else
-				mModel->SetSelectionAndFocus(Objects, nullptr, 0, true);
+				ActiveModel->SetSelectionAndFocus(Objects, nullptr, 0, true);
 		}
 		break;
 
 	case LC_TOOL_MOVE:
 	case LC_TOOL_ROTATE:
-		mModel->EndMouseTool(Tool, Accept);
+		ActiveModel->EndMouseTool(Tool, Accept);
 		break;
 
 	case LC_TOOL_ERASER:
@@ -2476,7 +2619,7 @@ void View::StopTracking(bool Accept)
 	case LC_TOOL_PAN:
 	case LC_TOOL_ROTATE_VIEW:
 	case LC_TOOL_ROLL:
-		mModel->EndMouseTool(Tool, Accept);
+		ActiveModel->EndMouseTool(Tool, Accept);
 		break;
 
 	case LC_TOOL_ZOOM_REGION:
@@ -2496,7 +2639,7 @@ void View::StopTracking(bool Accept)
 
 			UnprojectPoints(Points, 5);
 
-			lcVector3 Center = mModel->GetSelectionOrModelCenter();
+			lcVector3 Center = ActiveModel->GetSelectionOrModelCenter();
 
 			lcVector3 PlaneNormal(mCamera->mPosition - mCamera->mTargetPosition);
 			lcVector4 Plane(PlaneNormal, -lcDot(PlaneNormal, Center));
@@ -2505,7 +2648,7 @@ void View::StopTracking(bool Accept)
 			if (lcLineSegmentPlaneIntersection(&Target, Points[0], Points[1], Plane) && lcLineSegmentPlaneIntersection(&Corners[0], Points[2], Points[3], Plane) && lcLineSegmentPlaneIntersection(&Corners[1], Points[3], Points[4], Plane))
 			{
 				float AspectRatio = (float)mWidth / (float)mHeight;
-				mModel->ZoomRegionToolClicked(mCamera, AspectRatio, Points[0], Target, Corners);
+				ActiveModel->ZoomRegionToolClicked(mCamera, AspectRatio, Points[0], Target, Corners);
 			}
 		}
 		break;
@@ -2524,11 +2667,17 @@ void View::CancelTrackingOrClearSelection()
 	if (mTrackButton != LC_TRACKBUTTON_NONE)
 		StopTracking(false);
 	else
-		mModel->ClearSelection(true);
+	{
+		lcModel* ActiveModel = GetActiveModel();
+		if (ActiveModel)
+			ActiveModel->ClearSelection(true);
+	}
 }
 
 void View::OnButtonDown(lcTrackButton TrackButton)
 {
+	lcModel* ActiveModel = GetActiveModel();
+
 	switch (mTrackTool)
 	{
 	case LC_TRACKTOOL_NONE:
@@ -2541,7 +2690,7 @@ void View::OnButtonDown(lcTrackButton TrackButton)
 			if (!CurPiece)
 				break;
 
-			mModel->InsertPieceToolClicked(GetPieceInsertPosition(false, gMainWindow->GetCurrentPieceInfo()));
+			ActiveModel->InsertPieceToolClicked(GetPieceInsertPosition(false, gMainWindow->GetCurrentPieceInfo()));
 
 			if ((mInputState.Modifiers & Qt::ControlModifier) == 0)
 				gMainWindow->SetTool(LC_TOOL_SELECT);
@@ -2552,7 +2701,7 @@ void View::OnButtonDown(lcTrackButton TrackButton)
 
 	case LC_TRACKTOOL_POINTLIGHT:
 		{
-			mModel->PointLightToolClicked(UnprojectPoint(lcVector3((float)mInputState.x, (float)mInputState.y, 0.9f)));
+			ActiveModel->PointLightToolClicked(UnprojectPoint(lcVector3((float)mInputState.x, (float)mInputState.y, 0.9f)));
 
 			if ((mInputState.Modifiers & Qt::ControlModifier) == 0)
 				gMainWindow->SetTool(LC_TOOL_SELECT);
@@ -2571,11 +2720,11 @@ void View::OnButtonDown(lcTrackButton TrackButton)
 			lcObjectSection ObjectSection = FindObjectUnderPointer(false, false);
 
 			if (mInputState.Modifiers & Qt::ControlModifier)
-				mModel->FocusOrDeselectObject(ObjectSection);
+				ActiveModel->FocusOrDeselectObject(ObjectSection);
 			else if (mInputState.Modifiers & Qt::ShiftModifier)
-				mModel->RemoveFromSelection(ObjectSection);
+				ActiveModel->RemoveFromSelection(ObjectSection);
 			else
-				mModel->ClearSelectionAndSetFocus(ObjectSection, true);
+				ActiveModel->ClearSelectionAndSetFocus(ObjectSection, true);
 
 			StartTracking(TrackButton);
 		}
@@ -2588,7 +2737,7 @@ void View::OnButtonDown(lcTrackButton TrackButton)
 	case LC_TRACKTOOL_MOVE_XZ:
 	case LC_TRACKTOOL_MOVE_YZ:
 	case LC_TRACKTOOL_MOVE_XYZ:
-		if (mModel->AnyObjectsSelected())
+		if (ActiveModel->AnyObjectsSelected())
 			StartTracking(TrackButton);
 		break;
 
@@ -2597,22 +2746,22 @@ void View::OnButtonDown(lcTrackButton TrackButton)
 	case LC_TRACKTOOL_ROTATE_Z:
 	case LC_TRACKTOOL_ROTATE_XY:
 	case LC_TRACKTOOL_ROTATE_XYZ:
-		if (mModel->AnyPiecesSelected())
+		if (ActiveModel->AnyPiecesSelected())
 			StartTracking(TrackButton);
 		break;
 
 	case LC_TRACKTOOL_SCALE_PLUS:
 	case LC_TRACKTOOL_SCALE_MINUS:
-		if (mModel->AnyPiecesSelected())
+		if (ActiveModel->AnyPiecesSelected())
 			StartTracking(TrackButton);
 		break;
 
 	case LC_TRACKTOOL_ERASER:
-		mModel->EraserToolClicked(FindObjectUnderPointer(false, false).Object);
+		ActiveModel->EraserToolClicked(FindObjectUnderPointer(false, false).Object);
 		break;
 
 	case LC_TRACKTOOL_PAINT:
-		mModel->PaintToolClicked(FindObjectUnderPointer(true, false).Object);
+		ActiveModel->PaintToolClicked(FindObjectUnderPointer(true, false).Object);
 		break;
 
 	case LC_TRACKTOOL_ZOOM:
@@ -2637,6 +2786,9 @@ void View::OnLeftButtonDown()
 
 	gMainWindow->SetActiveView(this);
 
+	if (mViewSphere.OnLeftButtonDown())
+		return;
+
 	lcTrackTool OverrideTool = GetOverrideTrackTool(Qt::LeftButton);
 
 	if (OverrideTool != LC_TRACKTOOL_NONE)
@@ -2651,6 +2803,9 @@ void View::OnLeftButtonDown()
 void View::OnLeftButtonUp()
 {
 	StopTracking(mTrackButton == LC_TRACKBUTTON_LEFT);
+
+	if (mViewSphere.OnLeftButtonUp())
+		return;
 }
 
 void View::OnLeftButtonDoubleClick()
@@ -2658,13 +2813,14 @@ void View::OnLeftButtonDoubleClick()
 	gMainWindow->SetActiveView(this);
 
 	lcObjectSection ObjectSection = FindObjectUnderPointer(false, false);
+	lcModel* ActiveModel = GetActiveModel();
 
 	if (mInputState.Modifiers & Qt::ControlModifier)
-		mModel->FocusOrDeselectObject(ObjectSection);
+		ActiveModel->FocusOrDeselectObject(ObjectSection);
 	else if (mInputState.Modifiers & Qt::ShiftModifier)
-		mModel->RemoveFromSelection(ObjectSection);
+		ActiveModel->RemoveFromSelection(ObjectSection);
 	else
-		mModel->ClearSelectionAndSetFocus(ObjectSection, true);
+		ActiveModel->ClearSelectionAndSetFocus(ObjectSection, true);
 }
 
 void View::OnMiddleButtonDown()
@@ -2737,11 +2893,26 @@ void View::OnForwardButtonUp()
 
 void View::OnMouseMove()
 {
-	if (!mModel)
+	lcModel* ActiveModel = GetActiveModel();
+
+	if (!ActiveModel)
 		return;
 
 	if (mTrackButton == LC_TRACKBUTTON_NONE)
 	{
+		if (mViewSphere.OnMouseMove())
+		{
+			lcTrackTool NewTrackTool = mViewSphere.IsDragging() ? LC_TRACKTOOL_ORBIT_XY : LC_TRACKTOOL_NONE;
+
+			if (NewTrackTool != mTrackTool)
+			{
+				mTrackTool = NewTrackTool;
+				OnUpdateCursor();
+			}
+
+			return;
+		}
+
 		UpdateTrackTool();
 
 		if (mTrackTool == LC_TRACKTOOL_INSERT)
@@ -2751,7 +2922,7 @@ void View::OnMouseMove()
 	}
 
 	mTrackUpdated = true;
-	const float MouseSensitivity = 1.0f / (21.0f - lcGetPreferences().mMouseSensitivity);
+	const float MouseSensitivity = 0.5f / (21.0f - lcGetPreferences().mMouseSensitivity);
 
 	switch (mTrackTool)
 	{
@@ -2761,11 +2932,11 @@ void View::OnMouseMove()
 		break;
 
 	case LC_TRACKTOOL_SPOTLIGHT:
-		mModel->UpdateSpotLightTool(UnprojectPoint(lcVector3((float)mInputState.x, (float)mInputState.y, 0.9f)));
+		ActiveModel->UpdateSpotLightTool(UnprojectPoint(lcVector3((float)mInputState.x, (float)mInputState.y, 0.9f)));
 		break;
 
 	case LC_TRACKTOOL_CAMERA:
-		mModel->UpdateCameraTool(UnprojectPoint(lcVector3((float)mInputState.x, (float)mInputState.y, 0.9f)));
+		ActiveModel->UpdateCameraTool(UnprojectPoint(lcVector3((float)mInputState.x, (float)mInputState.y, 0.9f)));
 		break;
 
 	case LC_TRACKTOOL_SELECT:
@@ -2799,7 +2970,7 @@ void View::OnMouseMove()
 
 			lcVector3 Center;
 			lcMatrix33 RelativeRotation;
-			mModel->GetMoveRotateTransform(Center, RelativeRotation);
+			ActiveModel->GetMoveRotateTransform(Center, RelativeRotation);
 
 			if (mTrackTool == LC_TRACKTOOL_MOVE_X || mTrackTool == LC_TRACKTOOL_MOVE_Y || mTrackTool == LC_TRACKTOOL_MOVE_Z)
 			{
@@ -2821,7 +2992,7 @@ void View::OnMouseMove()
 
 				lcVector3 Distance = Intersection - MoveStart;
 				Distance = lcMul(Distance, lcMatrix33AffineInverse(RelativeRotation));
-				mModel->UpdateMoveTool(Distance, mTrackButton != LC_TRACKBUTTON_LEFT);
+				ActiveModel->UpdateMoveTool(Distance, mTrackButton != LC_TRACKBUTTON_LEFT);
 			}
 			else if (mTrackTool == LC_TRACKTOOL_MOVE_XY || mTrackTool == LC_TRACKTOOL_MOVE_XZ || mTrackTool == LC_TRACKTOOL_MOVE_YZ)
 			{
@@ -2846,7 +3017,7 @@ void View::OnMouseMove()
 					{
 						lcVector3 Distance = Intersection - MoveStart;
 						Distance = lcMul(Distance, lcMatrix33AffineInverse(RelativeRotation));
-						mModel->UpdateMoveTool(Distance, mTrackButton != LC_TRACKBUTTON_LEFT);
+						ActiveModel->UpdateMoveTool(Distance, mTrackButton != LC_TRACKBUTTON_LEFT);
 					}
 				}
 			}
@@ -2854,7 +3025,7 @@ void View::OnMouseMove()
 			{
 				lcMatrix44 NewPosition = GetPieceInsertPosition(true, mMouseDownPiece);
 				lcVector3 Distance = NewPosition.GetTranslation() - mMouseDownPosition;
-				mModel->UpdateMoveTool(Distance, mTrackButton != LC_TRACKBUTTON_LEFT);
+				ActiveModel->UpdateMoveTool(Distance, mTrackButton != LC_TRACKBUTTON_LEFT);
 			}
 			else if (mTrackTool == LC_TRACKTOOL_SCALE_PLUS || mTrackTool == LC_TRACKTOOL_SCALE_MINUS)
 			{
@@ -2869,7 +3040,7 @@ void View::OnMouseMove()
 				lcVector3 Intersection;
 				lcClosestPointsBetweenLines(Center, Center + Direction, CurrentStart, CurrentEnd, &Intersection, nullptr);
 
-				lcObject* Focus = mModel->GetFocusObject();
+				lcObject* Focus = ActiveModel->GetFocusObject();
 				if (Focus && Focus->IsPiece())
 				{
 					lcPiece* Piece = (lcPiece*)Focus;
@@ -2890,7 +3061,7 @@ void View::OnMouseMove()
 
 						float Scale = lcClamp(Distance, 0.1f, ScaleMax);
 
-						mModel->UpdateScaleTool(Scale);
+						ActiveModel->UpdateScaleTool(Scale);
 					}
 				}
 			}
@@ -2907,7 +3078,7 @@ void View::OnMouseMove()
 					if (lcLineSegmentPlaneIntersection(&MoveStart, MouseDownStart, MouseDownEnd, Plane))
 					{
 						lcVector3 Distance = Intersection - MoveStart;
-						mModel->UpdateMoveTool(Distance, mTrackButton != LC_TRACKBUTTON_LEFT);
+						ActiveModel->UpdateMoveTool(Distance, mTrackButton != LC_TRACKBUTTON_LEFT);
 					}
 				}
 			}
@@ -2965,7 +3136,7 @@ void View::OnMouseMove()
 			MoveX *= 36.0f * (float)(mInputState.x - mMouseDownX) * MouseSensitivity;
 			MoveY *= 36.0f * (float)(mInputState.y - mMouseDownY) * MouseSensitivity;
 
-			mModel->UpdateRotateTool(MoveX + MoveY, mTrackButton != LC_TRACKBUTTON_LEFT);
+			ActiveModel->UpdateRotateTool(MoveX + MoveY, mTrackButton != LC_TRACKBUTTON_LEFT);
 		}
 		break;
 
@@ -2977,7 +3148,7 @@ void View::OnMouseMove()
 
 			lcVector3 MoveX = 36.0f * (float)(mInputState.x - mMouseDownX) * MouseSensitivity * ScreenX;
 			lcVector3 MoveY = 36.0f * (float)(mInputState.y - mMouseDownY) * MouseSensitivity * ScreenY;
-			mModel->UpdateRotateTool(MoveX + MoveY, mTrackButton != LC_TRACKBUTTON_LEFT);
+			ActiveModel->UpdateRotateTool(MoveX + MoveY, mTrackButton != LC_TRACKBUTTON_LEFT);
 		}
 		break;
 
@@ -2985,7 +3156,7 @@ void View::OnMouseMove()
 		{
 			lcVector3 ScreenZ = lcNormalize(mCamera->mTargetPosition - mCamera->mPosition);
 
-			mModel->UpdateRotateTool(36.0f * (float)(mInputState.y - mMouseDownY) * MouseSensitivity * ScreenZ, mTrackButton != LC_TRACKBUTTON_LEFT);
+			ActiveModel->UpdateRotateTool(36.0f * (float)(mInputState.y - mMouseDownY) * MouseSensitivity * ScreenZ, mTrackButton != LC_TRACKBUTTON_LEFT);
 		}
 		break;
 
@@ -2994,7 +3165,7 @@ void View::OnMouseMove()
 		break;
 
 	case LC_TRACKTOOL_ZOOM:
-		mModel->UpdateZoomTool(mCamera, 2.0f * MouseSensitivity * (mInputState.y - mMouseDownY));
+		ActiveModel->UpdateZoomTool(mCamera, 2.0f * MouseSensitivity * (mInputState.y - mMouseDownY));
 		break;
 
 	case LC_TRACKTOOL_PAN:
@@ -3013,7 +3184,7 @@ void View::OnMouseMove()
 			const lcVector3& CurrentEnd = Points[1];
 			const lcVector3& MouseDownStart = Points[2];
 			const lcVector3& MouseDownEnd = Points[3];
-			lcVector3 Center = mModel->GetSelectionOrModelCenter();
+			lcVector3 Center = ActiveModel->GetSelectionOrModelCenter();
 
 			lcVector3 PlaneNormal(mCamera->mPosition - mCamera->mTargetPosition);
 			lcVector4 Plane(PlaneNormal, -lcDot(PlaneNormal, Center));
@@ -3028,24 +3199,24 @@ void View::OnMouseMove()
 					break;
 			}
 
-			mModel->UpdatePanTool(mCamera, MoveStart - Intersection);
+			ActiveModel->UpdatePanTool(mCamera, MoveStart - Intersection);
 		}
 		break;
 
 	case LC_TRACKTOOL_ORBIT_X:
-		mModel->UpdateOrbitTool(mCamera, 0.1f * MouseSensitivity * (mInputState.x - mMouseDownX), 0.0f);
+		ActiveModel->UpdateOrbitTool(mCamera, 0.1f * MouseSensitivity * (mInputState.x - mMouseDownX), 0.0f);
 		break;
 
 	case LC_TRACKTOOL_ORBIT_Y:
-		mModel->UpdateOrbitTool(mCamera, 0.0f, 0.1f * MouseSensitivity * (mInputState.y - mMouseDownY));
+		ActiveModel->UpdateOrbitTool(mCamera, 0.0f, 0.1f * MouseSensitivity * (mInputState.y - mMouseDownY));
 		break;
 
 	case LC_TRACKTOOL_ORBIT_XY:
-		mModel->UpdateOrbitTool(mCamera, 0.1f * MouseSensitivity * (mInputState.x - mMouseDownX), 0.1f * MouseSensitivity * (mInputState.y - mMouseDownY));
+		ActiveModel->UpdateOrbitTool(mCamera, 0.1f * MouseSensitivity * (mInputState.x - mMouseDownX), 0.1f * MouseSensitivity * (mInputState.y - mMouseDownY));
 		break;
 
 	case LC_TRACKTOOL_ROLL:
-		mModel->UpdateRollTool(mCamera, 2.0f * MouseSensitivity * (mInputState.x - mMouseDownX) * LC_DTOR);
+		ActiveModel->UpdateRollTool(mCamera, 2.0f * MouseSensitivity * (mInputState.x - mMouseDownX) * LC_DTOR);
 		break;
 
 	case LC_TRACKTOOL_ZOOM_REGION:
