@@ -10,6 +10,8 @@
 #include "lc_context.h"
 #include "view.h"
 #include "texfont.h"
+#include "lc_viewsphere.h"
+#include "lc_stringcache.h"
 #include "lc_texture.h"
 #include "lc_mesh.h"
 #include "lc_profile.h"
@@ -94,12 +96,14 @@ lcQGLWidget::lcQGLWidget(QWidget *parent, lcGLWidget *owner, bool view)
 	widget->MakeCurrent();
 
 	// TODO: Find a better place for the grid texture and font
+	gStringCache.AddRef(widget->mContext);
 	gTexFont.Load(widget->mContext);
 	if (gWidgetList.isEmpty())
 	{
 		lcInitializeGLExtensions(context());
 		lcContext::CreateResources();
 		View::CreateResources(widget->mContext);
+		lcViewSphere::CreateResources(widget->mContext);
 
 		if (!gSupportsShaderObjects && lcGetPreferences().mShadingMode == LC_SHADING_DEFAULT_LIGHTS)
 			lcGetPreferences().mShadingMode = LC_SHADING_FLAT;
@@ -128,6 +132,7 @@ lcQGLWidget::lcQGLWidget(QWidget *parent, lcGLWidget *owner, bool view)
 lcQGLWidget::~lcQGLWidget()
 {
 	gWidgetList.removeOne(this);
+	gStringCache.Release(widget->mContext);
 	gTexFont.Release();
 	makeCurrent();
 	if (gWidgetList.isEmpty())
@@ -135,6 +140,7 @@ lcQGLWidget::~lcQGLWidget()
 		lcGetPiecesLibrary()->ReleaseBuffers(widget->mContext);
 		View::DestroyResources(widget->mContext);
 		lcContext::DestroyResources();
+		lcViewSphere::DestroyResources(widget->mContext);
 
 		delete gPlaceholderMesh;
 		gPlaceholderMesh = nullptr;
@@ -153,9 +159,6 @@ QSize lcQGLWidget::sizeHint() const
 
 void lcQGLWidget::initializeGL()
 {
-	glEnable(GL_POLYGON_OFFSET_FILL);
-	glPolygonOffset(0.5f, 0.1f);
-
 	glEnable(GL_DEPTH_TEST);
 	glDepthFunc(GL_LEQUAL);
 	glDepthMask(GL_TRUE);
