@@ -14,7 +14,8 @@ public:
 	lcQModelListDialog(QWidget* Parent, QList<QPair<QString, lcModel*>>& Models);
 	~lcQModelListDialog();
 
-	int mActiveModel;
+	int GetActiveModelIndex() const;
+
 	QList<QPair<QString, lcModel*>>& mModels;
 
 public slots:
@@ -25,10 +26,12 @@ public slots:
 	void on_ExportModel_clicked();
 	void on_MoveUp_clicked();
 	void on_MoveDown_clicked();
+	void on_SetActiveModel_clicked();
 	void on_ModelList_itemDoubleClicked(QListWidgetItem* Item);
 	void on_ModelList_currentRowChanged(int CurrentRow);
 
 private:
+	QListWidgetItem* mActiveModelItem;
 	void UpdateButtons();
 	Ui::lcQModelListDialog* ui;
 };
