@@ -10,9 +10,8 @@
 #include "lc_glextensions.h"
 #include "pieceinf.h"
 
-lcQPreferencesDialog::lcQPreferencesDialog(QWidget *parent, void *data) :
-    QDialog(parent),
-    ui(new Ui::lcQPreferencesDialog)
+lcQPreferencesDialog::lcQPreferencesDialog(QWidget* Parent, lcPreferencesDialogOptions* Options)
+	: QDialog(Parent), mOptions(Options), ui(new Ui::lcQPreferencesDialog)
 {
     ui->setupUi(this);
 
@@ -25,53 +24,85 @@ lcQPreferencesDialog::lcQPreferencesDialog(QWidget *parent, void *data) :
 #endif
 
 	ui->lineWidth->setValidator(new QDoubleValidator(ui->lineWidth));
-	connect(ui->gridStudColor, SIGNAL(clicked()), this, SLOT(colorClicked()));
-	connect(ui->gridLineColor, SIGNAL(clicked()), this, SLOT(colorClicked()));
+	connect(ui->gridStudColor, SIGNAL(clicked()), this, SLOT(ColorButtonClicked()));
+	connect(ui->gridLineColor, SIGNAL(clicked()), this, SLOT(ColorButtonClicked()));
+	connect(ui->ViewSphereColorButton, SIGNAL(clicked()), this, SLOT(ColorButtonClicked()));
+	connect(ui->ViewSphereTextColorButton, SIGNAL(clicked()), this, SLOT(ColorButtonClicked()));
+	connect(ui->ViewSphereHighlightColorButton, SIGNAL(clicked()), this, SLOT(ColorButtonClicked()));
 	connect(ui->categoriesTree, SIGNAL(itemSelectionChanged()), this, SLOT(updateParts()));
 	ui->shortcutEdit->installEventFilter(this);
 	connect(ui->commandList, SIGNAL(currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)), this, SLOT(commandChanged(QTreeWidgetItem*)));
 	connect(ui->mouseTree, SIGNAL(currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)), this, SLOT(MouseTreeItemChanged(QTreeWidgetItem*)));
 
-	options = (lcPreferencesDialogOptions*)data;
+	ui->partsLibrary->setText(mOptions->LibraryPath);
+	ui->ColorConfigEdit->setText(mOptions->ColorConfigPath);
+	ui->MinifigSettingsEdit->setText(mOptions->MinifigSettingsPath);
+	ui->povrayExecutable->setText(mOptions->POVRayPath);
+	ui->lgeoPath->setText(mOptions->LGEOPath);
+	ui->authorName->setText(mOptions->DefaultAuthor);
+	ui->mouseSensitivity->setValue(mOptions->Preferences.mMouseSensitivity);
+	ui->checkForUpdates->setCurrentIndex(mOptions->CheckForUpdates);
+	ui->fixedDirectionKeys->setChecked(mOptions->Preferences.mFixedAxes);
+	ui->autoLoadMostRecent->setChecked(mOptions->Preferences.mAutoLoadMostRecent);
 
-	ui->authorName->setText(options->DefaultAuthor);
-	ui->partsLibrary->setText(options->LibraryPath);
-	ui->povrayExecutable->setText(options->POVRayPath);
-	ui->lgeoPath->setText(options->LGEOPath);
-	ui->mouseSensitivity->setValue(options->Preferences.mMouseSensitivity);
-	ui->checkForUpdates->setCurrentIndex(options->CheckForUpdates);
-	ui->fixedDirectionKeys->setChecked((options->Preferences.mFixedAxes) != 0);
-
-	ui->antiAliasing->setChecked(options->AASamples != 1);
-	if (options->AASamples == 8)
+	ui->antiAliasing->setChecked(mOptions->AASamples != 1);
+	if (mOptions->AASamples == 8)
 		ui->antiAliasingSamples->setCurrentIndex(2);
-	else if (options->AASamples == 4)
+	else if (mOptions->AASamples == 4)
 		ui->antiAliasingSamples->setCurrentIndex(1);
 	else
 		ui->antiAliasingSamples->setCurrentIndex(0);
-	ui->edgeLines->setChecked(options->Preferences.mDrawEdgeLines);
-	ui->lineWidth->setText(lcFormatValueLocalized(options->Preferences.mLineWidth));
-	ui->gridStuds->setChecked(options->Preferences.mDrawGridStuds);
-	ui->gridLines->setChecked(options->Preferences.mDrawGridLines);
-	ui->gridLineSpacing->setText(QString::number(options->Preferences.mGridLineSpacing));
-	ui->axisIcon->setChecked(options->Preferences.mDrawAxes);
+	ui->edgeLines->setChecked(mOptions->Preferences.mDrawEdgeLines);
+	ui->lineWidth->setText(lcFormatValueLocalized(mOptions->Preferences.mLineWidth));
+	ui->gridStuds->setChecked(mOptions->Preferences.mDrawGridStuds);
+	ui->gridLines->setChecked(mOptions->Preferences.mDrawGridLines);
+	ui->gridLineSpacing->setText(QString::number(mOptions->Preferences.mGridLineSpacing));
+	ui->axisIcon->setChecked(mOptions->Preferences.mDrawAxes);
+
+	ui->ViewSphereLocationCombo->setCurrentIndex((int)mOptions->Preferences.mViewSphereLocation);
+
+	switch (mOptions->Preferences.mViewSphereSize)
+	{
+	case 200:
+		ui->ViewSphereSizeCombo->setCurrentIndex(3);
+		break;
+	case 100:
+		ui->ViewSphereSizeCombo->setCurrentIndex(2);
+		break;
+	case 50:
+		ui->ViewSphereSizeCombo->setCurrentIndex(1);
+		break;
+	default:
+		ui->ViewSphereSizeCombo->setCurrentIndex(0);
+		break;
+	}
 
 	if (!gSupportsShaderObjects)
 		ui->ShadingMode->removeItem(LC_SHADING_DEFAULT_LIGHTS);
-	ui->ShadingMode->setCurrentIndex(options->Preferences.mShadingMode);
+	ui->ShadingMode->setCurrentIndex(mOptions->Preferences.mShadingMode);
 
 	QPixmap pix(12, 12);
 
-	pix.fill(QColor(LC_RGBA_RED(options->Preferences.mGridStudColor), LC_RGBA_GREEN(options->Preferences.mGridStudColor), LC_RGBA_BLUE(options->Preferences.mGridStudColor)));
+	pix.fill(QColor(LC_RGBA_RED(mOptions->Preferences.mGridStudColor), LC_RGBA_GREEN(mOptions->Preferences.mGridStudColor), LC_RGBA_BLUE(mOptions->Preferences.mGridStudColor)));
 	ui->gridStudColor->setIcon(pix);
 
-	pix.fill(QColor(LC_RGBA_RED(options->Preferences.mGridLineColor), LC_RGBA_GREEN(options->Preferences.mGridLineColor), LC_RGBA_BLUE(options->Preferences.mGridLineColor)));
+	pix.fill(QColor(LC_RGBA_RED(mOptions->Preferences.mGridLineColor), LC_RGBA_GREEN(mOptions->Preferences.mGridLineColor), LC_RGBA_BLUE(mOptions->Preferences.mGridLineColor)));
 	ui->gridLineColor->setIcon(pix);
 
+	pix.fill(QColor(LC_RGBA_RED(mOptions->Preferences.mViewSphereColor), LC_RGBA_GREEN(mOptions->Preferences.mViewSphereColor), LC_RGBA_BLUE(mOptions->Preferences.mViewSphereColor)));
+	ui->ViewSphereColorButton->setIcon(pix);
+
+	pix.fill(QColor(LC_RGBA_RED(mOptions->Preferences.mViewSphereTextColor), LC_RGBA_GREEN(mOptions->Preferences.mViewSphereTextColor), LC_RGBA_BLUE(mOptions->Preferences.mViewSphereTextColor)));
+	ui->ViewSphereTextColorButton->setIcon(pix);
+
+	pix.fill(QColor(LC_RGBA_RED(mOptions->Preferences.mViewSphereHighlightColor), LC_RGBA_GREEN(mOptions->Preferences.mViewSphereHighlightColor), LC_RGBA_BLUE(mOptions->Preferences.mViewSphereHighlightColor)));
+	ui->ViewSphereHighlightColorButton->setIcon(pix);
+
 	on_antiAliasing_toggled();
 	on_edgeLines_toggled();
 	on_gridStuds_toggled();
 	on_gridLines_toggled();
+	on_ViewSphereSizeCombo_currentIndexChanged(ui->ViewSphereSizeCombo->currentIndex());
 
 	updateCategories();
 	ui->categoriesTree->setCurrentItem(ui->categoriesTree->topLevelItem(0));
@@ -107,44 +138,89 @@ void lcQPreferencesDialog::accept()
 		return;
 	}
 
-	options->DefaultAuthor = ui->authorName->text();
-	options->LibraryPath = ui->partsLibrary->text();
-	options->POVRayPath = ui->povrayExecutable->text();
-	options->LGEOPath = ui->lgeoPath->text();
-	options->Preferences.mMouseSensitivity = ui->mouseSensitivity->value();
-	options->CheckForUpdates = ui->checkForUpdates->currentIndex();
-	options->Preferences.mFixedAxes = ui->fixedDirectionKeys->isChecked();
+	mOptions->LibraryPath = ui->partsLibrary->text();
+	mOptions->MinifigSettingsPath = ui->MinifigSettingsEdit->text();
+	mOptions->ColorConfigPath = ui->ColorConfigEdit->text();
+	mOptions->POVRayPath = ui->povrayExecutable->text();
+	mOptions->LGEOPath = ui->lgeoPath->text();
+	mOptions->DefaultAuthor = ui->authorName->text();
+	mOptions->Preferences.mMouseSensitivity = ui->mouseSensitivity->value();
+	mOptions->CheckForUpdates = ui->checkForUpdates->currentIndex();
+	mOptions->Preferences.mFixedAxes = ui->fixedDirectionKeys->isChecked();
+	mOptions->Preferences.mAutoLoadMostRecent = ui->autoLoadMostRecent->isChecked();
 
 	if (!ui->antiAliasing->isChecked())
-		options->AASamples = 1;
+		mOptions->AASamples = 1;
 	else if (ui->antiAliasingSamples->currentIndex() == 2)
-		options->AASamples = 8;
+		mOptions->AASamples = 8;
 	else if (ui->antiAliasingSamples->currentIndex() == 1)
-		options->AASamples = 4;
+		mOptions->AASamples = 4;
 	else
-		options->AASamples = 2;
+		mOptions->AASamples = 2;
 
-	options->Preferences.mDrawEdgeLines = ui->edgeLines->isChecked();
-	options->Preferences.mLineWidth = lcParseValueLocalized(ui->lineWidth->text());
+	mOptions->Preferences.mDrawEdgeLines = ui->edgeLines->isChecked();
+	mOptions->Preferences.mLineWidth = lcParseValueLocalized(ui->lineWidth->text());
 
-	options->Preferences.mDrawGridStuds = ui->gridStuds->isChecked();
-	options->Preferences.mDrawGridLines = ui->gridLines->isChecked();
-	options->Preferences.mGridLineSpacing = gridLineSpacing;
+	mOptions->Preferences.mDrawGridStuds = ui->gridStuds->isChecked();
+	mOptions->Preferences.mDrawGridLines = ui->gridLines->isChecked();
+	mOptions->Preferences.mGridLineSpacing = gridLineSpacing;
 
-	options->Preferences.mDrawAxes = ui->axisIcon->isChecked();
-	options->Preferences.mShadingMode = (lcShadingMode)ui->ShadingMode->currentIndex();
+	mOptions->Preferences.mDrawAxes = ui->axisIcon->isChecked();
+	mOptions->Preferences.mViewSphereLocation = (lcViewSphereLocation)ui->ViewSphereLocationCombo->currentIndex();
+
+	switch (ui->ViewSphereSizeCombo->currentIndex())
+	{
+	case 3:
+		mOptions->Preferences.mViewSphereSize = 200;
+		break;
+	case 2:
+		mOptions->Preferences.mViewSphereSize = 100;
+		break;
+	case 1:
+		mOptions->Preferences.mViewSphereSize = 50;
+		break;
+	default:
+		mOptions->Preferences.mViewSphereSize = 0;
+		break;
+	}
+
+	mOptions->Preferences.mShadingMode = (lcShadingMode)ui->ShadingMode->currentIndex();
 
 	QDialog::accept();
 }
 
 void lcQPreferencesDialog::on_partsLibraryBrowse_clicked()
 {
-	QString result = QFileDialog::getExistingDirectory(this, tr("Open Parts Library Folder"), ui->partsLibrary->text());
+	QString result = QFileDialog::getExistingDirectory(this, tr("Select Parts Library Folder"), ui->partsLibrary->text());
 
 	if (!result.isEmpty())
 		ui->partsLibrary->setText(QDir::toNativeSeparators(result));
 }
 
+void lcQPreferencesDialog::on_partsArchiveBrowse_clicked()
+{
+	QString result = QFileDialog::getOpenFileName(this, tr("Select Parts Library Archive"), ui->partsLibrary->text(), tr("Supported Archives (*.zip *.bin);;All Files (*.*)"));
+
+	if (!result.isEmpty())
+		ui->partsLibrary->setText(QDir::toNativeSeparators(result));
+}
+
+void lcQPreferencesDialog::on_ColorConfigBrowseButton_clicked()
+{
+	QString Result = QFileDialog::getOpenFileName(this, tr("Select Color Configuration File"), ui->ColorConfigEdit->text(), tr("Settings Files (*.ldr);;All Files (*.*)"));
+
+	if (!Result.isEmpty())
+		ui->ColorConfigEdit->setText(QDir::toNativeSeparators(Result));
+}
+
+void lcQPreferencesDialog::on_MinifigSettingsBrowseButton_clicked()
+{
+	QString Result = QFileDialog::getOpenFileName(this, tr("Select Minifig Settings File"), ui->MinifigSettingsEdit->text(), tr("Settings Files (*.ini);;All Files (*.*)"));
+
+	if (!Result.isEmpty())
+		ui->MinifigSettingsEdit->setText(QDir::toNativeSeparators(Result));
+}
+
 void lcQPreferencesDialog::on_povrayExecutableBrowse_clicked()
 {
 #ifdef Q_OS_WIN
@@ -153,7 +229,7 @@ void lcQPreferencesDialog::on_povrayExecutableBrowse_clicked()
 	QString filter(tr("All Files (*.*)"));
 #endif
 
-	QString result = QFileDialog::getOpenFileName(this, tr("Open POV-Ray Executable"), ui->povrayExecutable->text(), filter);
+	QString result = QFileDialog::getOpenFileName(this, tr("Select POV-Ray Executable"), ui->povrayExecutable->text(), filter);
 
 	if (!result.isEmpty())
 		ui->povrayExecutable->setText(QDir::toNativeSeparators(result));
@@ -161,13 +237,13 @@ void lcQPreferencesDialog::on_povrayExecutableBrowse_clicked()
 
 void lcQPreferencesDialog::on_lgeoPathBrowse_clicked()
 {
-	QString result = QFileDialog::getExistingDirectory(this, tr("Open LGEO Folder"), ui->lgeoPathBrowse->text());
+	QString result = QFileDialog::getExistingDirectory(this, tr("Open LGEO Folder"), ui->lgeoPath->text());
 
 	if (!result.isEmpty())
-		ui->lgeoPathBrowse->setText(QDir::toNativeSeparators(result));
+		ui->lgeoPath->setText(QDir::toNativeSeparators(result));
 }
 
-void lcQPreferencesDialog::colorClicked()
+void lcQPreferencesDialog::ColorButtonClicked()
 {
 	QObject *button = sender();
 	QString title;
@@ -176,16 +252,34 @@ void lcQPreferencesDialog::colorClicked()
 
 	if (button == ui->gridStudColor)
 	{
-		color = &options->Preferences.mGridStudColor;
+		color = &mOptions->Preferences.mGridStudColor;
 		title = tr("Select Grid Stud Color");
 		dialogOptions = QColorDialog::ShowAlphaChannel;
 	}
 	else if (button == ui->gridLineColor)
 	{
-		color = &options->Preferences.mGridLineColor;
+		color = &mOptions->Preferences.mGridLineColor;
 		title = tr("Select Grid Line Color");
 		dialogOptions = 0;
 	}
+	else if (button == ui->ViewSphereColorButton)
+	{
+		color = &mOptions->Preferences.mViewSphereColor;
+		title = tr("Select View Sphere Color");
+		dialogOptions = 0;
+	}
+	else if (button == ui->ViewSphereTextColorButton)
+	{
+		color = &mOptions->Preferences.mViewSphereTextColor;
+		title = tr("Select View Sphere Text Color");
+		dialogOptions = 0;
+	}
+	else if (button == ui->ViewSphereHighlightColorButton)
+	{
+		color = &mOptions->Preferences.mViewSphereHighlightColor;
+		title = tr("Select View Sphere Highlight Color");
+		dialogOptions = 0;
+	}
 	else
 		return;
 
@@ -221,6 +315,17 @@ void lcQPreferencesDialog::on_gridStuds_toggled()
 void lcQPreferencesDialog::on_gridLines_toggled()
 {
 	ui->gridLineColor->setEnabled(ui->gridLines->isChecked());
+	ui->gridLineSpacing->setEnabled(ui->gridLines->isChecked());
+}
+
+void lcQPreferencesDialog::on_ViewSphereSizeCombo_currentIndexChanged(int Index)
+{
+	bool Enabled = Index != 0;
+
+	ui->ViewSphereLocationCombo->setEnabled(Enabled);
+	ui->ViewSphereColorButton->setEnabled(Enabled);
+	ui->ViewSphereTextColorButton->setEnabled(Enabled);
+	ui->ViewSphereHighlightColorButton->setEnabled(Enabled);
 }
 
 void lcQPreferencesDialog::updateCategories()
@@ -230,9 +335,9 @@ void lcQPreferencesDialog::updateCategories()
 
 	tree->clear();
 
-	for (int categoryIndex = 0; categoryIndex < options->Categories.GetSize(); categoryIndex++)
+	for (int categoryIndex = 0; categoryIndex < mOptions->Categories.GetSize(); categoryIndex++)
 	{
-		categoryItem = new QTreeWidgetItem(tree, QStringList(options->Categories[categoryIndex].Name));
+		categoryItem = new QTreeWidgetItem(tree, QStringList(mOptions->Categories[categoryIndex].Name));
 		categoryItem->setData(0, CategoryRole, QVariant(categoryIndex));
 	}
 
@@ -259,7 +364,7 @@ void lcQPreferencesDialog::updateParts()
 	{
 		lcArray<PieceInfo*> singleParts, groupedParts;
 
-		Library->GetCategoryEntries(options->Categories[categoryIndex].Keywords.constData(), false, singleParts, groupedParts);
+		Library->GetCategoryEntries(mOptions->Categories[categoryIndex].Keywords.constData(), false, singleParts, groupedParts);
 
 		for (int partIndex = 0; partIndex < singleParts.GetSize(); partIndex++)
 		{
@@ -273,17 +378,17 @@ void lcQPreferencesDialog::updateParts()
 	}
 	else
 	{
-		for (const auto PartIt : Library->mPieces)
+		for (const auto& PartIt : Library->mPieces)
 		{
 			PieceInfo* Info = PartIt.second;
 
-			for (categoryIndex = 0; categoryIndex < options->Categories.GetSize(); categoryIndex++)
+			for (categoryIndex = 0; categoryIndex < mOptions->Categories.GetSize(); categoryIndex++)
 			{
-				if (Library->PieceInCategory(Info, options->Categories[categoryIndex].Keywords.constData()))
+				if (Library->PieceInCategory(Info, mOptions->Categories[categoryIndex].Keywords.constData()))
 					break;
 			}
 
-			if (categoryIndex == options->Categories.GetSize())
+			if (categoryIndex == mOptions->Categories.GetSize())
 			{
 				QStringList rowList(Info->m_strDescription);
 				rowList.append(Info->mFileName);
@@ -305,12 +410,12 @@ void lcQPreferencesDialog::on_newCategory_clicked()
 	if (dialog.exec() != QDialog::Accepted)
 		return;
 
-	options->CategoriesModified = true;
-	options->CategoriesDefault = false;
-	options->Categories.Add(category);
+	mOptions->CategoriesModified = true;
+	mOptions->CategoriesDefault = false;
+	mOptions->Categories.Add(category);
 
 	updateCategories();
-	ui->categoriesTree->setCurrentItem(ui->categoriesTree->topLevelItem(options->Categories.GetSize() - 1));
+	ui->categoriesTree->setCurrentItem(ui->categoriesTree->topLevelItem(mOptions->Categories.GetSize() - 1));
 }
 
 void lcQPreferencesDialog::on_editCategory_clicked()
@@ -326,12 +431,12 @@ void lcQPreferencesDialog::on_editCategory_clicked()
 	if (categoryIndex == -1)
 		return;
 
-	lcQCategoryDialog dialog(this, &options->Categories[categoryIndex]);
+	lcQCategoryDialog dialog(this, &mOptions->Categories[categoryIndex]);
 	if (dialog.exec() != QDialog::Accepted)
 		return;
 
-	options->CategoriesModified = true;
-	options->CategoriesDefault = false;
+	mOptions->CategoriesModified = true;
+	mOptions->CategoriesDefault = false;
 
 	updateCategories();
 	ui->categoriesTree->setCurrentItem(ui->categoriesTree->topLevelItem(categoryIndex));
@@ -350,13 +455,13 @@ void lcQPreferencesDialog::on_deleteCategory_clicked()
 	if (categoryIndex == -1)
 		return;
 
-	QString question = tr("Are you sure you want to delete the category '%1'?").arg(options->Categories[categoryIndex].Name);
+	QString question = tr("Are you sure you want to delete the category '%1'?").arg(mOptions->Categories[categoryIndex].Name);
 	if (QMessageBox::question(this, "LeoCAD", question, QMessageBox::Yes | QMessageBox::No) != QMessageBox::Yes)
 		return;
 
-	options->CategoriesModified = true;
-	options->CategoriesDefault = false;
-	options->Categories.RemoveIndex(categoryIndex);
+	mOptions->CategoriesModified = true;
+	mOptions->CategoriesDefault = false;
+	mOptions->Categories.RemoveIndex(categoryIndex);
 
 	updateCategories();
 }
@@ -375,9 +480,9 @@ void lcQPreferencesDialog::on_importCategories_clicked()
 		return;
 	}
 
-	options->Categories = Categories;
-	options->CategoriesModified = true;
-	options->CategoriesDefault = false;
+	mOptions->Categories = Categories;
+	mOptions->CategoriesModified = true;
+	mOptions->CategoriesDefault = false;
 }
 
 void lcQPreferencesDialog::on_exportCategories_clicked()
@@ -387,7 +492,7 @@ void lcQPreferencesDialog::on_exportCategories_clicked()
 	if (FileName.isEmpty())
 		return;
 
-	if (!lcSaveCategories(FileName, options->Categories))
+	if (!lcSaveCategories(FileName, mOptions->Categories))
 	{
 		QMessageBox::warning(this, "LeoCAD", tr("Error saving categories file."));
 		return;
@@ -399,10 +504,10 @@ void lcQPreferencesDialog::on_resetCategories_clicked()
 	if (QMessageBox::question(this, "LeoCAD", tr("Are you sure you want to load the default categories?"), QMessageBox::Yes | QMessageBox::No) != QMessageBox::Yes)
 		return;
 
-	lcResetCategories(options->Categories);
+	lcResetCategories(mOptions->Categories);
 
-	options->CategoriesModified = true;
-	options->CategoriesDefault = true;
+	mOptions->CategoriesModified = true;
+	mOptions->CategoriesDefault = true;
 
 	updateCategories();
 }
@@ -505,12 +610,12 @@ void lcQPreferencesDialog::updateCommandList()
 		}
 
 		QTreeWidgetItem *item = new QTreeWidgetItem;
-		QKeySequence sequence(options->KeyboardShortcuts.mShortcuts[actionIdx]);
+		QKeySequence sequence(mOptions->KeyboardShortcuts.mShortcuts[actionIdx]);
 		item->setText(0, subId);
 		item->setText(1, sequence.toString(QKeySequence::NativeText));
 		item->setData(0, Qt::UserRole, qVariantFromValue(actionIdx));
 
-		if (options->KeyboardShortcuts.mShortcuts[actionIdx] != gCommands[actionIdx].DefaultShortcut)
+		if (mOptions->KeyboardShortcuts.mShortcuts[actionIdx] != gCommands[actionIdx].DefaultShortcut)
 			setShortcutModified(item, true);
 
 		sections[section]->addChild(item);
@@ -538,10 +643,43 @@ void lcQPreferencesDialog::commandChanged(QTreeWidgetItem *current)
 	ui->shortcutGroup->setEnabled(true);
 
 	int shortcutIndex = qvariant_cast<int>(current->data(0, Qt::UserRole));
-	QKeySequence key(options->KeyboardShortcuts.mShortcuts[shortcutIndex]);
+	QKeySequence key(mOptions->KeyboardShortcuts.mShortcuts[shortcutIndex]);
 	ui->shortcutEdit->setText(key.toString(QKeySequence::NativeText));
 }
 
+void lcQPreferencesDialog::on_KeyboardFilterEdit_textEdited(const QString& Text)
+{
+	if (Text.isEmpty())
+	{
+		std::function<void(QTreeWidgetItem*)> ShowItems = [&ShowItems](QTreeWidgetItem* ParentItem)
+		{
+			for (int ChildIdx = 0; ChildIdx < ParentItem->childCount(); ChildIdx++)
+				ShowItems(ParentItem->child(ChildIdx));
+
+			ParentItem->setHidden(false);
+		};
+
+		ShowItems(ui->commandList->invisibleRootItem());
+	}
+	else
+	{
+		std::function<bool(QTreeWidgetItem*,bool)> ShowItems = [&ShowItems, &Text](QTreeWidgetItem* ParentItem, bool ForceVisible)
+		{
+			ForceVisible |= (bool)ParentItem->text(0).contains(Text, Qt::CaseInsensitive) | (bool)ParentItem->text(1).contains(Text, Qt::CaseInsensitive);
+			bool Visible = ForceVisible;
+
+			for (int ChildIdx = 0; ChildIdx < ParentItem->childCount(); ChildIdx++)
+				Visible |= ShowItems(ParentItem->child(ChildIdx), ForceVisible);
+
+			ParentItem->setHidden(!Visible);
+
+			return Visible;
+		};
+
+		ShowItems(ui->commandList->invisibleRootItem(), false);
+	}
+}
+
 void lcQPreferencesDialog::on_shortcutAssign_clicked()
 {
     QTreeWidgetItem *current = ui->commandList->currentItem();
@@ -550,14 +688,14 @@ void lcQPreferencesDialog::on_shortcutAssign_clicked()
 		return;
 
 	int shortcutIndex = qvariant_cast<int>(current->data(0, Qt::UserRole));
-	options->KeyboardShortcuts.mShortcuts[shortcutIndex] = ui->shortcutEdit->text();
+	mOptions->KeyboardShortcuts.mShortcuts[shortcutIndex] = ui->shortcutEdit->text();
 
 	current->setText(1, ui->shortcutEdit->text());
 
-	setShortcutModified(current, options->KeyboardShortcuts.mShortcuts[shortcutIndex] != gCommands[shortcutIndex].DefaultShortcut);
+	setShortcutModified(current, mOptions->KeyboardShortcuts.mShortcuts[shortcutIndex] != gCommands[shortcutIndex].DefaultShortcut);
 
-	options->KeyboardShortcutsModified = true;
-	options->KeyboardShortcutsDefault = false;
+	mOptions->KeyboardShortcutsModified = true;
+	mOptions->KeyboardShortcutsDefault = false;
 }
 
 void lcQPreferencesDialog::on_shortcutRemove_clicked()
@@ -581,10 +719,10 @@ void lcQPreferencesDialog::on_shortcutsImport_clicked()
 		return;
 	}
 
-	options->KeyboardShortcuts = Shortcuts;
+	mOptions->KeyboardShortcuts = Shortcuts;
 
-	options->KeyboardShortcutsModified = true;
-	options->KeyboardShortcutsDefault = false;
+	mOptions->KeyboardShortcutsModified = true;
+	mOptions->KeyboardShortcutsDefault = false;
 }
 
 void lcQPreferencesDialog::on_shortcutsExport_clicked()
@@ -594,7 +732,7 @@ void lcQPreferencesDialog::on_shortcutsExport_clicked()
 	if (FileName.isEmpty())
 		return;
 
-	if (!options->KeyboardShortcuts.Save(FileName))
+	if (!mOptions->KeyboardShortcuts.Save(FileName))
 	{
 		QMessageBox::warning(this, "LeoCAD", tr("Error saving keyboard shortcuts file."));
 		return;
@@ -606,11 +744,11 @@ void lcQPreferencesDialog::on_shortcutsReset_clicked()
 	if (QMessageBox::question(this, "LeoCAD", tr("Are you sure you want to load the default keyboard shortcuts?"), QMessageBox::Yes | QMessageBox::No) != QMessageBox::Yes)
 		return;
 
-	options->KeyboardShortcuts.Reset();
+	mOptions->KeyboardShortcuts.Reset();
 	updateCommandList();
 
-	options->KeyboardShortcutsModified = true;
-	options->KeyboardShortcutsDefault = true;
+	mOptions->KeyboardShortcutsModified = true;
+	mOptions->KeyboardShortcutsDefault = true;
 }
 
 void lcQPreferencesDialog::UpdateMouseTree()
@@ -623,7 +761,7 @@ void lcQPreferencesDialog::UpdateMouseTree()
 
 void lcQPreferencesDialog::UpdateMouseTreeItem(int ItemIndex)
 {
-	auto GetShortcutText = [this](Qt::MouseButton Button, Qt::KeyboardModifiers Modifiers)
+	auto GetShortcutText = [](Qt::MouseButton Button, Qt::KeyboardModifiers Modifiers)
 	{
 		QString Shortcut = QKeySequence(Modifiers).toString(QKeySequence::NativeText);
 
@@ -649,8 +787,8 @@ void lcQPreferencesDialog::UpdateMouseTreeItem(int ItemIndex)
 		return Shortcut;
 	};
 
-	QString Shortcut1 = GetShortcutText(options->MouseShortcuts.mShortcuts[ItemIndex].Button1, options->MouseShortcuts.mShortcuts[ItemIndex].Modifiers1);
-	QString Shortcut2 = GetShortcutText(options->MouseShortcuts.mShortcuts[ItemIndex].Button2, options->MouseShortcuts.mShortcuts[ItemIndex].Modifiers2);
+	QString Shortcut1 = GetShortcutText(mOptions->MouseShortcuts.mShortcuts[ItemIndex].Button1, mOptions->MouseShortcuts.mShortcuts[ItemIndex].Modifiers1);
+	QString Shortcut2 = GetShortcutText(mOptions->MouseShortcuts.mShortcuts[ItemIndex].Button2, mOptions->MouseShortcuts.mShortcuts[ItemIndex].Modifiers2);
 
 	QTreeWidgetItem* Item = ui->mouseTree->topLevelItem(ItemIndex);
 
@@ -707,36 +845,36 @@ void lcQPreferencesDialog::on_mouseAssign_clicked()
 			if (ToolIdx == ButtonIndex)
 				continue;
 
-			if (options->MouseShortcuts.mShortcuts[ToolIdx].Button2 == Button && options->MouseShortcuts.mShortcuts[ToolIdx].Modifiers2 == Modifiers)
+			if (mOptions->MouseShortcuts.mShortcuts[ToolIdx].Button2 == Button && mOptions->MouseShortcuts.mShortcuts[ToolIdx].Modifiers2 == Modifiers)
 			{
 				if (QMessageBox::question(this, tr("Override Shortcut"), tr("This shortcut is already assigned to '%1', do you want to replace it?").arg(tr(gToolNames[ToolIdx])), QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
 					return;
 
-				options->MouseShortcuts.mShortcuts[ToolIdx].Button2 = Qt::NoButton;
-				options->MouseShortcuts.mShortcuts[ToolIdx].Modifiers2 = Qt::NoModifier;
+				mOptions->MouseShortcuts.mShortcuts[ToolIdx].Button2 = Qt::NoButton;
+				mOptions->MouseShortcuts.mShortcuts[ToolIdx].Modifiers2 = Qt::NoModifier;
 			}
 
-			if (options->MouseShortcuts.mShortcuts[ToolIdx].Button1 == Button && options->MouseShortcuts.mShortcuts[ToolIdx].Modifiers1 == Modifiers)
+			if (mOptions->MouseShortcuts.mShortcuts[ToolIdx].Button1 == Button && mOptions->MouseShortcuts.mShortcuts[ToolIdx].Modifiers1 == Modifiers)
 			{
 				if (QMessageBox::question(this, tr("Override Shortcut"), tr("This shortcut is already assigned to '%1', do you want to replace it?").arg(tr(gToolNames[ToolIdx])), QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
 					return;
 
-				options->MouseShortcuts.mShortcuts[ToolIdx].Button1 = options->MouseShortcuts.mShortcuts[ToolIdx].Button2;
-				options->MouseShortcuts.mShortcuts[ToolIdx].Modifiers1 = options->MouseShortcuts.mShortcuts[ToolIdx].Modifiers2;
-				options->MouseShortcuts.mShortcuts[ToolIdx].Button2 = Qt::NoButton;
-				options->MouseShortcuts.mShortcuts[ToolIdx].Modifiers2 = Qt::NoModifier;
+				mOptions->MouseShortcuts.mShortcuts[ToolIdx].Button1 = mOptions->MouseShortcuts.mShortcuts[ToolIdx].Button2;
+				mOptions->MouseShortcuts.mShortcuts[ToolIdx].Modifiers1 = mOptions->MouseShortcuts.mShortcuts[ToolIdx].Modifiers2;
+				mOptions->MouseShortcuts.mShortcuts[ToolIdx].Button2 = Qt::NoButton;
+				mOptions->MouseShortcuts.mShortcuts[ToolIdx].Modifiers2 = Qt::NoModifier;
 			}
 		}
 	}
 
 	int ItemIndex = ui->mouseTree->indexOfTopLevelItem(Current);
-	options->MouseShortcuts.mShortcuts[ItemIndex].Button2 = options->MouseShortcuts.mShortcuts[ItemIndex].Button1;
-	options->MouseShortcuts.mShortcuts[ItemIndex].Modifiers2 = options->MouseShortcuts.mShortcuts[ItemIndex].Modifiers1;
-	options->MouseShortcuts.mShortcuts[ItemIndex].Button1 = Button;
-	options->MouseShortcuts.mShortcuts[ItemIndex].Modifiers1 = Modifiers;
+	mOptions->MouseShortcuts.mShortcuts[ItemIndex].Button2 = mOptions->MouseShortcuts.mShortcuts[ItemIndex].Button1;
+	mOptions->MouseShortcuts.mShortcuts[ItemIndex].Modifiers2 = mOptions->MouseShortcuts.mShortcuts[ItemIndex].Modifiers1;
+	mOptions->MouseShortcuts.mShortcuts[ItemIndex].Button1 = Button;
+	mOptions->MouseShortcuts.mShortcuts[ItemIndex].Modifiers1 = Modifiers;
 
-	options->MouseShortcutsModified = true;
-	options->MouseShortcutsDefault = false;
+	mOptions->MouseShortcutsModified = true;
+	mOptions->MouseShortcutsDefault = false;
 
 	UpdateMouseTreeItem(ItemIndex);
 }
@@ -749,13 +887,13 @@ void lcQPreferencesDialog::on_mouseRemove_clicked()
 		return;
 
 	int ItemIndex = ui->mouseTree->indexOfTopLevelItem(Current);
-	options->MouseShortcuts.mShortcuts[ItemIndex].Button1 = options->MouseShortcuts.mShortcuts[ItemIndex].Button2;
-	options->MouseShortcuts.mShortcuts[ItemIndex].Modifiers1 = options->MouseShortcuts.mShortcuts[ItemIndex].Modifiers2;
-	options->MouseShortcuts.mShortcuts[ItemIndex].Button2 = Qt::NoButton;
-	options->MouseShortcuts.mShortcuts[ItemIndex].Modifiers2 = Qt::NoModifier;
+	mOptions->MouseShortcuts.mShortcuts[ItemIndex].Button1 = mOptions->MouseShortcuts.mShortcuts[ItemIndex].Button2;
+	mOptions->MouseShortcuts.mShortcuts[ItemIndex].Modifiers1 = mOptions->MouseShortcuts.mShortcuts[ItemIndex].Modifiers2;
+	mOptions->MouseShortcuts.mShortcuts[ItemIndex].Button2 = Qt::NoButton;
+	mOptions->MouseShortcuts.mShortcuts[ItemIndex].Modifiers2 = Qt::NoModifier;
 
-	options->MouseShortcutsModified = true;
-	options->MouseShortcutsDefault = false;
+	mOptions->MouseShortcutsModified = true;
+	mOptions->MouseShortcutsDefault = false;
 
 	UpdateMouseTreeItem(ItemIndex);
 	MouseTreeItemChanged(Current);
@@ -766,11 +904,11 @@ void lcQPreferencesDialog::on_mouseReset_clicked()
 	if (QMessageBox::question(this, "LeoCAD", tr("Are you sure you want to load the default mouse shortcuts?"), QMessageBox::Yes | QMessageBox::No) != QMessageBox::Yes)
 		return;
 
-	options->MouseShortcuts.Reset();
+	mOptions->MouseShortcuts.Reset();
 	UpdateMouseTree();
 
-	options->MouseShortcutsModified = true;
-	options->MouseShortcutsDefault = true;
+	mOptions->MouseShortcutsModified = true;
+	mOptions->MouseShortcutsDefault = true;
 }
 
 void lcQPreferencesDialog::MouseTreeItemChanged(QTreeWidgetItem* Current)
@@ -785,7 +923,7 @@ void lcQPreferencesDialog::MouseTreeItemChanged(QTreeWidgetItem* Current)
 
 	int ToolIndex = ui->mouseTree->indexOfTopLevelItem(Current);
 
-	Qt::MouseButton Button = options->MouseShortcuts.mShortcuts[ToolIndex].Button1;
+	Qt::MouseButton Button = mOptions->MouseShortcuts.mShortcuts[ToolIndex].Button1;
 
 	switch (Button)
 	{
@@ -808,7 +946,7 @@ void lcQPreferencesDialog::MouseTreeItemChanged(QTreeWidgetItem* Current)
 		break;
 	}
 
-	Qt::KeyboardModifiers Modifiers = options->MouseShortcuts.mShortcuts[ToolIndex].Modifiers1;
+	Qt::KeyboardModifiers Modifiers = mOptions->MouseShortcuts.mShortcuts[ToolIndex].Modifiers1;
 	ui->mouseControl->setChecked((Modifiers & Qt::ControlModifier) != 0);
 	ui->mouseShift->setChecked((Modifiers & Qt::ShiftModifier) != 0);
 	ui->mouseAlt->setChecked((Modifiers & Qt::AltModifier) != 0);
