@@ -12,10 +12,10 @@ class lcQPreferencesDialog : public QDialog
 	Q_OBJECT
     
 public:
-	explicit lcQPreferencesDialog(QWidget *parent, void *data);
+	lcQPreferencesDialog(QWidget* Parent, lcPreferencesDialogOptions* Options);
 	~lcQPreferencesDialog();
 
-	lcPreferencesDialogOptions *options;
+	lcPreferencesDialogOptions* mOptions;
 
 	enum
 	{
@@ -27,13 +27,17 @@ public:
 public slots:
 	void accept();
 	void on_partsLibraryBrowse_clicked();
+	void on_partsArchiveBrowse_clicked();
+	void on_ColorConfigBrowseButton_clicked();
+	void on_MinifigSettingsBrowseButton_clicked();
 	void on_povrayExecutableBrowse_clicked();
 	void on_lgeoPathBrowse_clicked();
-	void colorClicked();
+	void ColorButtonClicked();
 	void on_antiAliasing_toggled();
 	void on_edgeLines_toggled();
 	void on_gridStuds_toggled();
 	void on_gridLines_toggled();
+	void on_ViewSphereSizeCombo_currentIndexChanged(int Index);
 	void updateParts();
 	void on_newCategory_clicked();
 	void on_editCategory_clicked();
@@ -47,6 +51,7 @@ public slots:
 	void on_shortcutsExport_clicked();
 	void on_shortcutsReset_clicked();
 	void commandChanged(QTreeWidgetItem *current);
+	void on_KeyboardFilterEdit_textEdited(const QString& Text);
 	void on_mouseAssign_clicked();
 	void on_mouseRemove_clicked();
 	void on_mouseReset_clicked();
