@@ -2,8 +2,8 @@
 #include "lc_qpropertiestree.h"
 #include "lc_qcolorpicker.h"
 #include "lc_application.h"
-#include "project.h"
 #include "lc_model.h"
+#include "lc_mainwindow.h"
 #include "object.h"
 #include "piece.h"
 #include "camera.h"
@@ -139,7 +139,7 @@ void lcQPropertiesTreeDelegate::paint(QPainter *painter, const QStyleOptionViewI
 	if (m_treeWidget)
 		hasValue = m_treeWidget->indexToItem(index)->data(0, lcQPropertiesTree::PropertyTypeRole).toInt() != lcQPropertiesTree::PropertyGroup;
 
-	QStyleOptionViewItemV3 opt = option;
+	QStyleOptionViewItem opt = option;
 
 	opt.state &= ~QStyle::State_HasFocus;
 
@@ -241,7 +241,7 @@ QSize lcQPropertiesTree::sizeHint() const
 
 void lcQPropertiesTree::drawRow(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
-	QStyleOptionViewItemV3 opt = option;
+	QStyleOptionViewItem opt = option;
 
 	QTreeWidgetItem *item = itemFromIndex(index);
 
@@ -480,7 +480,7 @@ QWidget *lcQPropertiesTree::createEditor(QWidget *parent, QTreeWidgetItem *item)
 			std::vector<PieceInfo*> SortedPieces;
 			SortedPieces.reserve(Library->mPieces.size());
 
-			for (const auto PartIt : Library->mPieces)
+			for (const auto& PartIt : Library->mPieces)
 				SortedPieces.push_back(PartIt.second);
 
 			auto PieceCompare = [](PieceInfo* Info1, PieceInfo* Info2)
@@ -526,7 +526,7 @@ void lcQPropertiesTree::updateColorEditor(QPushButton *editor, int value) const
 void lcQPropertiesTree::slotToggled(bool Value)
 {
 	QTreeWidgetItem* Item = m_delegate->editedItem();
-	lcModel* Model = lcGetActiveModel();
+	lcModel* Model = gMainWindow->GetActiveModel();
 
 	if (mWidgetMode == LC_PROPERTY_WIDGET_CAMERA)
 	{
@@ -548,7 +548,7 @@ void lcQPropertiesTree::slotReturnPressed()
 {
 	QLineEdit* Editor = (QLineEdit*)sender();
 	QTreeWidgetItem* Item = m_delegate->editedItem();
-	lcModel* Model = lcGetActiveModel();
+	lcModel* Model = gMainWindow->GetActiveModel();
 
 	if (mWidgetMode == LC_PROPERTY_WIDGET_PIECE)
 	{
@@ -694,7 +694,7 @@ void lcQPropertiesTree::slotReturnPressed()
 void lcQPropertiesTree::slotSetValue(int Value)
 {
 	QTreeWidgetItem* Item = m_delegate->editedItem();
-	lcModel* Model = lcGetActiveModel();
+	lcModel* Model = gMainWindow->GetActiveModel();
 
 	if (mWidgetMode == LC_PROPERTY_WIDGET_PIECE)
 	{
@@ -717,7 +717,7 @@ void lcQPropertiesTree::slotSetValue(int Value)
 void lcQPropertiesTree::slotColorButtonClicked()
 {
 	int ColorIndex = gDefaultColor;
-	lcObject* Focus = lcGetActiveModel()->GetFocusObject();
+	lcObject* Focus = gMainWindow->GetActiveModel()->GetFocusObject();
 	if (Focus && Focus->IsPiece())
 		ColorIndex = ((lcPiece*)Focus)->mColorIndex;
 
@@ -835,7 +835,7 @@ void lcQPropertiesTree::SetPiece(const lcArray<lcObject*>& Selection, lcObject*
 		mWidgetMode = LC_PROPERTY_WIDGET_PIECE;
 	}
 
-	lcModel* Model = lcGetActiveModel();
+	lcModel* Model = gMainWindow->GetActiveModel();
 	lcPiece* Piece = (Focus && Focus->IsPiece()) ? (lcPiece*)Focus : nullptr;
 	mFocus = Piece;
 
@@ -932,7 +932,9 @@ void lcQPropertiesTree::SetPiece(const lcArray<lcObject*>& Selection, lcObject*
 	partColor->setText(1, color->Name);
 	partColor->setData(0, PropertyValueRole, ColorIndex);
 
-	partID->setText(1, Info ? Info->m_strDescription : QString());
+	QString text = Info ? Info->m_strDescription : QString();
+	partID->setText(1, text);
+	partID->setToolTip(1, text);
 	partID->setData(0, PropertyValueRole, qVariantFromValue((void*)Info));
 }
 
