@@ -20,11 +20,16 @@ public slots:
 	void on_OutputBrowseButton_clicked();
 	void Update();
 
+protected slots:
+	void ReadStdErr();
+	void WriteStdLog(bool = false);
+
 protected:
 	QString GetOutputFileName() const;
 	QString GetPOVFileName() const;
 	void CloseProcess();
 	bool PromptCancel();
+	void ShowResult();
 
 #ifndef QT_NO_PROCESS
 	QProcess* mProcess;
@@ -33,6 +38,7 @@ protected:
 	QFile mOutputFile;
 	void* mOutputBuffer;
 	QImage mImage;
+	QStringList mStdErrList;
 
 	Ui::lcRenderDialog* ui;
 };
