@@ -70,7 +70,7 @@ static LONG WINAPI lcSehHandler(PEXCEPTION_POINTERS exceptionPointers)
 
 static void lcSehInit()
 {
-	if (GetTempPath(sizeof(gMinidumpPath), gMinidumpPath))
+	if (GetTempPath(sizeof(gMinidumpPath) / sizeof(gMinidumpPath[0]), gMinidumpPath))
 		lstrcat(gMinidumpPath, TEXT("leocad.dmp"));
 
 	SetUnhandledExceptionFilter(lcSehHandler);
@@ -125,9 +125,25 @@ int main(int argc, char *argv[])
 {
 	lcApplication Application(argc, argv);
 
+	QTranslator QtTranslator;
+	if (QtTranslator.load(QLocale::system(), "qt", "_", QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
+		Application.installTranslator(&QtTranslator);
+#ifdef Q_OS_WIN
+	else if (QtTranslator.load(QLocale::system(), "qt", "_", qApp->applicationDirPath() + "/translations"))
+		Application.installTranslator(&QtTranslator);
+#endif
+
+	QTranslator QtBaseTranslator;
+	if (QtBaseTranslator.load("qtbase_" + QLocale::system().name(), QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
+		Application.installTranslator(&QtBaseTranslator);
+#ifdef Q_OS_WIN
+	else if (QtBaseTranslator.load("qtbase_" + QLocale::system().name(), qApp->applicationDirPath() + "/translations"))
+		Application.installTranslator(&QtBaseTranslator);
+#endif
+
 	QTranslator Translator;
-	Translator.load(QString("leocad_") + QLocale::system().name().section('_', 0, 0) + ".qm", ":/resources");
-	Application.installTranslator(&Translator);
+	if (Translator.load("leocad_" + QLocale::system().name(), ":/resources"))
+		Application.installTranslator(&Translator);
 
 	qRegisterMetaTypeStreamOperators<QList<int> >("QList<int>");
 
