@@ -5,12 +5,11 @@
 char* strcasestr(const char *s, const char *find)
 {
 	char c, sc;
-	size_t len;
 
 	if ((c = *find++) != 0)
 	{
 		c = tolower((unsigned char)c);
-		len = strlen(find);
+		int len = (int)strlen(find);
 		do
 		{
 			do
